@@ -1,9 +1,9 @@
 //----------------------------------*-C++-*----------------------------------//
-// $Id: plstream.cc,v 1.27 2004/06/24 08:08:53 andrewross Exp $
+// $Id: plstream.cc,v 1.35 2005/12/06 16:36:02 andrewross Exp $
 // Geoffrey Furnish
 // Sep 21 1994
 //
-// Copyright (C) 2004  Andrew Ross
+// Copyright (C) 2004,2005  Andrew Ross
 // Copyright (C) 2004  Alan W. Irwin
 //
 // This file is part of PLplot.
@@ -20,7 +20,7 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //---------------------------------------------------------------------------//
 // @> Source file plstream.
@@ -223,8 +223,8 @@ void cxx_pltr2::xform( PLFLT x, PLFLT y, PLFLT& tx, PLFLT& ty ) const
     }
 }
 
-int plstream::next_stream = 0;
-int plstream::active_streams = 0;
+PLINT plstream::next_stream = 0;
+PLINT plstream::active_streams = 0;
 
 plstream::plstream()
 {
@@ -234,7 +234,7 @@ plstream::plstream()
     active_streams++;
 }
 
-plstream::plstream( PLS::stream_id sid, int strm /*=0*/ )
+plstream::plstream( PLS::stream_id sid, PLINT strm /*=0*/ )
 {
     switch(sid) {
     case PLS::Next:
@@ -255,7 +255,7 @@ plstream::plstream( PLS::stream_id sid, int strm /*=0*/ )
     }
 }
 
-plstream::plstream( int nx, int ny, const char *driver, const char *file )
+plstream::plstream( PLINT nx, PLINT ny, const char *driver, const char *file )
 {
     ::c_plsstrm(next_stream++);
 
@@ -271,7 +271,7 @@ plstream::plstream( int nx, int ny, const char *driver, const char *file )
     active_streams++;
 }
 
-plstream::plstream( int nx, int ny, int r, int g, int b, const char *driver, const char *file)
+plstream::plstream( PLINT nx, PLINT ny, PLINT r, PLINT g, PLINT b, const char *driver, const char *file)
 {
     ::c_plsstrm(next_stream++);
 
@@ -331,12 +331,21 @@ plstream::vect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale,
     plvect( u, v, nx, ny, scale, pltr, pltr_data );
 }
 
+void
+plstream::svect(PLFLT *arrow_x, PLFLT *arrow_y, PLINT npts, bool fill)
+{
+    set_stream();
+
+    plsvect(arrow_x, arrow_y, npts, (PLBOOL) fill);
+}
+
+// Depreciated version using PLINT instead of bool
 void
 plstream::svect(PLFLT *arrow_x, PLFLT *arrow_y, PLINT npts, PLINT fill)
 {
     set_stream();
 
-    plsvect(arrow_x, arrow_y, npts, fill);
+    plsvect(arrow_x, arrow_y, npts, (PLBOOL) fill);
 }
 
 // This functions similarly to plbox() except that the origin of the axes is
@@ -393,6 +402,25 @@ plstream::box3( const char *xopt, const char *xlabel, PLFLT xtick, PLINT nsubx,
 	    zopt, zlabel, ztick, nsubz );
 }
 
+// Calculate world coordinates and subpage from relative device coordinates.
+
+void plstream::calc_world(PLFLT rx, PLFLT ry, PLFLT& wx, PLFLT& wy, 
+		          PLINT& window) 
+{
+    set_stream();
+    
+    plcalc_world( rx, ry, &wx, &wy, &window ); 
+}
+
+// Clear the current subpage.
+
+void plstream::clear() 
+{
+    set_stream();
+
+    plclear();
+}
+
 // Set color, map 0.  Argument is integer between 0 and 15.
 
 void plstream::col0( PLINT icol0 )
@@ -462,11 +490,19 @@ void plstream::fcont( PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 //     plcpstrm(iplsr,flags);
 // }
 
+void plstream::cpstrm( plstream &pls, bool flags )
+{
+    set_stream();
+
+    plcpstrm(pls.stream,(PLBOOL) flags);
+}
+
+// Depreciated version using PLINT not bool
 void plstream::cpstrm( plstream &pls, PLINT flags )
 {
     set_stream();
 
-    plcpstrm(pls.stream,flags);
+    plcpstrm(pls.stream,(PLBOOL)flags);
 }
 
 // Converts input values from relative device coordinates to relative plot
@@ -576,6 +612,15 @@ void plstream::fill( PLINT n, PLFLT *x, PLFLT *y )
     plfill(n, x, y);
 }
 
+// Pattern fills the 3d polygon bounded by the input points.
+
+void plstream::fill3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z )
+{
+  //set_stream();
+
+    plfill3(n, x, y, z);
+}
+
 // Flushes the output stream.  Use sparingly, if at all.
 
 void plstream::flush()
@@ -630,6 +675,15 @@ void plstream::gcolbg( PLINT& r, PLINT& g, PLINT& b )
     plgcolbg( &r, &g, &b );
 }
 
+// Returns the current compression setting
+
+void plstream::gcompression(PLINT& compression) 
+{
+    set_stream();
+
+    plgcompression(&compression);
+}
+	    
 // Retrieve current window into device space.
 
 void plstream::gdidev( PLFLT& mar, PLFLT& aspect, PLFLT& jx,
@@ -658,6 +712,15 @@ void plstream::gdiplt( PLFLT& xmin, PLFLT& ymin, PLFLT& xmax, PLFLT& ymax )
     plgdiplt( &xmin, &ymin, &xmax, &ymax );
 }
 
+// Get FCI (font characterization integer)
+
+void plstream::gfci( PLUNICODE& pfci )
+{
+    set_stream();
+
+    plgfci(&pfci);
+}
+
 // Get family file parameters.
 
 void plstream::gfam( PLINT& fam, PLINT& num, PLINT& bmax )
@@ -743,6 +806,15 @@ void plstream::gver( char *p_ver )
     plgver(p_ver);
 }
 
+// Get viewport window in normalized world coordinates
+
+void plstream::gvpd(PLFLT& xmin, PLFLT& xmax, PLFLT& ymin, PLFLT& ymax)
+{
+    set_stream();
+
+    plgvpd(&xmin, &xmax, &ymin, &ymax);
+}
+
 // Get viewport window in world coordinates
 
 void plstream::gvpw(PLFLT& xmin, PLFLT& xmax, PLFLT& ymin, PLFLT& ymax)
@@ -937,7 +1009,21 @@ void plstream::surf3d( PLFLT *x, PLFLT *y, PLFLT **z,
 {
     set_stream();
 
-    ::plsurf3d(x,y,z,nx,ny,opt,clevel,nlevel);
+    plsurf3d(x,y,z,nx,ny,opt,clevel,nlevel);
+}
+
+// Plots a 3-d shaded representation of the function z[x][y] with 
+// y index limits
+
+void plstream::surf3dl( PLFLT *x, PLFLT *y, PLFLT **z,
+		        PLINT nx, PLINT ny, PLINT opt,
+		        PLFLT *clevel, PLINT nlevel,
+			PLINT ixstart, PLINT ixn, 
+			PLINT *indexymin, PLINT*indexymax)
+{
+    set_stream();
+
+    plsurf3dl(x,y,z,nx,ny,opt,clevel,nlevel,ixstart, ixn, indexymin, indexymax);
 }
 
 /* Plots a 3-d representation of the function z[x][y]. */
@@ -961,6 +1047,29 @@ void plstream::plot3dc( PLFLT *x, PLFLT *y, PLFLT **z,
     ::plot3dc(x,y,z,nx,ny,opt,clevel,nlevel);
 }
 
+// Plots a 3-d representation of the function z[x][y] with contour
+// and y index limits
+
+void plstream::plot3dcl( PLFLT *x, PLFLT *y, PLFLT **z,
+		        PLINT nx, PLINT ny, PLINT opt,
+		        PLFLT *clevel, PLINT nlevel,
+			PLINT ixstart, PLINT ixn, 
+			PLINT *indexymin, PLINT*indexymax)
+{
+    set_stream();
+
+    ::plot3dcl(x,y,z,nx,ny,opt,clevel,nlevel,ixstart, ixn, indexymin, indexymax);
+}
+
+/* Process options list using current options info. */
+
+int plstream::parseopts( int *p_argc, char **argv, PLINT mode )
+{
+    set_stream();
+
+    return ::plparseopts( p_argc, argv, mode );
+}
+
 /* Set fill pattern directly. */
 
 void plstream::pat( PLINT nlin, PLINT *inc, PLINT *del )
@@ -990,11 +1099,19 @@ void plstream::poin3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT code )
 
 /* Draws a polygon in 3 space.  */
 
+void plstream::poly3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, bool ifcc )
+{
+    set_stream();
+
+    plpoly3(n,x,y,z,draw,(PLBOOL) ifcc);
+}
+
+// Depreciated version using PLINT not bool
 void plstream::poly3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, PLINT ifcc )
 {
     set_stream();
 
-    plpoly3(n,x,y,z,draw,ifcc);
+    plpoly3(n,x,y,z,draw,(PLBOOL) ifcc);
 }
 
 /* Set the floating point precision (in number of places) in numeric labels. */
@@ -1100,13 +1217,37 @@ void plstream::scmap1( PLINT *r, PLINT *g, PLINT *b, PLINT ncol1 )
 /* Set color map 1 colors using a piece-wise linear relationship between */
 /* intensity [0,1] (cmap 1 index) and position in HLS or RGB color space. */
 
+void plstream::scmap1l( bool itype, PLINT npts, PLFLT *intensity,
+			PLFLT *coord1, PLFLT *coord2, PLFLT *coord3,
+			bool *rev )
+{
+    PLBOOL * loc_rev = new PLBOOL[npts];
+    for (int i=0;i<npts;i++) {
+        loc_rev[i] = (PLBOOL) rev[i];
+    }
+    
+    set_stream();
+
+    plscmap1l((PLBOOL) itype,npts,intensity,coord1,coord2,coord3,loc_rev);
+
+    delete loc_rev;
+}
+
+// Depreciated version using PLINT instead of bool
 void plstream::scmap1l( PLINT itype, PLINT npts, PLFLT *intensity,
 			PLFLT *coord1, PLFLT *coord2, PLFLT *coord3,
 			PLINT *rev )
 {
+    PLBOOL * loc_rev = new PLBOOL[npts];
+    for (int i=0;i<npts;i++) {
+        loc_rev[i] = (PLBOOL) rev[i];
+    }
+
     set_stream();
 
-    plscmap1l(itype,npts,intensity,coord1,coord2,coord3,rev);
+    plscmap1l((PLBOOL) itype,npts,intensity,coord1,coord2,coord3,loc_rev);
+
+    delete loc_rev;
 }
 
 /* Set a given color from color map 0 by 8 bit RGB value */
@@ -1136,6 +1277,15 @@ void plstream::scolor( PLINT color )
     plscolor(color);
 }
 
+// Sets the compression level
+
+void plstream::scompression(PLINT compression) 
+{
+    set_stream();
+
+    plscompression(compression);
+}
+	    
 /* Set the device (keyword) name */
 
 void plstream::sdev( const char *devname )
@@ -1236,6 +1386,15 @@ void plstream::sfam( PLINT fam, PLINT num, PLINT bmax )
     plsfam(fam,num,bmax);
 }
 
+// Set FCI (font characterization integer)
+
+void plstream::sfci( PLUNICODE fci )
+{
+    set_stream();
+
+    plsfci(fci);
+}
+
 /* Set the output file name. */
 
 void plstream::sfnam( const char *fnam )
@@ -1247,6 +1406,28 @@ void plstream::sfnam( const char *fnam )
 
 /* Shade region. */
 
+void
+plstream::shade( PLFLT **a, PLINT nx, PLINT ny,
+		 PLINT (*defined) (PLFLT, PLFLT),
+		 PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		 PLFLT shade_min, PLFLT shade_max,
+		 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		 PLINT min_color, PLINT min_width,
+		 PLINT max_color, PLINT max_width,
+		 void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
+		 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		 PLPointer pltr_data )
+{
+    set_stream();
+
+    plshade( a,nx,ny,defined,left,right,bottom,top,
+	     shade_min, shade_max,
+	     sh_cmap, sh_color, sh_width,
+	     min_color, min_width, max_color, max_width,
+	     fill, (PLBOOL) rectangular, pltr, pltr_data );
+}
+
+// Depreciated version using PLINT instead of bool
 void
 plstream::shade( PLFLT **a, PLINT nx, PLINT ny,
 		 PLINT (*defined) (PLFLT, PLFLT),
@@ -1265,8 +1446,27 @@ plstream::shade( PLFLT **a, PLINT nx, PLINT ny,
 	     shade_min, shade_max,
 	     sh_cmap, sh_color, sh_width,
 	     min_color, min_width, max_color, max_width,
-	     fill, rectangular, pltr, pltr_data );
+	     fill, (PLBOOL) rectangular, pltr, pltr_data );
 }
+
+void
+plstream::shades( PLFLT **a, PLINT nx, PLINT ny,
+		 PLINT (*defined) (PLFLT, PLFLT),
+		 PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax,
+		 PLFLT *clevel, PLINT nlevel, PLINT fill_width,
+		 PLINT cont_color, PLINT cont_width,
+		 void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
+		 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		 PLPointer pltr_data )
+{
+    set_stream();
+
+    plshades( a,nx,ny,defined,xmin,xmax,ymin,ymax,
+    	     clevel, nlevel, fill_width, cont_color, cont_width,
+	     fill, (PLBOOL) rectangular, pltr, pltr_data );
+}
+
+// Depreciated version using PLINT instead of bool
 void
 plstream::shades( PLFLT **a, PLINT nx, PLINT ny,
 		 PLINT (*defined) (PLFLT, PLFLT),
@@ -1281,9 +1481,34 @@ plstream::shades( PLFLT **a, PLINT nx, PLINT ny,
 
     plshades( a,nx,ny,defined,xmin,xmax,ymin,ymax,
     	     clevel, nlevel, fill_width, cont_color, cont_width,
-	     fill, rectangular, pltr, pltr_data );
+	     fill, (PLBOOL) rectangular, pltr, pltr_data );
 }
 
+void
+plstream::shade( Contourable_Data& d, PLFLT xmin, PLFLT xmax,
+		 PLFLT ymin, PLFLT ymax, PLFLT shade_min, PLFLT shade_max,
+		 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		 PLINT min_color, PLINT min_width,
+		 PLINT max_color, PLINT max_width,
+		 bool rectangular,
+		 Coord_Xformer *pcxf )
+{
+    set_stream();
+
+    int nx, ny;
+    d.elements( nx, ny );
+
+    ::plfshade( Contourable_Data_evaluator, &d,
+		NULL, NULL,
+		nx, ny,
+		xmin, xmax, ymin, ymax, shade_min, shade_max,
+		sh_cmap, sh_color, sh_width,
+		min_color, min_width, max_color, max_width,
+		::plfill, rectangular,
+		Coord_Xform_evaluator, pcxf );
+}
+
+// Depreciated version using PLINT not bool
 void
 plstream::shade( Contourable_Data& d, PLFLT xmin, PLFLT xmax,
 		 PLFLT ymin, PLFLT ymax, PLFLT shade_min, PLFLT shade_max,
@@ -1304,10 +1529,33 @@ plstream::shade( Contourable_Data& d, PLFLT xmin, PLFLT xmax,
 		xmin, xmax, ymin, ymax, shade_min, shade_max,
 		sh_cmap, sh_color, sh_width,
 		min_color, min_width, max_color, max_width,
-		::plfill, rectangular,
+		::plfill, (bool) rectangular,
 		Coord_Xform_evaluator, pcxf );
 }
 
+void
+plstream::shade1( PLFLT *a, PLINT nx, PLINT ny,
+		  PLINT (*defined) (PLFLT, PLFLT),
+		  PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		  PLFLT shade_min, PLFLT shade_max,
+		  PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		  PLINT min_color, PLINT min_width,
+		  PLINT max_color, PLINT max_width,
+		  void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
+		  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		  PLPointer pltr_data )
+{
+    set_stream();
+
+    plshade1( a, nx, ny, defined,
+	      left, right, bottom, top,
+	      shade_min, shade_max,
+	      sh_cmap, sh_color, sh_width,
+	      min_color, min_width, max_color, max_width,
+	      fill, (PLBOOL) rectangular, pltr, pltr_data );
+}
+
+// Depreciated version using PLINT not bool
 void
 plstream::shade1( PLFLT *a, PLINT nx, PLINT ny,
 		  PLINT (*defined) (PLFLT, PLFLT),
@@ -1327,9 +1575,36 @@ plstream::shade1( PLFLT *a, PLINT nx, PLINT ny,
 	      shade_min, shade_max,
 	      sh_cmap, sh_color, sh_width,
 	      min_color, min_width, max_color, max_width,
-	      fill, rectangular, pltr, pltr_data );
+	      fill, (PLBOOL) rectangular, pltr, pltr_data );
 }
 
+void
+plstream::fshade( PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
+		  PLPointer f2eval_data,
+		  PLFLT (*c2eval) (PLINT, PLINT, PLPointer),
+		  PLPointer c2eval_data,
+		  PLINT nx, PLINT ny,
+		  PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		  PLFLT shade_min, PLFLT shade_max,
+		  PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		  PLINT min_color, PLINT min_width,
+		  PLINT max_color, PLINT max_width,
+		  void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
+		  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		  PLPointer pltr_data )
+{
+    set_stream();
+
+    plfshade( f2eval, f2eval_data,
+	      c2eval, c2eval_data,
+	      nx, ny, left, right, bottom, top,
+	      shade_min, shade_max,
+	      sh_cmap, sh_color, sh_width,
+	      min_color, min_width, max_color, max_width,
+	      fill, (PLBOOL) rectangular, pltr, pltr_data );
+}
+
+// Depreciated version using PLINT not bool
 void
 plstream::fshade( PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 		  PLPointer f2eval_data,
@@ -1353,7 +1628,7 @@ plstream::fshade( PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 	      shade_min, shade_max,
 	      sh_cmap, sh_color, sh_width,
 	      min_color, min_width, max_color, max_width,
-	      fill, rectangular, pltr, pltr_data );
+	      fill, (PLBOOL) rectangular, pltr, pltr_data );
 }
 
 /* Set up lengths of major tick marks. */
@@ -1395,11 +1670,19 @@ void plstream::spage( PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
 
 /* Set the pause (on end-of-page) status */
 
+void plstream::spause( bool pause )
+{
+    set_stream();
+
+    plspause((PLBOOL) pause);
+}
+
+// Depreciated version using PLINT not bool
 void plstream::spause( PLINT pause )
 {
     set_stream();
 
-    plspause(pause);
+    plspause((PLBOOL) pause);
 }
 
 /* Set stream number.  */
@@ -1449,6 +1732,23 @@ void plstream::start( const char *devname, PLINT nx, PLINT ny )
 
 /* Create 1d stripchart */
 
+void plstream::stripc(PLINT *id, char *xspec, char *yspec,
+        PLFLT xmin, PLFLT xmax, PLFLT xjump, PLFLT ymin, PLFLT ymax,
+        PLFLT xlpos, PLFLT ylpos,
+        bool y_ascl, bool acc,
+        PLINT colbox, PLINT collab,
+        PLINT colline[], PLINT styline[], char *legline[],
+        char *labx, char *laby, char *labtop)
+{
+    set_stream();
+
+    plstripc(id, xspec, yspec, xmin, xmax, xjump, ymin, ymax, xlpos, ylpos,
+        (PLBOOL) y_ascl, (PLBOOL) acc, colbox, collab, colline, styline, 
+	legline, labx, laby, labtop);
+}
+
+
+// Depreciated version using PLINT not bool
 void plstream::stripc(PLINT *id, char *xspec, char *yspec,
         PLFLT xmin, PLFLT xmax, PLFLT xjump, PLFLT ymin, PLFLT ymax,
         PLFLT xlpos, PLFLT ylpos,
@@ -1460,8 +1760,8 @@ void plstream::stripc(PLINT *id, char *xspec, char *yspec,
     set_stream();
 
     plstripc(id, xspec, yspec, xmin, xmax, xjump, ymin, ymax, xlpos, ylpos,
-        y_ascl, acc, colbox, collab, colline, styline, legline,
-        labx, laby, labtop);
+        (PLBOOL) y_ascl, (PLBOOL) acc, colbox, collab, colline, styline, 
+        legline, labx, laby, labtop);
 }
 
 /* Add a point to a stripchart.  */
@@ -1639,14 +1939,30 @@ void plstream::wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax )
 /*  set xor mode; mode = 1-enter, 0-leave, status = 0 if not interactive device
  */
 
-void plstream::xormod(PLINT mode, PLINT *status)
+void plstream::xormod(bool mode, bool *status)
 {
+   PLBOOL loc_status;
+   
    set_stream();
 
-   plxormod(mode, status);
+   plxormod((PLBOOL) mode, &loc_status);
+
+   *status = (bool) loc_status;
 }
 
-	/* The rest for use from C only */
+// Depreciated version using PLINT not bool
+void plstream::xormod(PLINT mode, PLINT *status)
+{
+   PLBOOL loc_status;
+
+   set_stream();
+
+   plxormod((PLBOOL) mode, &loc_status);
+
+   *status = (PLINT) loc_status;
+}
+
+	/* The rest for use from C / C++ only */
 
 /* Returns a list of file-oriented device names and their menu strings */
 
@@ -1962,20 +2278,20 @@ void plstream::MinMax2dGrid(PLFLT **f, PLINT nx, PLINT ny, PLFLT *fmax, PLFLT *f
 
 /* Functions for converting between HLS and RGB color space */
 
-void plstream::HLS_RGB( PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g,
+void plstream::hlsrgb( PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g,
 			PLFLT *p_b )
 {
     set_stream();
 
-    plHLS_RGB(h,l,s,p_r,p_g,p_b);
+    ::c_plhlsrgb(h,l,s,p_r,p_g,p_b);
 }
 
-void plstream::RGB_HLS( PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l,
+void plstream::rgbhls( PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l,
 			PLFLT *p_s )
 {
     set_stream();
 
-    plRGB_HLS(r,g,b,p_h,p_l,p_s);
+    ::c_plrgbhls(r,g,b,p_h,p_l,p_s);
 }
 
 /* Wait for right button mouse event and translate to world coordinates */
