@@ -1,9 +1,9 @@
 //----------------------------------*-C++-*----------------------------------//
-// $Id: plstream.h,v 1.20 2004/06/24 08:08:53 andrewross Exp $
+// $Id: plstream.h,v 1.28 2005/12/06 16:36:02 andrewross Exp $
 // Geoffrey Furnish
 // Sep 21 1994
 //
-// Copyright (C) 2004  Andrew Ross
+// Copyright (C) 2004,2005  Andrew Ross
 // Copyright (C) 2004  Alan W. Irwin
 //
 // This file is part of PLplot.
@@ -20,7 +20,7 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 //---------------------------------------------------------------------------//
 // @> Header file plstream.
@@ -84,10 +84,10 @@ class cxx_pltr2 : public Coord_Xformer {
 
 class plstream {
 
-    int stream;
+    PLINT stream;
 
-    static int next_stream;
-    static int active_streams;
+    static PLINT next_stream;
+    static PLINT active_streams;
 
   private:
     // These have to be disabled till we implement reference counting.
@@ -95,16 +95,17 @@ class plstream {
     plstream( const plstream& );
     plstream& operator=( const plstream& );
 
-    void set_stream() { ::c_plsstrm(stream); }
+  protected:
+    virtual void set_stream() { ::c_plsstrm(stream); }
 
   public:
     plstream();
     plstream( plstream * pls );
-    plstream( PLS::stream_id sid, int strm =0 );
-    plstream( int _stream ) : stream(_stream) {}
-    plstream( int nx /*=1*/, int ny /*=1*/,
+    plstream( PLS::stream_id sid, PLINT strm =0 );
+    plstream( PLINT _stream ) : stream(_stream) {}
+    plstream( PLINT nx /*=1*/, PLINT ny /*=1*/,
 	      const char *driver =NULL, const char *file =NULL );
-    plstream( int nx /*=1*/, int ny /*=1*/, int r, int g, int b,
+    plstream( PLINT nx /*=1*/, PLINT ny /*=1*/, PLINT r, PLINT g, PLINT b,
 	      const char *driver =NULL, const char *file =NULL );
 
     ~plstream();
@@ -126,7 +127,7 @@ class plstream {
 		   PLPointer pltr_data);
 
 // Set the arrow style
-    void svect( PLFLT *arrox_x, PLFLT *arrow_y, PLINT npts, PLINT fill);
+    void svect( PLFLT *arrow_x, PLFLT *arrow_y, PLINT npts, bool fill);
 
 // This functions similarly to plbox() except that the origin of the axes is
 // placed at the user-specified point (x0, y0).
@@ -153,6 +154,14 @@ class plstream {
 	       const char *yopt, const char *ylabel, PLFLT ytick, PLINT nsuby,
 	       const char *zopt, const char *zlabel, PLFLT ztick, PLINT nsubz );
 
+// Calculate world coordinates and subpage from relative device coordinates.
+    
+    void calc_world(PLFLT rx, PLFLT ry, PLFLT& wx, PLFLT& wy, PLINT& window);
+    
+// Clear the current subpage.
+    
+    void clear();
+
 // Set color, map 0.  Argument is integer between 0 and 15.
 
     void col0( PLINT icol0 );
@@ -189,7 +198,7 @@ class plstream {
 // /* Copies state parameters from the reference stream to the current stream. */
 
 //     void cpstrm( PLINT iplsr, PLINT flags );
-     void cpstrm( plstream &pls, PLINT flags );
+     void cpstrm( plstream &pls, bool flags );
 
 // Converts input values from relative device coordinates to relative plot
 // coordinates.
@@ -243,6 +252,10 @@ class plstream {
 
     static void fill( PLINT n, PLFLT *x, PLFLT *y );
 
+// Pattern fills the 3d polygon bounded by the input points.
+
+    static void fill3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z );
+
 // Flushes the output stream.  Use sparingly, if at all.
 
     void flush();
@@ -267,6 +280,10 @@ class plstream {
 
     void gcolbg( PLINT& r, PLINT& g, PLINT& b );
 
+// Returns the current compression setting
+
+    void gcompression(PLINT& compression);
+    
 // Retrieve current window into device space.
 
     void gdidev( PLFLT& mar, PLFLT& aspect, PLFLT& jx, PLFLT& jy );
@@ -279,6 +296,10 @@ class plstream {
 
     void gdiplt( PLFLT& xmin, PLFLT& ymin, PLFLT& xmax, PLFLT& ymax );
 
+// Get FCI (font characterization integer)
+
+    void gfci( PLUNICODE& pfci);
+
 // Get family file parameters.
 
     void gfam( PLINT& fam, PLINT& num, PLINT& bmax );
@@ -302,9 +323,9 @@ class plstream {
 
 // grid irregularly sampled data
 
-    void griddata(PLFLT *x, PLFLT *y, PLFLT *z, int npts,
-                  PLFLT *xg, int nptsx, PLFLT *yg,  int nptsy,
-                  PLFLT **zg, int type, PLFLT data);
+    void griddata(PLFLT *x, PLFLT *y, PLFLT *z, PLINT npts,
+                  PLFLT *xg, PLINT nptsx, PLFLT *yg,  PLINT nptsy,
+                  PLFLT **zg, PLINT type, PLFLT data);
 
 // Get subpage boundaries in absolute coordinates.
 
@@ -320,6 +341,10 @@ class plstream {
 
     void gver( char *p_ver );
 
+// Get the viewport window in normalized device coordinates
+
+    void gvpd( PLFLT& xmin, PLFLT& xmax, PLFLT& ymin, PLFLT& ymax );
+
 // Get the viewport window in world coordinates
 
     void gvpw( PLFLT& xmin, PLFLT& xmax, PLFLT& ymin, PLFLT& ymax );
@@ -414,12 +439,34 @@ class plstream {
                  PLINT nx, PLINT ny, PLINT opt,
                  PLFLT *clevel, PLINT nlevel);
 
+// Plots a 3-d representation of the function z[x][y] with contour
+// and y index limits.
+
+    void plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
+                 PLINT nx, PLINT ny, PLINT opt,
+                 PLFLT *clevel, PLINT nlevel,
+                 PLINT ixstart, PLINT ixn, PLINT *indexymin, PLINT*indexymax);
+
+
 /* Plots a 3-d shaded representation of the function z[x][y]. */
 
     void surf3d( PLFLT *x, PLFLT *y, PLFLT **z,
 		 PLINT nx, PLINT ny, PLINT opt,
 		 PLFLT *clevel, PLINT nlevel);
 
+// Plots a 3-d shaded representation of the function z[x][y] with y 
+// index limits
+
+    void surf3dl( PLFLT *x, PLFLT *y, PLFLT **z,
+		 PLINT nx, PLINT ny, PLINT opt,
+		 PLFLT *clevel, PLINT nlevel,
+                 PLINT ixstart, PLINT ixn, PLINT *indexymin, PLINT*indexymax);
+
+    
+/* Process options list using current options info. */
+
+    int parseopts( int *p_argc, char **argv, PLINT mode );
+
 /* Set fill pattern directly. */
 
 void pat( PLINT nlin, PLINT *inc, PLINT *del );
@@ -434,7 +481,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 
 /* Draws a polygon in 3 space.  */
 
-    void poly3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, PLINT ifcc );
+    void poly3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, bool ifcc );
 
 /* Set the floating point precision (in number of places) in numeric labels. */
 
@@ -484,8 +531,8 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 /* Set color map 1 colors using a piece-wise linear relationship between */
 /* intensity [0,1] (cmap 1 index) and position in HLS or RGB color space. */
 
-    void scmap1l( PLINT itype, PLINT npts, PLFLT *intensity,
-		  PLFLT *coord1, PLFLT *coord2, PLFLT *coord3, PLINT *rev );
+    void scmap1l( bool itype, PLINT npts, PLFLT *intensity,
+		  PLFLT *coord1, PLFLT *coord2, PLFLT *coord3, bool *rev );
 
 /* Set a given color from color map 0 by 8 bit RGB value */
 
@@ -499,6 +546,10 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 
     void scolor( PLINT color );
 
+// Set the compression level
+
+   void scompression(PLINT compression);
+
 /* Set the device (keyword) name */
 
     void sdev( const char *devname );
@@ -546,6 +597,10 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 
     void sfam( PLINT fam, PLINT num, PLINT bmax );
 
+/* Set FCI (font characterization integer) */
+
+    void sfci( PLUNICODE fci);
+
 /* Set the output file name. */
 
     void sfnam( const char *fnam );
@@ -559,7 +614,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 		PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 		PLINT min_color, PLINT min_width,
 		PLINT max_color, PLINT max_width,
-		void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
 		void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 		PLPointer pltr_data );
 
@@ -567,7 +622,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
               	PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax,
 		PLFLT *clevel, PLINT nlevel, PLINT fill_width,
                 PLINT cont_color, PLINT cont_width,
-                void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+                void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
                 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
                 PLPointer pltr_data);
 
@@ -580,7 +635,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 		PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 		PLINT min_color, PLINT min_width,
 		PLINT max_color, PLINT max_width,
-		PLINT rectangular,
+		bool rectangular,
 		Coord_Xformer *pcxf );
 
     void  shade1( PLFLT *a, PLINT nx, PLINT ny,
@@ -590,7 +645,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 		  PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 		  PLINT min_color, PLINT min_width,
 		  PLINT max_color, PLINT max_width,
-		  void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		  void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
 		  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 		  PLPointer pltr_data);
 
@@ -604,7 +659,7 @@ void pat( PLINT nlin, PLINT *inc, PLINT *del );
 		 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 		 PLINT min_color, PLINT min_width,
 		 PLINT max_color, PLINT max_width,
-		 void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		 void (*fill) (PLINT, PLFLT *, PLFLT *), bool rectangular,
 		 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 		 PLPointer pltr_data );
 
@@ -627,7 +682,7 @@ void spage( PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
 
 /* Set the pause (on end-of-page) status */
 
-    void spause( PLINT pause );
+    void spause( bool pause );
 
 /* Set stream number.  */
 
@@ -654,7 +709,7 @@ void spage( PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
     void stripc(PLINT *id, char *xspec, char *yspec,
         PLFLT xmin, PLFLT xmax, PLFLT xjump, PLFLT ymin, PLFLT ymax,
         PLFLT xlpos, PLFLT ylpos,
-        PLINT y_ascl, PLINT acc,
+        bool y_ascl, bool acc,
         PLINT colbox, PLINT collab,
         PLINT colline[], PLINT styline[], char *legline[],
         char *labx, char *laby, char *labtop);
@@ -739,7 +794,7 @@ void spage( PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
     void wind( PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax );
 
 /* set xor mode; mode = 1-enter, 0-leave, status = 0 if not interactive device */
-    void xormod(PLINT mode, PLINT *status);
+    void xormod(bool mode, bool *status);
 
 	/* The rest for use from C only */
 
@@ -904,17 +959,90 @@ void spage( PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
 
 /* Functions for converting between HLS and RGB color space */
 
-    void HLS_RGB( PLFLT h, PLFLT l, PLFLT s,
+    void hlsrgb ( PLFLT h, PLFLT l, PLFLT s,
 		  PLFLT *p_r, PLFLT *p_g, PLFLT *p_b );
 
-    void RGB_HLS( PLFLT r, PLFLT g, PLFLT b,
+    void rgbhls ( PLFLT r, PLFLT g, PLFLT b,
 		  PLFLT *p_h, PLFLT *p_l, PLFLT *p_s );
 
 /* Wait for graphics input event and translate to world coordinates */
 
     int GetCursor( PLGraphicsIn *plg );
+
+// Depreciated versions of methods which use PLINT instead of bool
+    void svect( PLFLT *arrow_x, PLFLT *arrow_y, PLINT npts, PLINT fill);
+    void cpstrm( plstream &pls, PLINT flags );
+    void poly3( PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, PLINT ifcc );
+    void scmap1l( PLINT itype, PLINT npts, PLFLT *intensity,
+		  PLFLT *coord1, PLFLT *coord2, PLFLT *coord3, PLINT *rev );
+
+    void shade( PLFLT **a, PLINT nx, PLINT ny,
+		PLINT (*defined) (PLFLT, PLFLT),
+		PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		PLFLT shade_min, PLFLT shade_max,
+		PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		PLINT min_color, PLINT min_width,
+		PLINT max_color, PLINT max_width,
+		void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		PLPointer pltr_data );
+
+    void shades( PLFLT **a, PLINT nx, PLINT ny, PLINT (*defined) (PLFLT, PLFLT),
+              	PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax,
+		PLFLT *clevel, PLINT nlevel, PLINT fill_width,
+                PLINT cont_color, PLINT cont_width,
+                void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+                void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+                PLPointer pltr_data);
+
+    void shade( Contourable_Data& d, PLFLT xmin, PLFLT xmax,
+		PLFLT ymin, PLFLT ymax, PLFLT shade_min, PLFLT shade_max,
+		PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		PLINT min_color, PLINT min_width,
+		PLINT max_color, PLINT max_width,
+		PLINT rectangular,
+		Coord_Xformer *pcxf );
+
+    void  shade1( PLFLT *a, PLINT nx, PLINT ny,
+		  PLINT (*defined) (PLFLT, PLFLT),
+		  PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		  PLFLT shade_min, PLFLT shade_max,
+		  PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		  PLINT min_color, PLINT min_width,
+		  PLINT max_color, PLINT max_width,
+		  void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		  PLPointer pltr_data);
+
+    void fshade( PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
+		 PLPointer f2eval_data,
+		 PLFLT (*c2eval) (PLINT, PLINT, PLPointer),
+		 PLPointer c2eval_data,
+		 PLINT nx, PLINT ny,
+		 PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
+		 PLFLT shade_min, PLFLT shade_max,
+		 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
+		 PLINT min_color, PLINT min_width,
+		 PLINT max_color, PLINT max_width,
+		 void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+		 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+		 PLPointer pltr_data );
+
+    void spause( PLINT pause );
+
+    void stripc(PLINT *id, char *xspec, char *yspec,
+        PLFLT xmin, PLFLT xmax, PLFLT xjump, PLFLT ymin, PLFLT ymax,
+        PLFLT xlpos, PLFLT ylpos,
+        PLINT y_ascl, PLINT acc,
+        PLINT colbox, PLINT collab,
+        PLINT colline[], PLINT styline[], char *legline[],
+        char *labx, char *laby, char *labtop);
+
+    void xormod(PLINT mode, PLINT *status);
 };
 
+
+
 #endif                          // __plstream_h__
 
 //---------------------------------------------------------------------------//
