@@ -1,4 +1,4 @@
-/* $Id: scstubs.c,v 1.36 2004/02/24 10:06:04 rlaboiss Exp $
+/* $Id: scstubs.c,v 1.41 2005/04/27 06:43:10 rlaboiss Exp $
 
 	C stub routines.
 
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 
 	The stubs contained here are the ones that are relatively simple,
@@ -362,6 +362,12 @@ PLHLS(PLFLT *hue, PLFLT *light, PLFLT *sat)
     c_plhls(*hue, *light, *sat);
 }
 
+void
+PLHLSRGB(PLFLT *h, PLFLT *l, PLFLT *s, PLFLT *r, PLFLT *g, PLFLT *b)
+{
+    c_plhlsrgb(*h, *l, *s, r, g, b);
+}
+
 void
 PLINIT(void)
 {
@@ -417,18 +423,18 @@ PLMTEX7(char *side, PLFLT *disp, PLFLT *pos, PLFLT *just, char *text)
 }
 
 void
-PLPARSEOPTS7(PLINT *numargs, char *iargs, PLINT *mode, PLINT *maxindex)
+PLPARSEOPTS7(int *numargs, char *iargs, PLINT *mode, PLINT *maxindex)
 {
 /* Same as in plparseopts fortran subroutine that calls this one. */
 #define MAXARGS 20
    if(*numargs <= MAXARGS) {
       char *argv[MAXARGS];
-      PLINT i;
+      int i;
       for(i = 0; i < *numargs; i++) {
 	 argv[i] = iargs + (i* *maxindex);
 /*	 fprintf(stderr, "%s\n", argv[i]); */
       }
-      plParseOpts(numargs, argv, *mode);
+      c_plparseopts(numargs, argv, *mode);
    }  else
      fprintf(stderr,"plparseopts7: numargs too large\n");
 }
@@ -493,6 +499,12 @@ PLRGB1(PLINT *r, PLINT *g, PLINT *b)
     c_plrgb1(*r, *g, *b);
 }
 
+void
+PLRGBHLS(PLFLT *r, PLFLT *g, PLFLT *b, PLFLT *h, PLFLT *l, PLFLT *s)
+{
+    c_plrgbhls(*r, *g, *b, h, l, s);
+}
+
 void
 PLSCHR(PLFLT *def, PLFLT *scale)
 {
@@ -819,4 +831,3 @@ PLXORMOD(PLINT *mode, PLINT *status)
 {
     c_plxormod(*mode, status);
 }
-
