@@ -1,18 +1,47 @@
-/* $Id: ps.c,v 1.63 2004/01/28 09:57:10 rlaboiss Exp $
+/* $Id: ps.c,v 1.97 2005/05/11 16:29:14 andrewross Exp $
 
 	PLplot PostScript device driver.
+
+  Copyright (C) 1992, 2001  Geoffrey Furnish  
+  Copyright (C) 1992, 1993, 1994, 1995, 2001  Maurice LeBrun
+  Copyright (C) 2000, 2001, 2002, 2004, 2005  Alan W. Irwin 
+  Copyright (C) 2001, 2002  Joao Cardoso  
+  Copyright (C) 2001, 2003, 2004  Rafael Laboissiere
+  Copyright (C) 2004, 2005  Thomas J. Duck
+  Copyright (C) 2005  Andrew Ross
+
+  This file is part of PLplot.
+
+  PLplot is free software; you can redistribute it and/or modify
+  it under the terms of the GNU General Library Public License as published
+  by the Free Software Foundation; either version 2 of the License, or
+  (at your option) any later version.
+
+  PLplot is distributed in the hope that it will be useful,
+  but WITHOUT ANY WARRANTY; without even the implied warranty of
+  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+  GNU Library General Public License for more details.
+
+  You should have received a copy of the GNU Library General Public License
+  along with PLplot; if not, write to the Free Software
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
 */
+
 #include "plDevs.h"
 
-#ifdef PLD_ps
+#define DEBUG
 
+#ifdef PLD_ps
+#define NEED_PLDEBUG
 #include "plplotP.h"
 #include "drivers.h"
 #include "ps.h"
 
 #include <string.h>
 #include <time.h>
-
+#include "plunicode-type1.h"
+#include "plfci-type1.h"
 
 /* Device info */
 
@@ -30,17 +59,24 @@ static char  *ps_getdate	(void);
 static void  ps_init		(PLStream *);
 static void  fill_polygon	(PLStream *pls);
 static void  proc_str           (PLStream *, EscText *);
-static void  esc_purge          (char *, const char *);
+static void  esc_purge          (char *, unsigned char *);
 
 static char  outbuf[128];
-static int text = 0;
+static int text = 1;
 static int color;
+static int hrshsym = 1;
 
-static DrvOpt ps_options[] = {{"text", DRV_INT, &text, "Use Postscript text (text=0|1|2)"},
+static DrvOpt ps_options[] = {{"text", DRV_INT, &text, "Use Postscript text (text=0|1)"},
 			      {"color", DRV_INT, &color, "Use color (color=0|1)"},
+			      {"hrshsym", DRV_INT, &hrshsym, "Use Hershey symbol set (hrshsym=0|1)"},
 			      {NULL, DRV_INT, NULL, NULL}};
 
-/* text > 1 uses some postscript tricks, namely a transformation matrix
+static unsigned char 
+plunicode2type1 (const PLUNICODE index,
+		 const Unicode_to_Type1_table lookup[], 
+		 const int number_of_entries);
+
+/* text > 0 uses some postscript tricks, namely a transformation matrix
    that scales, rotates (with slanting) and offsets text strings.
    It has yet some bugs for 3d plots. */
 
@@ -119,8 +155,12 @@ ps_init(PLStream *pls)
     PLFLT pxlx = YPSSIZE/LPAGE_X;
     PLFLT pxly = XPSSIZE/LPAGE_Y;
 
-    if (text)
-      pls->dev_text = 1; /* want to draw text */
+    if (text) 
+     {
+	pls->dev_text = 1; /* want to draw text */
+	pls->dev_unicode = 1; /* want unicode */
+	if(hrshsym) pls->dev_hrshsym = 1; /* want Hershey symbols */
+     }
 
     pls->dev_fill0 = 1;		/* Can do solid fills */
 
@@ -305,6 +345,7 @@ ps_init(PLStream *pls)
     fprintf(OF, "/SW {stringwidth 2 index mul exch 2 index mul exch rmoveto pop} bind def\n");
     fprintf(OF, "/B {Z %d %d M %d %d D %d %d D %d %d D %d %d closepath} def\n",
 	    XMIN, YMIN, XMIN, YMAX, XMAX, YMAX, XMAX, YMIN, XMIN, YMIN);
+    fprintf(OF, "/CL {newpath M D D D closepath clip} def\n");
 
 /* End of dictionary definition */
 
@@ -334,7 +375,7 @@ void
 plD_line_ps(PLStream *pls, short x1a, short y1a, short x2a, short y2a)
 {
     PSDev *dev = (PSDev *) pls->dev;
-    int x1 = x1a, y1 = y1a, x2 = x2a, y2 = y2a;
+    PLINT x1 = x1a, y1 = y1a, x2 = x2a, y2 = y2a;
 
 /* Rotate by 90 degrees */
 
@@ -515,7 +556,7 @@ plD_state_ps(PLStream *pls, PLINT op)
     case PLSTATE_WIDTH:{
 	int width = 
 	    (pls->width < MIN_WIDTH) ? DEF_WIDTH :
-	    (pls->width > MAX_WIDTH) ? MAX_WIDTH : DEF_WIDTH * pls->width;
+	    (pls->width > MAX_WIDTH) ? MAX_WIDTH : pls->width;
 
 	fprintf(OF, " S\n%d W", width);
 
@@ -582,7 +623,7 @@ fill_polygon(PLStream *pls)
 {
     PSDev *dev = (PSDev *) pls->dev;
     PLINT n, ix = 0, iy = 0;
-    int x, y;
+    PLINT x, y;
 
     fprintf(OF, " Z\n");
 
@@ -651,290 +692,326 @@ ps_getdate(void)
     return p;
 }
 
-/* this needs to be splited and beaultified! */
+
+/*--------------------------------------------------------------------------*\
+ * proc_str()
+ *
+ * Prints postscript strings.
+ * N.B. Now unicode only, no string access!
+ *
+\*--------------------------------------------------------------------------*/
+
 void
 proc_str (PLStream *pls, EscText *args)
 {
-  PLFLT *t = args->xform, tt[4];
-  PLFLT a1, alpha, ft_ht, angle, ref;
+  PLFLT *t = args->xform, tt[4]; /* Transform matrices */
+  PLFLT theta;  /* Rotation angle and shear from the matrix */
+  PLFLT ft_ht, offset; /* Font height and offset */
+  PLFLT cs,sn,l1,l2;
   PSDev *dev = (PSDev *) pls->dev;
-  char *font, *ofont, str[128], esc, *strp;
-  const char *cur_str;
-  float font_factor = 1.4, up;
-  int symbol;
-  PLINT clxmin, clxmax, clymin, clymax;
+  char *font, esc;
+  /* Be generous.  Used to store lots of font changes which take
+   * 3 characters per change.*/
+  #define PROC_STR_STRING_LENGTH 1000
+  unsigned char *strp, str[PROC_STR_STRING_LENGTH], *cur_strp, 
+     cur_str[PROC_STR_STRING_LENGTH];
+  float font_factor = 1.4;
+  PLINT clxmin, clxmax, clymin, clymax; /* Clip limits */
+  PLINT clipx[4],clipy[4]; /* Current clip limits */
 
-  /* finish previous polyline */
+  PLFLT scale = 1., up = 0.; /* Font scaling and shifting parameters */
 
-  dev->xold = PL_UNDEFINED;
-  dev->yold = PL_UNDEFINED;
+  int i=0; /* String index */
 
-  /* font height */
+  short text_len;
 
-  ft_ht = pls->chrht * 72.0/25.4; /* ft_ht in points, ht is in mm */
+   /* unicode only! so test for it. */
+   if (args->unicode_array_len>0)
+     {
+	int j,s,f;
+	char  *fonts[PROC_STR_STRING_LENGTH];
+	int nlookup;
+	const Unicode_to_Type1_table *lookup;
+	const PLUNICODE *cur_text;
+	const PLUNICODE *cur_text_limit;
+	PLUNICODE fci;
+	/* translate from unicode into type 1 font index. */
+	/*
+	 * Choose the font family, style, variant, and weight using
+	 * the FCI (font characterization integer).
+	 */
 
-  /* correct font size when zooming */
-  /* fprintf(stderr,"%f %f\n", pls->dipxax, pls->dipyay); */
-  
-  /* calculate baseline text angle */
-
-  angle = ((PLFLT)(ORIENTATION-1) - pls->diorot) * 90.;
-  a1 = acos(t[0]) * 180. / PI;
-  if (t[2] > 0.)
-    alpha = a1 - angle;
-  else
-    alpha = 360. - a1 - angle;
-
-  /* 
-   * Reference point (center baseline of string)
-   *  If base = 0, it is aligned with the center of the text box
-   *  If base = 1, it is aligned with the baseline of the text box
-   *  If base = 2, it is aligned with the top of the text box
-   *  Currently plplot only uses base=0
-   *  postscript use base=1
-   */ 
-
-  if (args->base == 2) /* not supported by plplot */
-    ref = ENLARGE * ft_ht / 2.; /* half font height */
-  else if (args->base == 1)
-    ref = 0.;
-  else
-    ref = -ENLARGE * ft_ht / 2.;
-
-  /* apply transformations */
-
-  difilt(&args->x, &args->y, 1, &clxmin, &clxmax, &clymin, &clymax);
-
-  /* check clip limits. For now, only the reference point of the string is checked;
-     but the the whole string should be checked -- using a postscript construct
-     such as gsave/clip/grestore. This method can also be applied to the xfig and
-     pstex drivers. Zoom side effect: the font size must be adjusted! */
-
-  if ( args->x < clxmin || args->x > clxmax || args->y < clymin || args->y > clymax)
-    return;
-
-  /* rotate point in postscript is lower left corner, compensate */
-
-  args->y -=  ref * cos(alpha * PI/180.);
-  args->x +=  ref * sin(alpha * PI/180.);
-
-  /* ps driver is rotated by default, compensate */
-
-  plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax, &(args->x), &(args->y));
-
-  /*
-   *  font family, serie and shape. Currently not fully supported by plplot
-   *
-   *  Use Postscript Times
-   *  1: Normal font
-   *  2: Roman font
-   *  3: Italic font
-   *  4: cursive
-   */
-
-  switch (pls->cfont) {
-  case 1: ofont = "Helvetica"; break;
-  case 2: ofont = "Times-Roman"; break;
-  case 3: ofont = "Times-Italic"; break;
-  case 4: ofont = "ZapfChancery"; break; /* there is no script (cursive) font in the standard 35 postscript fonts.  ZapfChancery is the fanciest. */
-  default:  ofont = "Helvetica";
-  }
-
-  plgesc(&esc);
-
-  cur_str = args->string;
-
-  /* move to string reference point */
-
-  fprintf(OF, " %d %d M\n", args->x, args->y );
-
-  /* set string rotation */
-
-  if (text == 1) 
-    fprintf(OF, "gsave %.3f R\n", alpha - 90.);  
-  else {
-    /* I really don't understand this! The angle should be increased by
-     * 90 degrees, only, as the ps driver has a default orientation of 
-     * landscape, see plRotPhy(ORIENTATION,...) above */
-
-    /* AWI further comment.  Actually, the traditional value of ORIENTATION
-     * was 1 corresponding to seascape which may have contributed to the
-     * confusion.  I have now changed that in plplotP.h
-     * to be 3 corresponding to landscape.  I have proved the above change
-     * to the definition of angle works both with ORIENTATION of 1 and 3
-     * for text == 1 and not.  */
-
-    if (fmod(angle, 180.) == 0.)
-      angle += 90.;
-    else
-      angle -= 90.;
-    
-    angle = angle*PI/180.;
-    tt[0] = t[0]; tt[1] = t[1]; tt[2] = t[2]; tt[3] = t[3];
-    /* fprintf(OF,"%% %.1f %.1f %.1f %.1f:\n", t[0], t[1], t[2], t[3]);*/
-
-    /* add graph orientation angle to rotation matrix */
-
-    t[0] =  tt[0]*cos(angle) + tt[1]*sin(angle);
-    t[1] = -tt[0]*sin(angle) + tt[1]*cos(angle);
-    t[2] =  tt[2]*cos(angle) + tt[3]*sin(angle);
-    t[3] = -tt[2]*sin(angle) + tt[3]*cos(angle);
-
-    /* correct for vertically slanted, but baseline inclined, text
-       (as seen in 3d axis labels/ticks) */
-
-    if (tt[0] == 0. && tt[2] == 1.) {
-      tt[1] = t[1];
-      t[1] = -t[2];
-      t[2] = -tt[1];
-    }
-    else if (tt[1] == 0. && tt[3] == 1.) {
-      tt[0] = t[0];
-      t[0] = t[3];
-      t[3] = tt[0];
-    }
-    fprintf(OF, "gsave\n");
-  }
-
-  /* Purge escape sequences from string, so that postscript can find it's length.
-     The string length is computed with the current font, and can thus be wrong
-     if there are font change escape sequences in the string */
-
-  esc_purge(str, args->string);
-
-  if (text == 1)
-    fprintf(OF, "/%s %.1f SF\n", ofont, font_factor * ENLARGE * ft_ht);
-  else {
-    fprintf(OF, "/%s [%.3f %.3f %.3f %.3f 0 0] SF\n", ofont,
-	    font_factor * ENLARGE * ft_ht * t[0],
-	    font_factor * ENLARGE * ft_ht * t[2],
-	    font_factor * ENLARGE * ft_ht * t[1],
-	    font_factor * ENLARGE * ft_ht * t[3]);
-  }
-
-  /* move to start point, taking justification into account */
-
-  fprintf(OF, "%.3f (%s) SW\n", - args->just, str);
-
-  /* parse string for escape sequences */
-
-  do {
-    strp = str;
-    font = ofont;
-    symbol = 0;
-    up = 0.;
-
-    if (*cur_str == esc) {
-      cur_str++;
-
-      if (*cur_str == esc) { /* <esc><esc> */
-	*strp++ = *cur_str++;
-      }
-      else switch (*cur_str) {
-
-      case 'f':
-	cur_str++;
-	switch (*cur_str) {
-	case 'n': font = "Helvetica"; break;
-	case 'r': font = "Times-Roman"; break;
-	case 'i': font = "Times-Italic"; break;
-	case 's': font = "ZapfChancery"; break;  /* there is no script (cursive) font in the standard 35 postscript fonts.  ZapfChancery is the fanciest. */
-	default:  font = "Helvetica";
+	plgesc(&esc);
+	plgfci(&fci);
+	font = plP_FCI2FontName(fci, Type1Lookup, N_Type1Lookup);
+	if (font == NULL) {
+	   fprintf(stderr, "fci = 0x%x, font name pointer = NULL \n", fci);
+	   plabort("proc_str: FCI inconsistent with Type1Lookup; "
+		   "internal PLplot error");
+	   return;
 	}
-	cur_str++;
-	break;
+	/*pldebug("proc_str", "fci = 0x%x, font name = %s\n", fci, font);*/
+	if (!strcmp(font, "Symbol")) {
+	   nlookup = number_of_entries_in_unicode_to_symbol_table;
+	   lookup = unicode_to_symbol_lookup_table;
+	}
+	else {
+	   nlookup = number_of_entries_in_unicode_to_standard_table;
+	   lookup = unicode_to_standard_lookup_table;
+	}
+	cur_text =  args->unicode_array;
+	for (f=s=j=0; j < args->unicode_array_len; j++) {
+	   if (cur_text[j] & PL_FCI_MARK) {
+	      /* process an FCI by saving it and escaping cur_str
+	       * with an escff to make it a 2-character escape
+	       * that is not used in legacy Hershey code
+	       */
+	      if ((f < PROC_STR_STRING_LENGTH) && (s+3 < PROC_STR_STRING_LENGTH)) {
+		 fonts[f] = plP_FCI2FontName(cur_text[j], Type1Lookup, N_Type1Lookup);
+		 if (fonts[f] == NULL) {
+		    fprintf(stderr, "string-supplied FCI = 0x%x, font name pointer = NULL \n", cur_text[j]);
+		    plabort("proc_str: string-supplied FCI inconsistent with Type1Lookup;");
+		    return;
+		 }
+		 /*pldebug("proc_str", "string-supplied FCI = 0x%x, font name = %s\n", cur_text[j], fonts[f]);*/
+		 if (!strcmp(fonts[f++], "Symbol")) {
+		    lookup = unicode_to_symbol_lookup_table;
+		    nlookup = number_of_entries_in_unicode_to_symbol_table;
+		 }
+		 else {
+		    lookup = unicode_to_standard_lookup_table;
+		    nlookup = number_of_entries_in_unicode_to_standard_table;
+		 }
+		 cur_str[s++] = esc;
+		 cur_str[s++] = 'f';
+		 cur_str[s++] = 'f';
+	      }
+	   }
+	   else if (s+1 < PROC_STR_STRING_LENGTH) {
+	      cur_str[s++] = plunicode2type1(cur_text[j], lookup, nlookup);
+	      /*pldebug("proc_str", "unicode = 0x%x, type 1 code = %d\n",
+	                cur_text[j], cur_str[j]);*/
+	   }
+	}
+	cur_str[s] = '\0';
+	
+	/* finish previous polyline */
+	
+	dev->xold = PL_UNDEFINED;
+	dev->yold = PL_UNDEFINED;
+	
+	/* Determine the font height */
+	ft_ht = pls->chrht * 72.0/25.4; /* ft_ht in points, ht is in mm */
+	
+	
+	/* The transform matrix has only rotations and shears; extract them */
+	theta = acos(t[0]) * 180. / PI;  /* Determine the rotation (in degrees)... */
+	if (t[2] < 0.) theta *= -1.;     /* ... and sign ... */
+	cs = cos(theta*PI/180.);
+	sn = sin(theta*PI/180.);
+	tt[0] = t[0]*cs + t[2]*sn;
+	tt[1] = t[1]*cs + t[3]*sn;
+	tt[2] = -t[0]*sn + t[2]*cs;
+	tt[3] = -t[1]*sn + t[3]*cs;
+	
+	/* 
+	 * Reference point conventions:
+	 *   If base = 0, it is aligned with the center of the text box
+	 *   If base = 1, it is aligned with the baseline of the text box
+	 *   If base = 2, it is aligned with the top of the text box
+	 *
+	 * Currently plplot only uses base=0
+	 * Postscript uses base=1
+	 *
+	 * We must calculate the difference between the two and apply the offset.
+	 */ 
+	
+	if (args->base == 2) /* not supported by plplot */
+	  offset = ENLARGE * ft_ht / 2.; /* half font height */
+	else if (args->base == 1)
+	  offset = 0.;
+	else
+	  offset = -ENLARGE * ft_ht / 2.;
+	
+	/* Determine the adjustment for page orientation */
+	theta -= 90.*pls->diorot;
+	args->y += offset*cos(theta*PI/180.);
+	args->x -= offset*sin(theta*PI/180.);
+		
+	/* ps driver is rotated by default */
+	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax, 
+		 &(args->x), &(args->y));
+	
+	/* Correct for the fact ps driver uses landscape by default */
+	theta += 90.;
+	
+	/* Output */
+	/* Set clipping */
+	clipx[0]=pls->clpxmi;
+	clipx[2]=pls->clpxma;
+	clipy[0]=pls->clpymi;
+	clipy[2]=pls->clpyma;
+	clipx[1]=clipx[2];
+	clipy[1]=clipy[0];
+	clipx[3]=clipx[0];
+	clipy[3]=clipy[2];
+	difilt(clipx, clipy, 4, &clxmin, &clxmax, &clymin, &clymax);
+	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax,
+	         &clipx[0], &clipy[0]);
+	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax,
+	         &clipx[1], &clipy[1]);
+	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax,
+	         &clipx[2], &clipy[2]);
+	plRotPhy(ORIENTATION, dev->xmin, dev->ymin, dev->xmax, dev->ymax,
+	         &clipx[3], &clipy[3]);
+	fprintf(OF," gsave %d %d %d %d %d %d %d %d CL\n",clipx[0],clipy[0],clipx[1],clipy[1],clipx[2],clipy[2],clipx[3],clipy[3]);
+	
+	/* move to string reference point */
+	fprintf(OF, " %d %d M\n", args->x, args->y );
+	
+	/* Save the current position and set the string rotation */
+	fprintf(OF, "gsave %.3f R\n",theta);
+	
+	/* Purge escape sequences from string, so that postscript can find it's 
+	 * length.  The string length is computed with the current font, and can
+	 * thus be wrong if there are font change escape sequences in the string 
+	 */
+	
+	esc_purge(str, cur_str);
+	
+	fprintf(OF, "/%s %.3f SF\n", font,font_factor * ENLARGE * ft_ht);    
+	
+	/* Output string, while escaping the '(', ')' and '\' characters.
+	 * this string is output for measurement purposes only.
+	 */
+	fprintf(OF, "%.3f (", - args->just);
+	while (str[i]!='\0') {
+	   if (str[i]=='(' || str[i]==')' || str[i]=='\\')
+	     fprintf(OF,"\\%c",str[i]);
+	   else
+	     fprintf(OF,"%c",str[i]);
+	   i++;
+	}
+	fprintf(OF,") SW\n");
+	
+	
+	/* Parse string for PLplot escape sequences and print everything out */
+	
+	cur_strp = cur_str;
+	f = 0;
+	do {
+	   
+	   strp = str;
+	   
+	   if (*cur_strp == esc) {
+	      cur_strp++;
+	      
+	      if (*cur_strp == esc) { /* <esc><esc> */
+		 *strp++ = *cur_strp++;
+	      }
+	      else if (*cur_strp == 'f') {
+		 cur_strp++;
+		 if (*cur_strp++ != 'f') {
+		    /* escff occurs because of logic above. But any suffix
+		     * other than "f" should never happen. */
+		    plabort("proc_str, internal PLplot logic error;"
+			    "wrong escf escape sequence");
+		    return;
+		 }
+		 font = fonts[f++];
+		 /*pldebug("proc_str", "string-specified fci = 0x%x, font name = %s\n", fci, font);*/
+		 continue;
+	      }
+	      else switch (*cur_strp++) {
+		 
+	       case 'd':
+	       case 'D':
+		 if(up>0.) scale *= 1.25;  /* Subscript scaling parameter */
+		 else scale *= 0.8;  /* Subscript scaling parameter */
+		 up -= font_factor * ENLARGE * ft_ht / 2.;
+		 break;
+		 
+	       case 'u':
+	       case 'U':
+		 if(up<0.) scale *= 1.25;  /* Subscript scaling parameter */
+		 else scale *= 0.8;  /* Subscript scaling parameter */
+		 up += font_factor * ENLARGE * ft_ht / 2.;
+		 break;
+		 
+		 /* ignore the next sequences */
+		 
+	       case '+':
+	       case '-':
+	       case 'b':
+	       case 'B':
+		 plwarn("'+', '-', and 'b/B' text escape sequences not processed.");
+		 break;
+	      }
+	   }
+	   
+	   /* copy from current to next token, adding a postscript escape 
+	    * char '\' if necessary 
+	    */
+	   while(*cur_strp && *cur_strp != esc) {
+	      if (*cur_strp == '(' || *cur_strp == ')' || *cur_strp == '\\')
+		*strp++ = '\\';
+	      *strp++ = *cur_strp++;
+	   }
+	   *strp = '\0';
+	   
+	   if(fabs(up)<0.001) up = 0.; /* Watch out for small differences */
+	   
+	   /* Apply the scaling and the shear */
+	   fprintf(OF, "/%s [%.3f %.3f %.3f %.3f 0 0] SF\n",
+		   font,
+		   tt[0]*font_factor * ENLARGE * ft_ht * scale,
+		   tt[2]*font_factor * ENLARGE * ft_ht * scale,
+		   tt[1]*font_factor * ENLARGE * ft_ht * scale,
+		   tt[3]*font_factor * ENLARGE * ft_ht * scale);
+	   
+	   /* if up/down escape sequences, save current point and adjust baseline;
+	    * take the shear into account */
+	   if(up!=0.) fprintf(OF, "gsave %.3f %.3f rmoveto\n",up*tt[1],up*tt[3]);
+	   
+	   /* print the string */
+	   fprintf(OF, "(%s) show\n", str);  
+	   
+	   /* back to baseline */
+	   if (up!=0.) fprintf(OF, "grestore (%s) stringwidth rmoveto\n", str);
+	   
+	}while(*cur_strp);
+	
+	fprintf(OF, "grestore\n");
+	fprintf(OF, "grestore\n");
+	
+	/*
+	 * keep driver happy -- needed for background and orientation.
+	 * arghhh! can't calculate it, as I only have the string reference
+	 * point, not its extent!
+	 * Still a hack - but at least it takes into account the string
+	 * length and justification. Character width is assumed to be
+	 * 0.6 * character height. Add on an extra 1.5 * character height 
+	 * for safety. 
+	 */
+	cs = cos(theta/180.*PI);
+	sn = sin(theta/180.*PI);
+	l1 = -i*args->just;
+	l2 = i*(1.-args->just);
+	/* Factor of 0.6 is an empirical fudge to convert character 
+	 * height to average character width */
+	l1 *= 0.6;
+	l2 *= 0.6;
+	
+	dev->llx = MIN(dev->llx, args->x + (MIN(l1*cs,l2*cs)-1.5) * font_factor * ft_ht * ENLARGE );
+	dev->lly = MIN(dev->lly, args->y + (MIN(l1*sn,l2*sn)-1.5) * font_factor * ft_ht * ENLARGE );
+	dev->urx = MAX(dev->urx, args->x + (MAX(l1*cs,l2*cs)+1.5) * font_factor * ft_ht * ENLARGE );
+	dev->ury = MAX(dev->ury, args->y + (MAX(l1*sn,l2*sn)+1.5) * font_factor * ft_ht * ENLARGE );
 
-      case 'd':
-	up = - font_factor * ENLARGE * ft_ht / 2.;
-	cur_str++;
-	break;
-
-      case 'u':
-	up = font_factor * ENLARGE * ft_ht / 2.;
-	cur_str++;
-	break;
-
-      case 'g':
-	cur_str++;
-	ofont = font;
-	*strp++ = *cur_str++;
-	symbol = 1;
-	font = "Symbol";
-	break;
-
-	/* ignore the next sequences */
-
-      case '+':
-      case '-':
-      case 'b':
-	plwarn("'+', '-', and 'b' text escape sequences not processed.");
-	cur_str++;
-	break;
-
-      case '(':
-	plwarn("'g(...)' text escape sequence not processed.");
-	while (*cur_str++ != ')');
-	break;
-      }
-    }
-
-    /* copy from current to next token, adding a postscript escape char \ if necessary */
-
-    while(!symbol && *cur_str && *cur_str != esc) {
-      if (*cur_str == '(' || *cur_str == ')')
-	*strp++ = '\\';
-      *strp++ = *cur_str++;
-    }
-    *strp = '\0';
-
-    /* set the font */
-
-    if (text == 1)
-      fprintf(OF, "/%s %.1f SF\n", font, font_factor * ENLARGE * ft_ht);    
-    else {
-      fprintf(OF, "/%s [%.3f %.3f %.3f %.3f %.3f %.3f] SF\n", font,
-	      font_factor * ENLARGE * ft_ht * t[0],
-	      font_factor * ENLARGE * ft_ht * t[2],
-	      font_factor * ENLARGE * ft_ht * t[1],
-	      font_factor * ENLARGE * ft_ht * t[3],
-	      -fabs(up)*sin(angle), fabs(up)*cos(angle));
-    }
-
-    /* if up/down escape sequences, save current point and adjust baseline */
-
-    if (text == 1 && up)
-      fprintf(OF, "0 %.3f rmoveto gsave\n", up);
-
-    /* print the string */
-
-    fprintf(OF, "(%s) show\n", str);  
-
-
-    /* back to baseline */
-
-    if (text == 1 && up)
-      fprintf(OF, "grestore (%s) stringwidth rmoveto\n", str);
-
-  }while(*cur_str);
-
-  fprintf(OF, "grestore\n");
-
-  /*
-   * keep driver happy -- needed for background and orientation.
-   * arghhh! can't calculate it, as I only have the string reference
-   * point, not its extent!
-   * Quick (and final?) *hack*, ASSUME that no more than two char height
-   * extents after/before the string reference point.
-   */
-
-  dev->llx = MIN(dev->llx, args->x - 2. * font_factor * ft_ht * 25.4 / 72. * pls->xpmm);
-  dev->lly = MIN(dev->lly, args->y - 2. * font_factor * ft_ht * 25.4 / 72. * pls->ypmm);
-  dev->urx = MAX(dev->urx, args->x + 2. * font_factor * ft_ht * 25.4 / 72. * pls->xpmm);
-  dev->ury = MAX(dev->ury, args->y + 2. * font_factor * ft_ht * 25.4 / 72. * pls->ypmm);
+     }
 }
 
 static void
-esc_purge(char *dstr, const char *sstr) {
+esc_purge(char *dstr, unsigned char *sstr) {
   char esc;
 
   plgesc(&esc);
@@ -946,19 +1023,17 @@ esc_purge(char *dstr, const char *sstr) {
     }
 
     sstr++;
-    if (*sstr == esc)
+    if (*sstr == esc) {
+      *dstr++ = *sstr++;
       continue;
+    }
+     
     else {
       switch(*sstr++) {
-      case 'g': 
       case 'f':
 	sstr++;
 	break; /* two chars sequence */
 
-      case '(':
-	while (*sstr++ != ')'); /* multi chars sequence */
-	break;
-
       default:
 	break;  /* single char escape */
       }
@@ -967,6 +1042,48 @@ esc_purge(char *dstr, const char *sstr) {
   *dstr = '\0';
 }
 
+/*--------------------------------------------------------------------------*\
+ *  unsigned char plunicode2type1 (const PLUNICODE index, 
+ *       const Unicode_to_Type1_table lookup[], const int number_of_entries)
+ *
+ *  Function takes an input unicode index, looks through the lookup
+ *  table (which must be sorted by PLUNICODE Unicode), then returns the 
+ *  corresponding Type1 code in the lookup table.  If the Unicode index is
+ *  not present the returned value is 32 (which is normally a blank
+ *  for Type 1 fonts).
+\*--------------------------------------------------------------------------*/
+
+static unsigned char 
+plunicode2type1 (const PLUNICODE index,
+		 const Unicode_to_Type1_table lookup[],
+		 const int nlookup)
+{
+   int jlo = -1, jmid, jhi = nlookup;
+   while (jhi - jlo > 1) 
+     {
+	/* Note that although jlo or jhi can be just outside valid
+	 * range (see initialization above) because of while condition
+	 * jlo < jmid < jhi and jmid must be in valid range.
+	 */
+	jmid = (jlo+jhi)/2;
+	if (index > lookup[jmid].Unicode)
+	  jlo = jmid;
+	else if (index < lookup[jmid].Unicode)
+	  jhi = jmid;
+	else
+	  /* We have found it!
+	   * index == lookup[jmid].Unicode 
+	   */
+	  return (lookup[jmid].Type1);
+     }
+   /* jlo is invalid or it is valid and index > lookup[jlo].Unicode.
+    * jhi is invalid or it is valid and index < lookup[jhi].Unicode.
+    * All these conditions together imply index cannot be found in lookup.
+    * Mark with 32 (which is normally a blank in type 1 fonts).
+    */
+   return(32);
+}
+
 #else
 int 
 pldummy_ps()
