@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x01.cc,v 1.13 2004/03/26 08:30:41 andrewross Exp $
+// $Id: x01.cc,v 1.20 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,25 +19,18 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 1 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
+#include "plc++demos.h"
+
 #include "plevent.h"
-
-#include <iostream>
 #include <cctype>
 
-#ifdef HAVE_CMATH
-#include <cmath>
-#else
-#include <math.h>
-#endif
-
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
 #endif
@@ -137,7 +130,7 @@ x01::x01( int argc, char ** argv ) {
 
   // Parse and process command line arguments.
   pls->MergeOpts(options, "x01c options", notes);
-  pls->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls->parseopts( &argc, argv, PL_PARSE_FULL );
 
   // Print out version number, just for kicks.
   pls->gver(ver);
@@ -193,7 +186,7 @@ x01::x01( int argc, char ** argv ) {
     pls2->sfnam(f_name);       /* file name */
     pls2->sdev("psc");         /* device type */
 
-    pls2->cpstrm(*pls, 0);     /* copy old stream parameters to new stream */
+    pls2->cpstrm(*pls, false); /* copy old stream parameters to new stream */
     pls2->replot();            /* do the save by replaying the plot buffer */
     
     delete pls2;
@@ -231,7 +224,7 @@ x01::x01( int argc, char ** argv ) {
 void x01::plot1( int do_test )
 {
   int i;
-  PLINT st;
+  bool st;
   PLFLT xmin, xmax, ymin, ymax;
   PLFLT *x = new PLFLT[60];
   PLFLT *y = new PLFLT[60];
@@ -241,7 +234,7 @@ void x01::plot1( int do_test )
   for( i=0; i < 60; i++ )
     {
       x[i] = xoff + xscale * (i + 1) / 60.0;
-      y[i] = yoff + yscale * pow(x[i], 2.);
+      y[i] = yoff + yscale * pow((double)x[i], 2.);
     }
 
   xmin = x[0];
@@ -279,7 +272,7 @@ void x01::plot1( int do_test )
 
   if (do_test && test_xor) {
 #ifdef HAVE_USLEEP
-    pls->xormod(1, &st); /* enter xor mode */
+    pls->xormod(true, &st); /* enter xor mode */
     if (st) {
       for (i=0; i<60; i++) {
 	pls->poin(1, x+i, y+i,9);      /* draw a point */
@@ -287,7 +280,7 @@ void x01::plot1( int do_test )
 	pls->flush();                  /* force an update of the tk driver */
 	pls->poin(1, x+i, y+i,9);      /* erase point */
       }
-      pls->xormod(0, &st);                     /* leave xor mode */
+      pls->xormod(false, &st);                     /* leave xor mode */
     }
 #else
     cout << "The -xor command line option can only be exercised if your system has usleep(), which does not seems to happen." << endl;
@@ -338,10 +331,10 @@ void x01::plot2()
 void x01::plot3()
 {
   int i;
-  int space0 = 0;
-  int mark0 = 0;
-  int space1 = 1500;
-  int mark1 = 1500;
+  PLINT space0 = 0;
+  PLINT mark0 = 0;
+  PLINT space1 = 1500;
+  PLINT mark1 = 1500;
   PLFLT *x = new PLFLT[101];
   PLFLT *y = new PLFLT[101];
 
