@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x08.cc,v 1.7 2004/03/03 17:41:16 andrewross Exp $
+// $Id: x08.cc,v 1.15 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,17 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 8 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -102,7 +99,7 @@ void x08::cmap1_init(int gray)
   PLFLT *h = new PLFLT[2];
   PLFLT *l = new PLFLT[2];
   PLFLT *s = new PLFLT[2];
-  int *rev = new int[2];
+  bool *rev = new bool[2];
 
   i[0] = 0.0;         // left boundary
   i[1] = 1.0;         // right boundary
@@ -128,11 +125,11 @@ void x08::cmap1_init(int gray)
     s[1] = 0.8;
   }
 
-  rev[0] = 0;         // interpolate on front side of colour wheel.
-  rev[1] = 0;         // interpolate on front side of colour wheel.
+  rev[0] = false;         // interpolate on front side of colour wheel.
+  rev[1] = false;         // interpolate on front side of colour wheel.
 
   pls->scmap1n(256);
-  pls->scmap1l(0, 2, i, h, l, s, rev);
+  pls->scmap1l(false, 2, i, h, l, s, rev);
 
   delete[] i;
   delete[] h;
@@ -163,7 +160,7 @@ x08::x08( int argc, char **argv ) {
 
   // Parse and process command line arguments.
   pls->MergeOpts(options, "x08c options",  NULL);
-  pls->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls->parseopts( &argc, argv, PL_PARSE_FULL );
   if (sombrero) {
     rosen = false;
   }
@@ -203,7 +200,7 @@ x08::x08( int argc, char **argv ) {
 	{
 	  yy = y[j];
 	  if (rosen) {
-	    z[i][j] = pow(1. - xx,2.) + 100 * pow(yy - pow(xx,2.),2.);
+	    z[i][j] = pow((double)(1. - xx),2.) + 100 * pow((double)(yy - pow((double)xx,2.)),2.);
 	    /* The log argument may be zero for just the right grid.  */
 	    if (z[i][j] > 0.)
 	       z[i][j] = log(z[i][j]);
@@ -259,9 +256,12 @@ x08::x08( int argc, char **argv ) {
 	    cmap1_init(0);
 	    pls->surf3d( x, y, z, XPTS, YPTS, MAG_COLOR, NULL, 0 );
 	  } else if (ifshade == 2) { /*  magnitude colored plot with faceted squares */
+	    cmap1_init(0);
 	    pls->surf3d( x, y, z, XPTS, YPTS, MAG_COLOR | FACETED, NULL, 0 );
-	  } else                     /* magnitude colored plot with contours */
+	  } else {                   /* magnitude colored plot with contours */
+	    cmap1_init(0);
 	    pls->surf3d( x, y, z, XPTS, YPTS, MAG_COLOR | SURF_CONT | BASE_CONT, clevel, LEVELS );
+	  }
 	}
     }
 
