@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x11.cc,v 1.5 2004/03/03 17:41:16 andrewross Exp $
+// $Id: x11.cc,v 1.11 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,17 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 11 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -76,7 +73,7 @@ void x11::cmap1_init()
   PLFLT *h = new PLFLT[2];
   PLFLT *l = new PLFLT[2];
   PLFLT *s = new PLFLT[2];
-  int *rev = new int[2];
+  bool *rev = new bool[2];
 
   i[0] = 0.0;         // left boundary
   i[1] = 1.0;         // right boundary
@@ -90,11 +87,11 @@ void x11::cmap1_init()
   s[0] = 0.8;
   s[1] = 0.8;
 
-  rev[0] = 0;         // interpolate on front side of colour wheel.
-  rev[1] = 0;         // interpolate on front side of colour wheel.
+  rev[0] = false;         // interpolate on front side of colour wheel.
+  rev[1] = false;         // interpolate on front side of colour wheel.
 
   pls->scmap1n(256);
-  pls->scmap1l(0, 2, i, h, l, s, rev);
+  pls->scmap1l(false, 2, i, h, l, s, rev);
 
   delete[] i;
   delete[] h;
@@ -124,7 +121,7 @@ x11::x11( int argc, char **argv ) {
 
   // Parse and process command line arguments.
 
-  pls->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls->parseopts( &argc, argv, PL_PARSE_FULL );
 
 
   // Initialize plplot.
@@ -147,7 +144,7 @@ x11::x11( int argc, char **argv ) {
 	{
 	  yy = y[j];
 	  z[i][j] = 3. * (1.-xx)*(1.-xx) * exp(-(xx*xx) - (yy+1.)*(yy+1.)) -
-	    10. * (xx/5. - pow(xx,3.) - pow(yy,5.)) * exp(-xx*xx-yy*yy) -
+	    10. * (xx/5. - pow((double)xx,3.) - pow((double)yy,5.)) * exp(-xx*xx-yy*yy) -
 	    1./3. * exp(-(xx+1)*(xx+1) - (yy*yy));
 	  if(false) { /* Jungfraujoch/Interlaken */
 	    if (z[i][j] < -1.)
