@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x14.cc,v 1.8 2004/03/03 17:41:16 andrewross Exp $
+// $Id: x14.cc,v 1.13 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,17 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 14 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -106,7 +103,7 @@ x14::x14( int argc, char ** argv ) {
   pls1 = new plstream();
 
   // Parse and process command line arguments.
-  pls1->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls1->parseopts( &argc, argv, PL_PARSE_FULL );
 
   pls1->gdev(driver);
 
@@ -128,7 +125,7 @@ x14::x14( int argc, char ** argv ) {
   // Turn off pause to make this a slave (must follow master)
 
   pls2->SetOpt("geometry", geometry_slave);
-  pls2->spause(0);
+  pls2->spause(false);
   pls2->sdev(driver);
   pls2->init();
 
@@ -198,7 +195,7 @@ void x14::plot1(plstream *pls)
 
     for (i = 0; i < 60; i++) {
 	x[i] = xoff + xscale * (i + 1) / 60.0;
-	y[i] = yoff + yscale * pow(x[i], 2.);
+	y[i] = yoff + yscale * pow((double)x[i], 2.);
     }
 
     xmin = x[0];
