@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x16.cc,v 1.8 2004/06/24 15:44:27 airwin Exp $
+// $Id: x16.cc,v 1.13 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,17 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 16 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -130,7 +127,7 @@ static void mypltr(PLFLT x, PLFLT y, PLFLT *tx, PLFLT *ty, void *pltr_data) {
   *ty = x16::tr[3] * x + x16::tr[4] * y + x16::tr[5];
 }
 
-int zdefined (PLFLT x, PLFLT y) {
+PLINT zdefined (PLFLT x, PLFLT y) {
   PLFLT z = sqrt(x * x + y * y);
 
   return z < 0.4 || z > 0.6;
@@ -159,7 +156,7 @@ x16::x16( int argc, char ** argv ) {
 
   // Parse and process command line arguments.
   pls->MergeOpts(options, "x16c options", notes);
-  pls->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls->parseopts( &argc, argv, PL_PARSE_FULL );
 
   // Reduce colors in cmap 0 so that cmap 1 is useful on a
   //16-color display
@@ -238,7 +235,7 @@ x16::x16( int argc, char ** argv ) {
   pls->shades(z, nx, ny, NULL, -1., 1., -1., 1.,
 	       shedge, ns+1, fill_width,
 	       cont_color, cont_width,
-	       plstream::fill, 1, NULL, NULL);
+	       plstream::fill, true, NULL, NULL);
 
   pls->col0(1);
   pls->box("bcnst", 0.0, 0, "bcnstv", 0.0, 0);
@@ -256,7 +253,7 @@ x16::x16( int argc, char ** argv ) {
   pls->shades(z, nx, ny, NULL, -1., 1., -1., 1.,
 	       shedge, ns+1, fill_width,
 	       cont_color, cont_width,
-	       plstream::fill, 1, plstream::tr1, (void *) &cgrid1);
+	       plstream::fill, true, plstream::tr1, (void *) &cgrid1);
 
   pls->col0(1);
   pls->box("bcnst", 0.0, 0, "bcnstv", 0.0, 0);
@@ -275,7 +272,7 @@ x16::x16( int argc, char ** argv ) {
   pls->shades(z, nx, ny, NULL, -1., 1., -1., 1.,
 	       shedge, ns+1, fill_width,
 	       cont_color, cont_width,
-	       plstream::fill, 0, plstream::tr2, (void *) &cgrid2);
+	       plstream::fill, false, plstream::tr2, (void *) &cgrid2);
 
   pls->col0(1);
   pls->box("bcnst", 0.0, 0, "bcnstv", 0.0, 0);
@@ -295,7 +292,7 @@ x16::x16( int argc, char ** argv ) {
   pls->shades(z, nx, ny, NULL, -1., 1., -1., 1.,
 	       shedge, ns+1, fill_width,
 	       2, 3,
-	       plstream::fill, 0, plstream::tr2, (void *) &cgrid2);
+	       plstream::fill, false, plstream::tr2, (void *) &cgrid2);
 
   pls->col0(1);
   pls->box("bcnst", 0.0, 0, "bcnstv", 0.0, 0);
@@ -316,7 +313,7 @@ x16::x16( int argc, char ** argv ) {
     pls->shades(z, nx, ny, zdefined, -1., 1., -1., 1.,
              shedge, ns+1, fill_width,
              cont_color, cont_width,
-             plstream::fill, 0, plstream::tr2, (void *) &cgrid2);
+             plstream::fill, false, plstream::tr2, (void *) &cgrid2);
 
     pls->col0(1);
     pls->box("bcnst", 0.0, 0, "bcnstv", 0.0, 0);
@@ -355,7 +352,7 @@ x16::x16( int argc, char ** argv ) {
   pls->shades(z, nx, ny, NULL, -1., 1., -1., 1.,
 	       shedge, ns+1, fill_width,
 	       cont_color, cont_width,
-	       plstream::fill, 0, plstream::tr2, (void *) &cgrid2);
+	       plstream::fill, false, plstream::tr2, (void *) &cgrid2);
 
   // Now we can draw the perimeter.  (If do before, shade stuff may overlap.)
   for (i = 0; i < PERIMETERPTS; i++) {
