@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x18.cc,v 1.5 2004/03/03 17:41:16 andrewross Exp $
+// $Id: x18.cc,v 1.10 2005/12/06 16:36:03 andrewross Exp $
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
@@ -19,17 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 18 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -72,7 +69,7 @@ x18::x18( int argc, char ** argv ) {
   pls = new plstream();
 
   // Parse and process command line arguments.
-  pls->ParseOpts( &argc, argv, PL_PARSE_FULL );
+  pls->parseopts( &argc, argv, PL_PARSE_FULL );
 
   // Initialize PLplot.
   pls->init();
@@ -141,7 +138,7 @@ PLFLT x18::PHI (int a) {
 
 void x18::test_poly(int k) {
   int i, j;
-  int draw[4][4]  = {
+  PLINT draw[4][4]  = {
     { 1, 1, 1, 1 },
     { 1, 0, 1, 0 },
     { 0, 1, 0, 1 },
@@ -190,7 +187,7 @@ void x18::test_poly(int k) {
       y[4] = sin( PHI(j) ) * sin( THETA(i) );
       z[4] = cos( PHI(j) );
 
-      pls->poly3(5, x, y, z, draw[k], 1 );
+      pls->poly3(5, x, y, z, draw[k], true );
     }
   }
 
