@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x22.cc,v 1.8 2004/06/24 08:08:53 andrewross Exp $
+// $Id: x22.cc,v 1.14 2005/12/06 16:36:03 andrewross Exp $
 //    Simple vector plot example
 //---------------------------------------------------------------------------//
 //
@@ -19,18 +19,14 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 //
 //---------------------------------------------------------------------------//
 // Implementation of PLplot example 22 in C++.
 //---------------------------------------------------------------------------//
 
-#include "plstream.h"
-#include "plevent.h"
-
-#include <iostream>
-#include <cmath>
+#include "plc++demos.h"
 
 #ifdef USE_NAMESPACE
 using namespace std;
@@ -173,13 +169,13 @@ x22::potential() {
     d1 = rmax/4.;
 
     q1i = - q1*rmax/d1;
-    d1i = pow(rmax, 2.)/d1;
+    d1i = pow((double)rmax, 2.)/d1;
 
     q2 = -1.;
     d2 = rmax/4.;
 
     q2i = - q2*rmax/d2;
-    d2i = pow(rmax, 2.)/d2;
+    d2i = pow((double)rmax, 2.)/d2;
 
     for (i = 0; i < nr; i++) {
 	r = 0.5 + (double) i;
@@ -189,15 +185,15 @@ x22::potential() {
 	    y = r*sin(theta);
 	    cgrid2.xg[i][j] = x;
 	    cgrid2.yg[i][j] = y;
-	    div1 = sqrt(pow(x-d1, 2.) + pow(y-d1, 2.) + pow(eps, 2.));
-	    div1i = sqrt(pow(x-d1i, 2.) + pow(y-d1i, 2.) + pow(eps, 2.));
-	    div2 = sqrt(pow(x-d2, 2.) + pow(y+d2, 2.) + pow(eps, 2.));
-	    div2i = sqrt(pow(x-d2i, 2.) + pow(y+d2i, 2.) + pow(eps, 2.));
+	    div1 = sqrt(pow((double)(x-d1), 2.) + pow((double)(y-d1), 2.) + pow((double)eps, 2.));
+	    div1i = sqrt(pow((double)(x-d1i), 2.) + pow((double)(y-d1i), 2.) + pow((double)eps, 2.));
+	    div2 = sqrt(pow((double)(x-d2), 2.) + pow((double)(y+d2), 2.) + pow((double)eps, 2.));
+	    div2i = sqrt(pow((double)(x-d2i), 2.) + pow((double)(y+d2i), 2.) + pow((double)eps, 2.));
 	    z[i][j] = q1/div1 + q1i/div1i + q2/div2 + q2i/div2i;
-	    u[i][j] = -q1*(x-d1)/pow(div1,3.) - q1i*(x-d1i)/pow(div1i,3.0) 
-		- q2*(x-d2)/pow(div2,3.) - q2i*(x-d2i)/pow(div2i,3.);
-	    v[i][j] = -q1*(y-d1)/pow(div1,3.) - q1i*(y-d1i)/pow(div1i,3.0) 
-		- q2*(y+d2)/pow(div2,3.) - q2i*(y+d2i)/pow(div2i,3.);
+	    u[i][j] = -q1*(x-d1)/pow((double)div1,3.) - q1i*(x-d1i)/pow((double)div1i,3.0) 
+		- q2*(x-d2)/pow((double)div2,3.) - q2i*(x-d2i)/pow((double)div2i,3.);
+	    v[i][j] = -q1*(y-d1)/pow((double)div1,3.) - q1i*(y-d1i)/pow((double)div1i,3.0) 
+		- q2*(y+d2)/pow((double)div2,3.) - q2i*(y+d2i)/pow((double)div2i,3.);
 	}
     }
 
@@ -254,7 +250,8 @@ x22::f2mnmx(PLFLT **f, PLINT nx, PLINT ny, PLFLT *fmin, PLFLT *fmax)
 
 x22::x22( int argc, char ** argv ) {
 
-    PLINT narr, fill;
+    PLINT narr;
+    bool fill;
 
     // Set of points making a polygon to use as the arrow
     PLFLT arrow_x[6] = {-0.5, 0.5, 0.3, 0.5, 0.3, 0.5};
@@ -268,7 +265,7 @@ x22::x22( int argc, char ** argv ) {
 
     // Parse and process command line arguments 
 
-    pls->ParseOpts(&argc, argv, PL_PARSE_FULL);
+    pls->parseopts(&argc, argv, PL_PARSE_FULL);
 
     // Initialize plplot 
 
@@ -289,7 +286,7 @@ x22::x22( int argc, char ** argv ) {
     circulation();
 
     narr = 6;
-    fill = 0;
+    fill = false;
 
     // Set arrow style using arrow_x and arrow_y then 
     // plot using these arrows.
