@@ -1,9 +1,10 @@
-/* $Id: x17c.c,v 1.16 2002/12/03 08:39:24 airwin Exp $
+/* $Id: x17c.c,v 1.20 2005/10/07 17:53:37 airwin Exp $
 
  * Plots a simple stripchart with four pens.
  */
 
 #include "plcdemos.h"
+#include <stdlib.h>
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #else
@@ -12,6 +13,8 @@
 # endif
 #endif
 
+
+
 /* Variables for holding error return info from PLplot */
 
 static PLINT errcode;
@@ -29,12 +32,11 @@ main(int argc, char *argv[])
     PLFLT t, tmin, tmax, tjump, dt, noise;
     PLINT colbox, collab, colline[4], styline[4];
     char *legline[4], toplab[20];
-    double drand48(void);
 
 /* plplot initialization */
 /* Parse and process command line arguments */
 
-    (void) plParseOpts(&argc, argv, PL_PARSE_FULL);
+    (void) plparseopts(&argc, argv, PL_PARSE_FULL);
 
 /* If db is used the plot is much more smooth. However, because of the
    async X behaviour, one does not have a real-time scripcharter.
@@ -129,9 +131,9 @@ main(int argc, char *argv[])
 # endif
 #endif
 	t = (double)n * dt;
-	noise = drand48() - 0.5;
+	noise = ((double)rand()/(RAND_MAX+1.0)) - 0.5;
 	y1 = y1 + noise;
-	y2 = sin(t*PI/18.);
+	y2 = sin(t*M_PI/18.);
 	y3 = y2 * noise;
 	y4 = y2 + noise/3.;
 
@@ -155,3 +157,4 @@ main(int argc, char *argv[])
     plend();
     exit(0);
 }
+
