@@ -1,4 +1,4 @@
-/* $Id: x20c.c,v 1.15 2004/06/03 03:57:20 aroach Exp $
+/* $Id: x20c.c,v 1.18 2005/10/07 17:53:37 airwin Exp $
 
 	plimage demo
 
@@ -85,7 +85,7 @@ main(int argc, char *argv[])
   /* Parse and process command line arguments */
 
   plMergeOpts(options, "x20c options", NULL);
-  plParseOpts(&argc, argv, PL_PARSE_FULL);
+  plparseopts(&argc, argv, PL_PARSE_FULL);
 
   /* Initialize plplot */
 
@@ -121,12 +121,12 @@ main(int argc, char *argv[])
   if (!nosombrero) { 
     plAlloc2dGrid(&r, XDIM, YDIM);
     plcol0(2); /* draw a yellow plot box, useful for diagnostics! :( */
-    plenv(0., 2.*PI, 0, 3.*PI, 1, -1);
+    plenv(0., 2.*M_PI, 0, 3.*M_PI, 1, -1);
 
     for (i=0; i<XDIM; i++)
-      x[i] = i*2.*PI/(XDIM-1);
+      x[i] = i*2.*M_PI/(XDIM-1);
     for (i=0; i<YDIM; i++)
-      y[i] = i*3.*PI/(YDIM-1);
+      y[i] = i*3.*M_PI/(YDIM-1);
 
     for (i=0; i<XDIM; i++)
       for (j=0; j<YDIM; j++) {
@@ -136,8 +136,8 @@ main(int argc, char *argv[])
 
     pllab("No, an amplitude clipped \"sombrero\"", "", "Saturn?");
     plptex(2., 2., 3., 4., 0., "Transparent image");
-    plimage(z, XDIM, YDIM, 0., 2.*PI, 0, 3.*PI, 0.05, 1.,
-	    0., 2.*PI, 0, 3.*PI); 
+    plimage(z, XDIM, YDIM, 0., 2.*M_PI, 0, 3.*M_PI, 0.05, 1.,
+	    0., 2.*M_PI, 0, 3.*M_PI); 
     plFree2dGrid(r, XDIM, YDIM);
 
     /* save the plot */
@@ -272,7 +272,7 @@ int read_img(char *fname, PLFLT ***img_f, int *width, int *height, int *num_col)
 /* save plot */
 void save_plot(char *fname)   
 {
-  int cur_strm, new_strm; 
+  PLINT cur_strm, new_strm; 
 
   plgstrm(&cur_strm); /* get current stream */
   plmkstrm(&new_strm); /* create a new one */ 
@@ -292,7 +292,7 @@ int get_clip(PLFLT *xi, PLFLT *xe, PLFLT *yi, PLFLT *ye)
 {
   PLGraphicsIn gin;
   PLFLT xxi=*xi, yyi=*yi, xxe=*xe, yye=*ye, t;
-  int st, start = 0;
+  PLINT st, start = 0;
 
   plxormod(1, &st); /* enter xor mode to draw a selection rectangle */
 
