@@ -1,5 +1,5 @@
 //---------------------------------------------------------------------------//
-// $Id: x01.java,v 1.21 2004/06/17 12:41:02 andrewross Exp $
+// $Id: x01.java,v 1.30 2006/04/06 22:05:40 andrewross Exp $
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -22,7 +22,7 @@
 //
 // You should have received a copy of the GNU Library General Public License
 // along with PLplot; if not, write to the Free Software
-// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA
+// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 //---------------------------------------------------------------------------//
 
 //---------------------------------------------------------------------------//
@@ -38,8 +38,14 @@ import java.lang.Math;
 class x01 {
 
     double xscale, yscale, xoff, yoff;
-    PLStreamc plsdummy = new PLStreamc();
-    plplotjavac pls = new plplotjavac();
+    PLStream pls = new PLStream();
+
+    // Set this to 1 to test the xormod method
+    static boolean test_xor = false;
+
+    static boolean locate_mode = false;
+    static int fontset = 1;
+    static String f_name = null;
 
    public static void main( String[] args ) 
      {
@@ -52,25 +58,28 @@ class x01 {
     // plplot initialization
     // Divide page into 2x2 plots unless user overrides.
 
-        pls.plssub(2, 2);
+        pls.ssub(2, 2);
 
     // Parse and process command line arguments.
 
 //         plMergeOpts(options, "x01c options", notes);
-        pls.plParseOpts( args, pls.PL_PARSE_FULL|pls.PL_PARSE_NOPROGRAM );
+        pls.parseopts( args, PLStream.PL_PARSE_FULL|PLStream.PL_PARSE_NOPROGRAM );
 
 // Print out version number.
 
 	StringBuffer version = new StringBuffer(80);
 	// plgver no longer works for unexplained reasons.
-	pls.plgver(version);
+	pls.gver(version);
 	System.out.println("PLplot library version: " + version);
 
     // Initialize PLplot.
-        pls.plinit();
+        pls.init();
 
     // Select the multi-stroke font.
-        pls.plfontld( 1 );
+        if (fontset == 1) 
+            pls.fontld( 1 );
+	else
+            pls.fontld( 0 );
 
     // Set up the data
     // Original case
@@ -81,7 +90,7 @@ class x01 {
         yoff = 0.;
 
     // Do a plot
-        plot1(0);
+        plot1(false);
 
     // Set up the data
 
@@ -92,14 +101,17 @@ class x01 {
     // Do a plot
 
         int digmax = 5;
-        pls.plsyax(digmax, 0);
+        pls.syax(digmax, 0);
 
-        plot1(1);
+        plot1(true);
 
         plot2();
 
         plot3();
 
+	// Test option to save file
+	if (f_name != null) {
+	}
     // Let's get some user input
 
 //         if (locate_mode) {
@@ -120,10 +132,10 @@ class x01 {
 //         }
 
     // Don't forget to call plend() to finish off!
-        pls.plend();
+        pls.end();
     }
 
-    void plot1( int do_test )
+    void plot1( boolean do_test )
     {
         int i;
         double xmin, xmax, ymin, ymax;
@@ -131,6 +143,9 @@ class x01 {
         double y[] = new double[60];
         double xs[] = new double[6];
         double ys[] = new double[6];
+	boolean st[] = new boolean[1];
+	double xx[] = new double[1];
+	double yy[] = new double[1];
 
         for( i=0; i < 60; i++ )
         {
@@ -153,20 +168,43 @@ class x01 {
     // 6.0, and the range in Y is 0.0 to 30.0. The axes are scaled separately
     // (just = 0), and we just draw a labelled box (axis = 0).
 
-        pls.plcol0(1);
-        pls.plenv( xmin, xmax, ymin, ymax, 0, 0 );
-        pls.plcol0(2);
-        pls.pllab( "(x)", "(y)", "#frPLplot Example 1 - y=x#u2" );
+        pls.col0(1);
+        pls.env( xmin, xmax, ymin, ymax, 0, 0 );
+        pls.col0(2);
+        pls.lab( "(x)", "(y)", "#frPLplot Example 1 - y=x#u2" );
 
     // Plot the data points.
 
-        pls.plcol0(4);
-        pls.plpoin( xs, ys, 9 );
+        pls.col0(4);
+        pls.poin( xs, ys, 9 );
 
     // Draw the line through the data.
 
-        pls.plcol0(3);
-        pls.plline(x, y);
+        pls.col0(3);
+        pls.line(x, y);
+
+        // xor mode enable erasing a line/point/text by replotting it again
+        // it does not work in double buffering mode, however
+
+        if (do_test && test_xor ) {
+            pls.xormod(true, st); // enter xor mode
+            if (st[0]) {
+                for (i=0; i<60; i++) {
+		    xx[0] = x[i];
+		    yy[0] = y[i];
+                    pls.poin(xx, yy,9);    // draw a point
+		    try {
+                        Thread.sleep(50);           // wait a little
+		    }
+		    catch (InterruptedException ie) {
+		    }
+                    pls.flush();        // force an update of the tk driver
+                    pls.poin(xx, yy,9);    // erase point
+                }
+                pls.xormod(false, st);       // leave xor mode
+            }
+        }
+
     }
 
     void plot2()
@@ -179,10 +217,10 @@ class x01 {
     // 10.0, and the range in Y is -0.4 to 2.0. The axes are scaled
     // separately (just = 0), and we draw a box with axes (axis = 1).
 
-        pls.plcol0(1);
-        pls.plenv(-2.0, 10.0, -0.4, 1.2, 0, 1);
-        pls.plcol0(2);
-        pls.pllab("(x)", "sin(x)/x", "#frPLplot Example 1 - Sinc Function");
+        pls.col0(1);
+        pls.env(-2.0, 10.0, -0.4, 1.2, 0, 1);
+        pls.col0(2);
+        pls.lab("(x)", "sin(x)/x", "#frPLplot Example 1 - Sinc Function");
 
     // Fill up the arrays.
 
@@ -195,10 +233,10 @@ class x01 {
 
     // Draw the line.
 
-        pls.plcol0(3);
-        pls.plwid(2);
-        pls.plline(x, y);
-        pls.plwid(1);
+        pls.col0(3);
+        pls.wid(2);
+        pls.line(x, y);
+        pls.wid(1);
     }
 
     void plot3()
@@ -214,29 +252,29 @@ class x01 {
     // For the final graph we wish to override the default tick intervals,
     // and so do not use plenv().
 
-        pls.pladv(0);
+        pls.adv(0);
 
     // Use standard viewport, and define X range from 0 to 360 degrees, Y
     // range from -1.2 to 1.2.
 
-        pls.plvsta();
-        pls.plwind( 0.0, 360.0, -1.2, 1.2 );
+        pls.vsta();
+        pls.wind( 0.0, 360.0, -1.2, 1.2 );
 
     // Draw a box with ticks spaced 60 degrees apart in X, and 0.2 in Y.
 
-        pls.plcol0(1);
-        pls.plbox("bcnst", 60.0, 2, "bcnstv", 0.2, 2);
+        pls.col0(1);
+        pls.box("bcnst", 60.0, 2, "bcnstv", 0.2, 2);
 
     // Superimpose a dashed line grid, with 1.5 mm marks and spaces. 
     // plstyl expects a pointer!
 
-        pls.plstyl(mark1, space1);
-        pls.plcol0(2);
-        pls.plbox("g", 30.0, 0, "g", 0.2, 0);
-        pls.plstyl(mark0, space0);
+        pls.styl(mark1, space1);
+        pls.col0(2);
+        pls.box("g", 30.0, 0, "g", 0.2, 0);
+        pls.styl(mark0, space0);
 
-        pls.plcol0(3);
-        pls.pllab( "Angle (degrees)", "sine",
+        pls.col0(3);
+        pls.lab( "Angle (degrees)", "sine",
                  "#frPLplot Example 1 - Sine function" );
 
         for (i = 0; i < 101; i++) {
@@ -244,8 +282,8 @@ class x01 {
             y[i] = Math.sin(x[i] * Math.PI / 180.0);
         }
 
-        pls.plcol0(4);
-        pls.plline(x, y);
+        pls.col0(4);
+        pls.line(x, y);
     }
 }
 
