@@ -1,9 +1,10 @@
-/* $Id: plcore.h,v 1.68 2004/06/14 21:51:57 rlaboiss Exp $
+/* $Id: plcore.h,v 1.77 2006/04/17 11:54:12 andrewross Exp $
 
 	Contains declarations for core plplot data structures.  This file
 	should be included only by plcore.c.
 
     Copyright (C) 2004  Andrew Roach
+    Copyright (C) 2005  Thomas J. Duck
 
     This file is part of PLplot.
 
@@ -19,7 +20,7 @@
 
     You should have received a copy of the GNU Library General Public License
     along with PLplot; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 */
 
@@ -37,11 +38,15 @@ typedef lt_ptr (*PLDispatchInit)( PLDispatchTable *pdt );
 #else
 typedef void (*PLDispatchInit)( PLDispatchTable *pdt );
 #endif
-    
+
+#ifdef HAVE_LIBUNICODE
+#include <unicode.h>
+#endif
 
 
 /* Static function prototypes */
 
+static char	*utf8_to_ucs4	(const char *ptr, PLUNICODE *unichar);
 static void	grline		(short *, short *, PLINT);
 static void	grpolyline	(short *, short *, PLINT);
 static void	grfill		(short *, short *, PLINT);
@@ -91,7 +96,7 @@ PLStream *plsc = &pls0;
  * Initialize dispatch table.
  *
  * Each device is selected by the appropriate define, passed in from the
- * makefile.  When installing plplot you may wish to exclude devices not 
+ * makefile.  When installing plplot you may wish to exclude devices not
  * present on your system in order to reduce screen clutter.
  *
  * If you hit a <CR> in response to the plinit() prompt, you get the FIRST
@@ -120,6 +125,9 @@ static PLDispatchInit static_device_initializers[] = {
 #if defined(PLD_gnome) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_gnome,
 #endif
+#if defined(PLD_gcw) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_gcw,
+#endif
 #if defined(PLD_tk) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_tk,
 #endif
@@ -220,6 +228,9 @@ static PLDispatchInit static_device_initializers[] = {
 #if defined(PLD_jpeg) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_jpeg,
 #endif
+#if defined(PLD_gif) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_gif,
+#endif
 #if defined(PLD_pstex) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_pstex,
 #endif
@@ -240,6 +251,16 @@ static PLDispatchInit static_device_initializers[] = {
 #endif
 #if defined(PLD_wingcc) && !defined(ENABLE_DYNDRIVERS)
     plD_dispatch_init_wingcc,
+#endif
+#if defined(PLD_aqt) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_aqt,
+#endif
+#if defined(PLD_wxwidgets) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_wxwidgets,
+#endif
+#if defined(PLD_psttf) && !defined(ENABLE_DYNDRIVERS)
+    plD_dispatch_init_psttfm,
+    plD_dispatch_init_psttfc,
 #endif
     NULL
 };
@@ -264,7 +285,7 @@ typedef struct {
 typedef struct {
     char *drvnam;
     lt_dlhandle dlhand;
-    
+
 } PLLoadableDriver;
 
 static PLLoadableDevice *loadable_device_list;
