@@ -1,4 +1,4 @@
-/* $Id: plplot.h,v 1.154 2004/06/14 21:51:57 rlaboiss Exp $
+/* $Id: plplot.h,v 1.173 2006/04/08 16:36:36 hbabcock Exp $
 
     Macros and prototypes for the PLplot package.  This header file must
     be included by all user codes.
@@ -27,7 +27,7 @@
 
     You should have received a copy of the GNU Library General Public License
     along with PLplot; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #ifndef __PLPLOT_H__
@@ -115,16 +115,19 @@
  * Only those that are necessary for function prototypes are defined here.
  * Notes:
  *
- * PLINT is typedef'd to an int by default.  This is a change from some
- * previous versions, where a long was used.  Under MSDOS, a PLINT is
- * typedef'd to a long, since 16 bits is too inaccurate for some PLplot
- * functions.  So under MSDOS you must use type PLINT for integer array
- * arguments to PLplot functions, but on other systems you can just use
- * an int.
+ * PLINT is typedef'd to a long by default.  This is a change from some
+ * previous versions, where a int was used.  However, so long as you have
+ * used type PLINT for integer array arguments as specified by the API,
+ * this change will be transparent for you.
  *
  * short is currently used for device page coordinates, so they are
  * bounded by (-32767, 32767).  This gives a max resolution of about 3000
  * dpi, and improves performance in some areas over using a PLINT.
+ *
+ * PLUNICODE should be a 32-bit unsigned integer on all platforms.
+ * For now, we are using unsigned int for our Linux ix86 unicode experiments,
+ * but that doesn't guarantee 32 bits exactly on all platforms so this will
+ * be subject to change.
 \*--------------------------------------------------------------------------*/
 
 #if defined(PL_DOUBLE) || defined(DOUBLE)
@@ -137,12 +140,21 @@ typedef float PLFLT;
 #define PLFLT_MIN  FLT_MIN
 #endif
 
-#if defined(MSDOS) && !defined(__WIN32__)
-typedef long PLINT;
+#if (defined(HAVE_STDINT_H) && !defined(__cplusplus)) ||\
+(defined(__cplusplus) && defined(HAVE_CXX_STDINT_H))
+#include <stdint.h>
+/* This is apparently portable if stdint.h exists. */
+typedef uint32_t PLUNICODE;
+typedef int32_t PLINT;
 #else
+/* A reasonable back-up in case stdint.h does not exist on the platform. */
+typedef unsigned int PLUNICODE;
 typedef int PLINT;
 #endif
 
+/* For identifying logical (boolean) arguments */
+typedef PLINT PLBOOL;
+
 /* For passing user data, as with X's XtPointer */
 
 typedef void* PLPointer;
@@ -225,11 +237,34 @@ typedef void* PLPointer;
 #define PL_PARSE_NODASH		0x0040	/* Set if leading dash NOT required */
 #define PL_PARSE_SKIP		0x0080	/* Skip over unrecognized args */
 
+/* FCI (font characterization integer) related constants. */
+#define PL_FCI_MARK 0x80000000
+#define PL_FCI_IMPOSSIBLE 0x00000000
+#define PL_FCI_HEXDIGIT_MASK 0xf
+#define PL_FCI_HEXPOWER_MASK 0x7
+#define PL_FCI_HEXPOWER_IMPOSSIBLE 0xf
+/* These define hexpower values corresponding to each font attribute. */
+#define PL_FCI_FAMILY  0x0
+#define PL_FCI_STYLE  0x1
+#define PL_FCI_WEIGHT  0x2
+/* These are legal values for font family attribute */
+#define PL_FCI_SANS 0x0
+#define PL_FCI_SERIF 0x1
+#define PL_FCI_MONO 0x2
+#define PL_FCI_SCRIPT 0x3
+#define PL_FCI_SYMBOL 0x4
+/* These are legal values for font style attribute */
+#define PL_FCI_UPRIGHT 0x0
+#define PL_FCI_ITALIC 0x1
+#define PL_FCI_OBLIQUE 0x2
+/* These are legal values for font weight attribute */
+#define PL_FCI_MEDIUM 0x0
+#define PL_FCI_BOLD 0x1
+
 /* Obsolete names */
 
-#define plParseInternalOpts(a, b, c)	plParseOpts(a, b, c)
+#define plParseInternalOpts(a, b, c)	c_plparseopts(a, b, c)
 #define plSetInternalOpt(a, b)		plSetOpt(a, b)
-
 /* Option table definition */
 
 typedef struct {
@@ -460,6 +495,7 @@ typedef struct {
 #define    plgdiori	c_plgdiori
 #define    plgdiplt	c_plgdiplt
 #define    plgfam	c_plgfam
+#define    plgfci	c_plgfci
 #define    plgfnam	c_plgfnam
 #define    plglevel	c_plglevel
 #define    plgpage	c_plgpage
@@ -475,6 +511,7 @@ typedef struct {
 #define    plgzax	c_plgzax
 #define    plhist	c_plhist
 #define    plhls        c_plhls
+#define    plhlsrgb     c_plhlsrgb
 #define    plinit	c_plinit
 #define    pljoin	c_pljoin
 #define    pllab	c_pllab
@@ -482,6 +519,8 @@ typedef struct {
 #define    plline	c_plline
 #define    plline3	c_plline3
 #define    pllsty	c_pllsty
+#define    plmap	c_plmap
+#define    plmeridians	c_plmeridians
 #define    plmesh	c_plmesh
 #define    plmeshc	c_plmeshc
 #define    plmkstrm	c_plmkstrm
@@ -489,6 +528,7 @@ typedef struct {
 #define    plot3d	c_plot3d
 #define    plot3dc	c_plot3dc
 #define    plot3dcl	c_plot3dcl
+#define    plparseopts	c_plparseopts
 #define    plpat	c_plpat
 #define    plpoin	c_plpoin
 #define    plpoin3	c_plpoin3
@@ -499,6 +539,7 @@ typedef struct {
 #define    plreplot	c_plreplot
 #define    plrgb	c_plrgb
 #define    plrgb1	c_plrgb1
+#define    plrgbhls     c_plrgbhls
 #define    plschr	c_plschr
 #define    plscmap0	c_plscmap0
 #define    plscmap0n	c_plscmap0n
@@ -518,6 +559,7 @@ typedef struct {
 #define    plsesc	c_plsesc
 #define    plsetopt	c_plsetopt
 #define    plsfam	c_plsfam
+#define    plsfci	c_plsfci
 #define    plsfnam	c_plsfnam
 #define    plshade	c_plshade
 #define    plshade1	c_plshade1
@@ -612,11 +654,11 @@ plarrows(PLFLT *u, PLFLT *v, PLFLT *x, PLFLT *y, PLINT n,
 
 void
 c_plvect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale,
-void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
-              PLPointer pltr_data);
+	void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+       	PLPointer pltr_data);
 
 void
-c_plsvect(PLFLT *arrowx, PLFLT *arrowy, PLINT npts, PLINT fill);
+c_plsvect(PLFLT *arrowx, PLFLT *arrowy, PLINT npts, PLBOOL fill);
 
 /* This functions similarly to plbox() except that the origin of the axes */
 /* is placed at the user-specified point (x0, y0). */
@@ -627,8 +669,14 @@ c_plaxes(PLFLT x0, PLFLT y0, const char *xopt, PLFLT xtick, PLINT nxsub,
 
 /* Plot a histogram using x to store data values and y to store frequencies */
 
+/* Flags for plbin() - opt argument */
+#define PL_BIN_DEFAULT          0
+#define PL_BIN_CENTRED          1
+#define PL_BIN_NOEXPAND         2
+#define PL_BIN_NOEMPTY          4
+
 void
-c_plbin(PLINT nbin, PLFLT *x, PLFLT *y, PLINT center);
+c_plbin(PLINT nbin, PLFLT *x, PLFLT *y, PLINT opt);
 
 /* Start new page.  Should only be used with pleop(). */
 
@@ -694,7 +742,7 @@ plfcont(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 /* Copies state parameters from the reference stream to the current stream. */
 
 void
-c_plcpstrm(PLINT iplsr, PLINT flags);
+c_plcpstrm(PLINT iplsr, PLBOOL flags);
 
 /* Converts input values from relative device coordinates to relative plot */
 /* coordinates. */
@@ -817,6 +865,11 @@ c_plgdiori(PLFLT *p_rot);
 void
 c_plgdiplt(PLFLT *p_xmin, PLFLT *p_ymin, PLFLT *p_xmax, PLFLT *p_ymax);
 
+/* Get FCI (font characterization integer) */
+
+void
+c_plgfci(PLUNICODE *pfci);
+
 /* Get family file parameters */
 
 void
@@ -901,15 +954,28 @@ c_plgzax(PLINT *p_digmax, PLINT *p_digits);
 
 /* Draws a histogram of n values of a variable in array data[0..n-1] */
 
+/* Flags for plhist() - opt argument; note: some flags are passed to
+   plbin() for the actual plotting */
+#define PL_HIST_DEFAULT         0
+#define PL_HIST_NOSCALING       1
+#define PL_HIST_IGNORE_OUTLIERS 2
+#define PL_HIST_NOEXPAND        8
+#define PL_HIST_NOEMPTY        16
+
 void
 c_plhist(PLINT n, PLFLT *data, PLFLT datmin, PLFLT datmax,
-	 PLINT nbin, PLINT oldwin);
+	 PLINT nbin, PLINT opt);
 
 /* Set current color (map 0) by hue, lightness, and saturation. */
 
 void
 c_plhls(PLFLT h, PLFLT l, PLFLT s);
 
+/* Functions for converting between HLS and RGB color space */
+
+void
+c_plhlsrgb(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b);
+
 /* Initializes PLplot, using preset or default options */
 
 void
@@ -947,13 +1013,13 @@ c_pllsty(PLINT lin);
 /* plot continental outline in world coordinates */
 
 void
-plmap( void (*mapform)(PLINT, PLFLT *, PLFLT *), char *type,
+c_plmap( void (*mapform)(PLINT, PLFLT *, PLFLT *), char *type,
          PLFLT minlong, PLFLT maxlong, PLFLT minlat, PLFLT maxlat );
 
 /* Plot the latitudes and longitudes on the background. */
 
 void
-plmeridians( void (*mapform)(PLINT, PLFLT *, PLFLT *),
+c_plmeridians( void (*mapform)(PLINT, PLFLT *, PLFLT *),
                PLFLT dlong, PLFLT dlat,
                PLFLT minlong, PLFLT maxlong, PLFLT minlat, PLFLT maxlat );
 
@@ -998,7 +1064,7 @@ c_plot3dc(PLFLT *x, PLFLT *y, PLFLT **z,
 void
 c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
 	 PLINT nx, PLINT ny, PLINT opt,
-	 PLFLT *clevel, PLINT nlevel, 
+	 PLFLT *clevel, PLINT nlevel,
 	 PLINT ixstart, PLINT ixn, PLINT *indexymin, PLINT*indexymax);
 
 /*
@@ -1047,7 +1113,7 @@ c_plpoin3(PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT code);
 /* Draws a polygon in 3 space.  */
 
 void
-c_plpoly3(PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, PLINT ifcc);
+c_plpoly3(PLINT n, PLFLT *x, PLFLT *y, PLFLT *z, PLINT *draw, PLBOOL ifcc);
 
 /* Set the floating point precision (in number of places) in numeric labels. */
 
@@ -1079,6 +1145,11 @@ c_plrgb(PLFLT r, PLFLT g, PLFLT b);
 void
 c_plrgb1(PLINT r, PLINT g, PLINT b);
 
+/* Functions for converting between HLS and RGB color space */
+
+void
+c_plrgbhls(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);
+
 /* Set character height. */
 
 void
@@ -1103,8 +1174,8 @@ c_plscmap1(PLINT *r, PLINT *g, PLINT *b, PLINT ncol1);
 /* intensity [0,1] (cmap 1 index) and position in HLS or RGB color space. */
 
 void
-c_plscmap1l(PLINT itype, PLINT npts, PLFLT *intensity,
-	    PLFLT *coord1, PLFLT *coord2, PLFLT *coord3, PLINT *rev);
+c_plscmap1l(PLBOOL itype, PLINT npts, PLFLT *intensity,
+	    PLFLT *coord1, PLFLT *coord2, PLFLT *coord3, PLBOOL *rev);
 
 /* Set number of colors in cmap 1 */
 
@@ -1173,6 +1244,11 @@ c_plsesc(char esc);
 void
 c_plsfam(PLINT fam, PLINT num, PLINT bmax);
 
+/* Set FCI (font characterization integer) */
+
+void
+c_plsfci(PLUNICODE fci);
+
 /* Set the output file name. */
 
 void
@@ -1187,7 +1263,7 @@ c_plshade(PLFLT **a, PLINT nx, PLINT ny, PLINT (*defined) (PLFLT, PLFLT),
 	  PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 	  PLINT min_color, PLINT min_width,
 	  PLINT max_color, PLINT max_width,
-	  void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+	  void (*fill) (PLINT, PLFLT *, PLFLT *), PLBOOL rectangular,
 	  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	  PLPointer pltr_data);
 
@@ -1198,7 +1274,7 @@ c_plshade1(PLFLT *a, PLINT nx, PLINT ny, PLINT (*defined) (PLFLT, PLFLT),
 	 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 	 PLINT min_color, PLINT min_width,
 	 PLINT max_color, PLINT max_width,
-	 void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+	 void (*fill) (PLINT, PLFLT *, PLFLT *), PLBOOL rectangular,
 	 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	 PLPointer pltr_data);
 
@@ -1207,7 +1283,7 @@ c_plshades( PLFLT **a, PLINT nx, PLINT ny, PLINT (*defined) (PLFLT, PLFLT),
 	  PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax,
 	  PLFLT *clevel, PLINT nlevel, PLINT fill_width,
 	  PLINT cont_color, PLINT cont_width,
-	  void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+	  void (*fill) (PLINT, PLFLT *, PLFLT *), PLBOOL rectangular,
 	  void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	  PLPointer pltr_data);
 
@@ -1222,7 +1298,7 @@ plfshade(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 	 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
 	 PLINT min_color, PLINT min_width,
 	 PLINT max_color, PLINT max_width,
-	 void (*fill) (PLINT, PLFLT *, PLFLT *), PLINT rectangular,
+	 void (*fill) (PLINT, PLFLT *, PLFLT *), PLBOOL rectangular,
 	 void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	 PLPointer pltr_data);
 
@@ -1255,7 +1331,7 @@ c_plspage(PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng,
 /* Set the pause (on end-of-page) status */
 
 void
-c_plspause(PLINT pause);
+c_plspause(PLBOOL pause);
 
 /* Set stream number.  */
 
@@ -1293,7 +1369,7 @@ void
 c_plstripc(PLINT *id, char *xspec, char *yspec,
 	PLFLT xmin, PLFLT xmax, PLFLT xjump, PLFLT ymin, PLFLT ymax,
 	PLFLT xlpos, PLFLT ylpos,
-	PLINT y_ascl, PLINT acc,
+	PLBOOL y_ascl, PLBOOL acc,
 	PLINT colbox, PLINT collab,
 	PLINT colline[], PLINT styline[], char *legline[],
 	char *labx, char *laby, char *labtop);
@@ -1407,7 +1483,7 @@ c_plwind(PLFLT xmin, PLFLT xmax, PLFLT ymin, PLFLT ymax);
 /*  set xor mode; mode = 1-enter, 0-leave, status = 0 if not interactive device  */
 
 void
-c_plxormod(PLINT mode, PLINT *status);
+c_plxormod(PLBOOL mode, PLBOOL *status);
 
 /*--------------------------------------------------------------------------*\
  *		Functions for use from C or C++ only
@@ -1494,11 +1570,6 @@ pltr0f(PLFLT x, PLFLT y, PLFLT *tx, PLFLT *ty, void *pltr_data);
 void
 pltr2f(PLFLT x, PLFLT y, PLFLT *tx, PLFLT *ty, void *pltr_data);
 
-/* Example linear transformation function for contour plotter. */
-
-void
-xform(PLFLT x, PLFLT y, PLFLT * tx, PLFLT * ty);
-
 	/* Function evaluators */
 
 /* Does a lookup from a 2d function array.  Array is of type (PLFLT **), */
@@ -1554,7 +1625,7 @@ plSetOpt(char *opt, char *optarg);
 /* Process options list using current options info. */
 
 int
-plParseOpts(int *p_argc, char **argv, PLINT mode);
+c_plparseopts(int *p_argc, char **argv, PLINT mode);
 
 /* Print usage & syntax message. */
 
@@ -1626,14 +1697,6 @@ plFree2dGrid(PLFLT **f, PLINT nx, PLINT ny);
 void
 plMinMax2dGrid(PLFLT **f, PLINT nx, PLINT ny, PLFLT *fmax, PLFLT *fmin);
 
-/* Functions for converting between HLS and RGB color space */
-
-void
-plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b);
-
-void
-plRGB_HLS(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);
-
 /* Wait for graphics input event and translate to world coordinates */
 
 int
@@ -1644,6 +1707,17 @@ plGetCursor(PLGraphicsIn *gin);
 int
 plTranslateCursor(PLGraphicsIn *gin);
 
+/* Deprecated function names which are handled as wrappers for strict
+ * backwards compatibility of the library API*/
+
+int
+plParseOpts(int *p_argc, char **argv, PLINT mode);
+
+void
+plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b);
+
+void
+plRGB_HLS(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);
 #ifdef __cplusplus
 }
 #endif
