@@ -1,4 +1,4 @@
-/* $Id: plargs.c,v 1.68 2004/06/14 21:51:57 rlaboiss Exp $
+/* $Id: plargs.c,v 1.74 2005/11/16 18:24:43 airwin Exp $
 
     Copyright 1993, 1994, 1995
     Maurice LeBrun			mjl@dino.ph.utexas.edu
@@ -21,7 +21,7 @@
 
     You should have received a copy of the GNU Library General Public License
     along with PLplot; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
     Some parts of this code were derived from "xterm.c" and "ParseCmd.c" of
     the X-windows Version 11 distribution.  The copyright notice is
@@ -41,7 +41,7 @@ and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
     structure, along with strings giving the syntax, long help message, and
     option handler.
 
-    The command line parser -- plParseOpts() -- removes all recognized flags
+    The command line parser -- plparseopts() -- removes all recognized flags
     (decreasing argc accordingly), so that invalid input may be readily
     detected.  It can also be used to process user command line flags.  The
     user can merge an option table of type PLOptionTable into the internal
@@ -49,7 +49,7 @@ and the Massachusetts Institute of Technology, Cambridge, Massachusetts.
     specify that ONLY the external table(s) be parsed by calling
     plClearOpts() before plMergeOpts().
 
-    The default action taken by plParseOpts() is as follows:
+    The default action taken by plparseopts() is as follows:
 	- Returns with an error if an unrecognized option or badly formed
 	  option-value pair are encountered.
 	- Returns immediately (return code 0) when the first non-option
@@ -147,9 +147,6 @@ static int opt_tcl_cmd		(char *, char *, void *);
 static int opt_auto_path	(char *, char *, void *);
 static int opt_bufmax		(char *, char *, void *);
 static int opt_server_name	(char *, char *, void *);
-static int opt_server_host	(char *, char *, void *);
-static int opt_server_port	(char *, char *, void *);
-static int opt_user		(char *, char *, void *);
 static int opt_tk_file          (char *, char *, void *);
 static int opt_dpi		(char *, char *, void *);
 static int opt_dev_compression	(char *, char *, void *);
@@ -407,7 +404,7 @@ static PLOptionTable ploption_table[] = {
     NULL,
     PL_OPT_FUNC | PL_OPT_ARG,
     "-bg color",
-    "Background color (0=black, FFFFFF=white)" },
+    "Background color (000000=black, FFFFFF=white)" },
 {
     "ncol0",			/* Allocated colors in cmap 0 */
     opt_ncol0,
@@ -504,30 +501,6 @@ static PLOptionTable ploption_table[] = {
     PL_OPT_FUNC | PL_OPT_ARG,
     "-server_name name",
     "Main window name of PLplot server (tk driver)" },
-{
-    "server_host",		/* Host to run server on */
-    opt_server_host,
-    NULL,
-    NULL,
-    PL_OPT_FUNC | PL_OPT_ARG,
-    "-server_host name",
-    "Host to run PLplot server on (dp driver)" },
-{
-    "server_port",		/* Port to talk to server on */
-    opt_server_port,
-    NULL,
-    NULL,
-    PL_OPT_FUNC | PL_OPT_ARG,
-    "-server_port name",
-    "Port to talk to PLplot server on (dp driver)" },
-{
-    "user",			/* user name on remote node */
-    opt_user,
-    NULL,
-    NULL,
-    PL_OPT_FUNC | PL_OPT_ARG,
-    "-user name",
-    "User name on remote node (dp driver)" },
 {
     "plserver",			/* PLplot server name */
     opt_plserver,
@@ -535,7 +508,7 @@ static PLOptionTable ploption_table[] = {
     NULL,
     PL_OPT_FUNC | PL_OPT_ARG | PL_OPT_INVISIBLE,
     "-plserver name",
-    "Invoked name of PLplot server (tk or dp driver)" },
+    "Invoked name of PLplot server (tk driver)" },
 {
     "plwindow",			/* PLplot container window name */
     opt_plwindow,
@@ -543,7 +516,7 @@ static PLOptionTable ploption_table[] = {
     NULL,
     PL_OPT_FUNC | PL_OPT_ARG | PL_OPT_INVISIBLE,
     "-plwindow name",
-    "Name of PLplot container window (tk or dp driver)" },
+    "Name of PLplot container window (tk driver)" },
 {
     "tcl_cmd",			/* TCL initialization command */
     opt_tcl_cmd,
@@ -559,7 +532,7 @@ static PLOptionTable ploption_table[] = {
     NULL,
     PL_OPT_FUNC | PL_OPT_ARG | PL_OPT_INVISIBLE,
     "-auto_path dir",
-    "Additional directory(s) to autoload (tk or dp driver)" },
+    "Additional directory(s) to autoload (tk driver)" },
 {
     "tk_file",      /* -file option for plserver */
     opt_tk_file,
@@ -567,7 +540,7 @@ static PLOptionTable ploption_table[] = {
     NULL,
     PL_OPT_FUNC | PL_OPT_ARG | PL_OPT_INVISIBLE,
     "-tk_file file",
-    "file for plserver (tk or dp driver)" },
+    "file for plserver (tk driver)" },
 {
     "dpi",			/* Dots per inch */
     opt_dpi,
@@ -683,7 +656,7 @@ plSetOpt(char *opt, char *optarg)
 	PL_PARSE_NOPROGRAM |
 	PL_PARSE_NODASH;
 
-    status = plParseOpts(&argc, argv, mode);
+    status = plparseopts(&argc, argv, mode);
     if (status) {
 	fprintf( stderr, "plSetOpt: Unrecognized option %s\n", opt);
     }
@@ -761,7 +734,7 @@ plResetOpts(void)
 }
 
 /*--------------------------------------------------------------------------*\
- * plParseOpts()
+ * plparseopts()
  *
  * Process options list using current ploptions_info structure.
  * An error in parsing the argument list causes a program exit if
@@ -769,7 +742,7 @@ plResetOpts(void)
 \*--------------------------------------------------------------------------*/
 
 int
-plParseOpts(int *p_argc, char **argv, PLINT mode)
+c_plparseopts(int *p_argc, char **argv, PLINT mode)
 {
     char **argsave, **argend;
     int	i, myargc, status = 0;
@@ -1328,6 +1301,35 @@ plHelpDrvOpts(DrvOpt *acc_opt) {
   }
 }
 
+/*--------------------------------------------------------------------------*\
+ * tidyDrvOpts
+ *
+ * Tidy up and free memory associated with driver options
+\*--------------------------------------------------------------------------*/
+
+void
+plP_FreeDrvOpts() {
+  DrvOptCmd *drvp, *drvpl;
+
+  if (!drv_opt.option) 
+    return;
+
+  drvp = &drv_opt;
+  do {
+    drvpl = drvp;
+    drvp = drvpl->next;
+
+    free(drvpl->option);
+    free(drvpl->value);
+    /* Free additional DrvOptCmd variables - 
+     * first entry in list is a static global variable */ 
+    if (drvpl != &drv_opt)
+      free(drvpl);
+
+  } while(drvp != NULL);
+}
+
+
 /*--------------------------------------------------------------------------*\
  * Option handlers
 \*--------------------------------------------------------------------------*/
@@ -1944,7 +1946,7 @@ opt_bufmax(char *opt, char *optarg, void *client_data)
  * opt_server_name()
  *
  * Performs appropriate action for option "server_name":
- * Sets main window name of server (Tcl/TK/DP driver only)
+ * Sets main window name of server (Tcl/TK driver only)
 \*--------------------------------------------------------------------------*/
 
 static int
@@ -1954,53 +1956,11 @@ opt_server_name(char *opt, char *optarg, void *client_data)
     return 0;
 }
 
-/*--------------------------------------------------------------------------*\
- * opt_server_host()
- *
- * Performs appropriate action for option "server_host":
- * Sets host to run server on (Tcl/TK/DP driver only)
-\*--------------------------------------------------------------------------*/
-
-static int
-opt_server_host(char *opt, char *optarg, void *client_data)
-{
-    plsc->server_host = plstrdup(optarg);
-    return 0;
-}
-
-/*--------------------------------------------------------------------------*\
- * opt_server_port()
- *
- * Performs appropriate action for option "server_port":
- * Sets port to talk to server on (Tcl/TK/DP driver only)
-\*--------------------------------------------------------------------------*/
-
-static int
-opt_server_port(char *opt, char *optarg, void *client_data)
-{
-    plsc->server_port = plstrdup(optarg);
-    return 0;
-}
-
-/*--------------------------------------------------------------------------*\
- * opt_user()
- *
- * Performs appropriate action for option "user":
- * Sets user name on remote node (for remsh), dp driver only
-\*--------------------------------------------------------------------------*/
-
-static int
-opt_user(char *opt, char *optarg, void *client_data)
-{
-    plsc->user = plstrdup(optarg);
-    return 0;
-}
-
 /*--------------------------------------------------------------------------*\
  * opt_plserver()
  *
  * Performs appropriate action for option "plserver":
- * Sets name to use when invoking server (Tcl/TK/DP driver only)
+ * Sets name to use when invoking server (Tcl/TK driver only)
 \*--------------------------------------------------------------------------*/
 
 static int
@@ -2228,3 +2188,5 @@ opt_dev_compression(char *opt, char *optarg, void *client_data)
 
     return 0;
 }
+
+
