@@ -1,4 +1,4 @@
-/* $Id: plcont.c,v 1.29 2004/05/19 22:51:11 airwin Exp $
+/* $Id: plcont.c,v 1.35 2005/12/06 02:24:11 airwin Exp $
 
 	Contour plotter.
 
@@ -21,11 +21,9 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
-#include <math.h>
-
 #include "plplotP.h"
 
 #ifdef MSDOS
@@ -297,7 +295,14 @@ static void pl_drawcontlabel(PLFLT tpx, PLFLT tpy, char *flabel, PLFLT *distance
 static void plfloatlabel(PLFLT value, char *string)
 {
     PLINT  setpre, precis;
-    char   form[10], tmpstring[10];
+    /* form[10] gives enough space for all non-malicious formats.
+     * tmpstring[15] gives enough room for 3 digits in a negative exponent
+     * or 4 digits in a positive exponent + null termination.  That
+     * should be enough for all non-malicious use.
+     * Obviously there are security issues here that
+     * should be addressed as well.
+     */
+    char   form[10], tmpstring[15];
     PLINT  exponent = 0;
     PLFLT  mant, tmp;
 
@@ -332,7 +337,6 @@ static void plfloatlabel(PLFLT value, char *string)
 
     sprintf(form, "%%.%df", prec-1);
     sprintf(string, form, mant);
-    /* sprintf(tmpstring, "#(229)10#u%d", exponent); */
     sprintf(tmpstring, "#(229)10#u%d", exponent);
     strcat(string, tmpstring);
 
@@ -497,7 +501,7 @@ plfcont(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 	void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	PLPointer pltr_data)
 {
-    PLINT i, j, **ipts;
+    PLINT i, **ipts;
 
     if (kx < 1 || kx >= lx) {
 	plabort("plfcont: indices must satisfy  1 <= kx <= lx <= nx");
@@ -548,8 +552,11 @@ plcntr(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 {
     PLINT kcol, krow, lastindex;
     PLFLT distance;
+    PLFLT save_def, save_scale;
 
     char  flabel[30];
+    plgchr(&save_def, &save_scale);
+    save_scale = save_scale/save_def;
 
     cont_new_store(flev);
 
@@ -581,7 +588,7 @@ plcntr(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 	}
 
     }
-    plschr(0.0, 1.0);
+    plschr(save_def, save_scale);
 }
 
 /*--------------------------------------------------------------------------*\
