@@ -1,4 +1,4 @@
-/* $Id: plcore.c,v 1.134 2004/06/14 21:51:57 rlaboiss Exp $
+/* $Id: plcore.c,v 1.168 2006/04/25 15:35:06 andrewross Exp $
 
 	Central dispatch facility for PLplot.
 	Also contains the PLplot main data structures, external access
@@ -8,8 +8,11 @@
 
 
   Copyright (C) 2004  Joao Cardoso
-  Copyright (C) 2004  Rafael Laboissiere
-  Copyright (C) 2004  Andrew Ross
+  Copyright (C) 2004, 2005  Rafael Laboissiere
+  Copyright (C) 2004, 2006  Andrew Ross
+  Copyright (C) 2004  Andrew Roach
+  Copyright (C) 2005  Alan W. Irwin
+  Copyright (C) 2005  Thomas J. Duck
 
   This file is part of PLplot.
 
@@ -25,7 +28,7 @@
 
   You should have received a copy of the GNU Library General Public License
   along with PLplot; if not, write to the Free Software
-  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 */
 
@@ -70,7 +73,13 @@
 enum {AT_BOP, DRAWING, AT_EOP};
 
 /* Initialize device. */
-/* The plot buffer must be called last */
+/* The plot buffer must be called last. */
+
+/* The following array of chars is used both here and in plsym.c for
+ * translating the Greek characters from the #g escape sequences into
+ * the Hershey and Unicode codings
+ */
+const char plP_greek_mnemonic[] = "ABGDEZYHIKLMNCOPRSTUFXQWabgdezyhiklmncoprstufxqw";
 
 void
 plP_init(void)
@@ -84,15 +93,15 @@ plP_init(void)
 }
 
 /* End of page */
-/* The plot buffer must be called first */
-/* Ignore instruction if there's nothing drawn */
+/* The plot buffer must be called first. */
+/* Ignore instruction if already at eop. */
 
 void
 plP_eop(void)
 {
     int skip_driver_eop = 0;
 
-    if (plsc->page_status != DRAWING)
+    if (plsc->page_status == AT_EOP)
 	return;
 
     plsc->page_status = AT_EOP;
@@ -110,8 +119,8 @@ plP_eop(void)
 }
 
 /* Set up new page. */
-/* The plot buffer must be called last */
-/* Ignore if the bop was already issued. */
+/* The plot buffer must be called last. */
+/* Ignore if already at bop. */
 /* It's not actually necessary to be AT_EOP here, so don't check for it. */
 
 void
@@ -163,10 +172,9 @@ plP_tidy(void)
 void
 plP_state(PLINT op)
 {
-    (*plsc->dispatch_table->pl_state) ((struct PLStream_struct *) plsc, op);
+    if (plsc->plbuf_write) plbuf_state(plsc, op);
 
-    if (plsc->plbuf_write)
-	plbuf_state(plsc, op);
+    (*plsc->dispatch_table->pl_state) ((struct PLStream_struct *) plsc, op);
 }
 
 /* Escape function, for driver-specific commands. */
@@ -174,10 +182,23 @@ plP_state(PLINT op)
 void
 plP_esc(PLINT op, void *ptr)
 {
-    (*plsc->dispatch_table->pl_esc) ((struct PLStream_struct *) plsc, op, ptr);
+  PLINT clpxmi, clpxma, clpymi, clpyma;
+  EscText* args;
 
-    if (plsc->plbuf_write)
-	plbuf_esc(plsc, op, ptr);
+  /* The plot buffer must be called first */
+  if(plsc->plbuf_write) plbuf_esc(plsc, op, ptr);
+
+  /* Text coordinates must pass through the driver interface filter */
+  if(op==PLESC_HAS_TEXT && plsc->dev_unicode) {
+
+    /* Apply the driver interface filter */
+    if (plsc->difilt) {
+      args = (EscText*)ptr;
+      difilt(&(args->x),&(args->y),1,&clpxmi,&clpxma,&clpymi,&clpyma);
+    }
+  }
+  
+  (*plsc->dispatch_table->pl_esc) ((struct PLStream_struct *) plsc, op, ptr);
 }
 
 /* Set up plot window parameters. */
@@ -351,56 +372,480 @@ plP_fill(short *x, short *y, PLINT npts)
 #define DEBUG_TEXT
 */
 
+#define hex2dec( a ) isdigit(a) ? a - 48 : (toupper(a) - 65) + 10
+
+/*--------------------------------------------------------------------------*\
+ *  int text2num( char *text, char end, PLUNICODE *num)
+ *       char *text - pointer to the text to be parsed
+ *       char end   - end character (i.e. ')' or ']' to stop parsing
+ *       PLUNICODE *num - pointer to an PLUNICODE to store the value
+ *
+ *    Function takes a string, which can be either hex or decimal,
+ *    and converts it into an PLUNICODE, stopping at either a null,
+ *    or the character pointed to by 'end'. It is a bit brain-dead,
+ *    and probably should make more checks, but it works.
+\*--------------------------------------------------------------------------*/
+
+int text2num( const char *text, char end, PLUNICODE *num)
+{
+  int base=10;
+  unsigned short i=0;
+  *num=0;
+
+  if (text[1]=='x')
+    {
+      base=16;
+      i=2;
+    }
+
+  while ((text[i]!=end)&&(text[i]!=0))
+    {
+      *num*=base;
+      *num+=hex2dec(text[i]);
+      i++;
+    }
+  return(i);
+}
+
+/*--------------------------------------------------------------------------*\
+ *  int text2fci( char *text, unsigned char *hexdigit, unsigned char *hexpower)
+ *       char *text - pointer to the text to be parsed
+ *       unsigned char *hexdigit - pointer to hex value that is stored.
+ *       unsigned char *hexpower - pointer to hex power (left shift) that is stored.
+ *
+ *    Function takes a pointer to a string, which is looked up in a table
+ *    to determine the corresponding FCI (font characterization integer)
+ *    hex digit value and hex power (left shift).  All matched strings
+ *    start with "<" and end with the two characters "/>".
+ *    If the lookup succeeds, hexdigit and hexpower are set to the appropriate
+ *    values in the table, and the function returns the number of characters
+ *    in text that are consumed by the matching string in the table lookup.
+ *
+ *    If the lookup fails, hexdigit is set to 0, hexpower is set to and
+ *    impossible value, and the function returns 0.
+\*--------------------------------------------------------------------------*/
+
+int text2fci( const char *text, unsigned char *hexdigit, unsigned char *hexpower)
+{
+   typedef struct
+     {
+	char *ptext;
+	unsigned char hexdigit;
+	unsigned char hexpower;
+     }
+   TextLookupTable;
+   /* This defines the various font control commands and the corresponding
+    * hexdigit and hexpower in the FCI.
+    */
+#define N_TextLookupTable 10
+   const TextLookupTable lookup[N_TextLookupTable] = {
+	{"<sans-serif/>", PL_FCI_SANS, PL_FCI_FAMILY},
+	{"<serif/>", PL_FCI_SERIF, PL_FCI_FAMILY},
+	{"<monospace/>", PL_FCI_MONO, PL_FCI_FAMILY},
+	{"<script/>", PL_FCI_SCRIPT, PL_FCI_FAMILY},
+	{"<symbol/>", PL_FCI_SYMBOL, PL_FCI_FAMILY},
+	{"<upright/>", PL_FCI_UPRIGHT, PL_FCI_STYLE},
+	{"<italic/>", PL_FCI_ITALIC, PL_FCI_STYLE},
+	{"<oblique/>", PL_FCI_OBLIQUE, PL_FCI_STYLE},
+	{"<medium/>", PL_FCI_MEDIUM, PL_FCI_WEIGHT},
+	{"<bold/>", PL_FCI_BOLD, PL_FCI_WEIGHT}
+   };
+   int i, length;
+   for (i=0; i<N_TextLookupTable; i++) {
+      length = strlen(lookup[i].ptext);
+      if (! strncmp(text, lookup[i].ptext, length)) {
+	 *hexdigit = lookup[i].hexdigit;
+	 *hexpower = lookup[i].hexpower;
+	 return(length);
+      }
+   }
+   *hexdigit = 0;
+   *hexpower = PL_FCI_HEXPOWER_IMPOSSIBLE;
+   return(0);
+}
+
+PLUNICODE unicode_buffer[1024];
+
 void
 plP_text(PLINT base, PLFLT just, PLFLT *xform, PLINT x, PLINT y,
 	 PLINT refx, PLINT refy, const char *string)
 {
-    if (plsc->dev_text) {
-	EscText args;
 
-	args.base = base;
-	args.just = just;
-	args.xform = xform;
-	args.x = x;
-	args.y = y;
-	args.refx = refx;
-	args.refy = refy;
-	args.string = string;
+   if (plsc->dev_text) { /* Does the device render it's own text ? */
+      EscText args;
+      short len=0;
+      char skip;
+      unsigned short i,j, k;
+      PLUNICODE code;
+      char esc;
+      int idx;
 
-	if (plsc->plbuf_write)
-	    plbuf_esc(plsc, PLESC_HAS_TEXT, &args);
+      args.base = base;
+      args.just = just;
+      args.xform = xform;
+      args.x = x;
+      args.y = y;
+      args.refx = refx;
+      args.refy = refy;
+      args.string = string;
 
-	plP_esc(PLESC_HAS_TEXT, &args);
-#ifndef DEBUG_TEXT
-    } else {
+      if (plsc->dev_unicode) { /* Does the device also understand unicode? */
+	 PLINT ig;
+	 PLUNICODE fci, fcisave;
+	 unsigned char hexdigit, hexpower;
+
+	 PLINT clpxmi, clpxma, clpymi, clpyma;
+
+	 /* Now process the text string */
+
+	 if (string!=NULL) {        /* If the string isn't blank, then we will 
+                                     * continue 
+				     */
+
+	    len=strlen(string);     /* this length is only used in the loop 
+				     * counter, we will work out the length of 
+				     * the unicode string as we go */
+	    plgesc(&esc);
+	
+	    /* At this stage we will do some translations into unicode, like 
+	     * conversion to Greek , and will save other translations such as 
+	     * superscript for the driver to do later on. As we move through 
+	     * the string and do the translations, we will get
+	     * rid of the esc character sequence, just replacing it with 
+	     * unicode.
+	     */
+	
+	    /* Obtain FCI (font characterization integer) for start of 
+	     * string. */
+	    plgfci(&fci);
+	    for (j=i=0;i<len;i++) {    /* Walk through the string, and convert 
+					* some stuff to unicode on the fly */
+	       skip=0;
+	
+	       if (string[i]==esc) {
+		  switch(string[i+1]) {
+		   case '(':  /* hershey code */
+		     i+=2+text2num(&string[i+2],')',&code);
+		     idx=plhershey2unicode(code);
+		     /* momentarily switch to symbol font. */
+		     fcisave = fci;
+		     plP_hex2fci(PL_FCI_SYMBOL, PL_FCI_FAMILY, &fci);
+		     unicode_buffer[j++]= fci;
+		     unicode_buffer[j++] = \
+		       (PLUNICODE)hershey_to_unicode_lookup_table[idx].Unicode;
+
+		     /* if unicode_buffer[j-1] corresponds to the escape 
+		      * character must unescape it by appending one more.  
+		      * This will probably always be necessary since it is 
+		      * likely unicode_buffer will always have to contain 
+		      * escape characters that are interpreted by the device 
+		      * driver.
+		      */
+		     if (unicode_buffer[j-1]==esc) unicode_buffer[j++]=esc;
+		     fci = fcisave;
+		     unicode_buffer[j]= fci;
+		     skip=1;
+		     break;
+		
+		   case '[':  /* unicode */
+		     i+=2+text2num(&string[i+2],']',&code);
+		     /* momentarily switch to symbol font. */
+		     fcisave = fci;
+		     plP_hex2fci(PL_FCI_SYMBOL, PL_FCI_FAMILY, &fci);
+		     unicode_buffer[j++]= fci;
+		     unicode_buffer[j++]=code;
+		     /* if unicode_buffer[j-1] corresponds to the escape 
+		      * character must unescape it by appending one more.  
+		      * This will probably always be necessary since it is 
+		      * likely unicode_buffer will always have to contain 
+		      * escape characters that are interpreted by the device 
+		      * driver.
+		      */
+		     if (unicode_buffer[j-1]==esc) unicode_buffer[j++]=esc;
+		     fci = fcisave;
+		     unicode_buffer[j] = fci;
+		     skip=1;
+		     break;
+		
+		   case '<':  /* change font*/
+		     if ('0' <= string[i+2] && string[i+2] <= '9' ) {
+			i+=2+text2num(&string[i+2],'>', &code);
+			if (code & PL_FCI_MARK) {
+			   /* code is a complete FCI (font characterization
+			    * integer): change FCI to this value.
+			    */
+			   fci = code;
+			   unicode_buffer[j]=fci;
+			   skip=1;
+			}
+			else {
+			   /* code is not complete FCI. Change
+			    * FCI with hex power in rightmost hex
+			    * digit and hex digit value in second rightmost
+			    * hex digit.
+			    */
+			   hexdigit = (code >> 4) & PL_FCI_HEXDIGIT_MASK;
+			   hexpower = code & PL_FCI_HEXPOWER_MASK;
+			   plP_hex2fci(hexdigit, hexpower, &fci);
+			   unicode_buffer[j]=fci;
+			   skip=1;
+			}
+		     }
+		
+		     else {
+			i+=text2fci(&string[i+1], &hexdigit, &hexpower);
+			if (hexpower < 7) {
+			   plP_hex2fci(hexdigit, hexpower, &fci);
+			   unicode_buffer[j]=fci;
+			   skip=1;
+			}
+		     }
+		     break;
+		
+		   case 'f':  /* Deprecated Hershey-style font change*/
+		   case 'F':  /* Deprecated Hershey-style font change*/
+		     /* We implement an approximate response here so that 
+		      * reasonable results are obtained for unicode fonts, 
+		      * but this method is deprecated and the #<nnn> or
+		      * #<command string> methods should be used instead
+		      * to change unicode fonts in mid-string.
+		      */
+		     fci = PL_FCI_MARK;
+		     if (string[i+2] == 'n') {
+			/* medium, upright, sans-serif */
+			plP_hex2fci(PL_FCI_SANS, PL_FCI_FAMILY, &fci);
+		     } else if (string[i+2] == 'r') {
+			/* medium, upright, serif */
+			plP_hex2fci(PL_FCI_SERIF, PL_FCI_FAMILY, &fci);
+		     } else if (string[i+2] == 'i') {
+			/* medium, italic, serif */
+			plP_hex2fci(PL_FCI_ITALIC, PL_FCI_STYLE, &fci);
+			plP_hex2fci(PL_FCI_SERIF, PL_FCI_FAMILY, &fci);
+		     } else if (string[i+2] == 's') {
+			/* medium, upright, script */
+			plP_hex2fci(PL_FCI_SCRIPT, PL_FCI_FAMILY, &fci);
+		     } else
+		       fci = PL_FCI_IMPOSSIBLE;
+		
+		     if (fci != PL_FCI_IMPOSSIBLE){
+			i+=2;
+			unicode_buffer[j] = fci;
+			skip = 1;
+		     }
+		     break;
+		
+		   case 'g':  /* Greek font */
+		   case 'G':  /* Greek font */
+		     /* Get the index in the lookup table
+		      * 527 = upper case alpha displacement in Hershey Table
+		      * 627 = lower case alpha displacement in Hershey Table
+		      */
+		     /* momentarily switch to symbol font. */
+		     fcisave = fci;
+		     plP_hex2fci(PL_FCI_SYMBOL, PL_FCI_FAMILY, &fci);
+		     unicode_buffer[j++]= fci;
+		     ig = plP_strpos(plP_greek_mnemonic, string[i+2]);
+		     if (ig >= 0) {
+			if (ig >= 24)
+			  ig = ig + 100 - 24;
+			idx=plhershey2unicode(ig+527);
+			unicode_buffer[j++] = \
+			  (PLUNICODE)hershey_to_unicode_lookup_table[idx].Unicode;
+			i+=2;
+			skip=1;  /* skip is set if we have copied something 
+				  * into the unicode table */
+		     }
+		     else {
+			/* Use "unknown" unicode character if string[i+2] 
+			 * is not in the Greek array.*/
+			unicode_buffer[j++]=(PLUNICODE)0x00;
+			i+=2;
+			skip=1;  /* skip is set if we have copied something 
+				  * into  the unicode table */
+		     }
+		     fci = fcisave;
+		     unicode_buffer[j]= fci;
+		     break;
+		
+		  }
+	       }
+	
+	       if (skip==0) {
+                  PLUNICODE unichar;
+#ifdef HAVE_LIBUNICODE
+		  char* ptr = unicode_get_utf8 (string + i, &unichar);
+#else
+		  char* ptr = utf8_to_ucs4 (string + i, &unichar);
 #endif
-	plstr(base, xform, refx, refy, string);
-    }
+                  if (ptr == NULL) {
+                    char buf[80];
+                    strncpy (buf, string, 30);
+                    sprintf (buf, "UTF-8 string is malformed: %s%s",
+                             buf, strlen (string) > 30 ? "[...]" : "");
+                    plabort (buf);
+                  }
+                  unicode_buffer [j] = unichar;
+                  i += ptr - (string + i) - 1;
+
+		  /* Search for escesc (an unescaped escape) in the input 
+		   * string and adjust unicode_buffer accordingly).
+		   */
+		  if (unicode_buffer[j] == esc && string[i+1] == esc) {
+		    i++;
+		    unicode_buffer[++j] = esc;
+		  }
+	       }
+	       j++;
+	    }
+	    if (j > 0) {
+	       args.unicode_array_len=j; /* Much easier to set the length than 
+					  * work it out later :-) */
+	       args.unicode_array=&unicode_buffer[0]; /* Get address of the 
+						       * unicode buffer (even 
+						       * though it is 
+						       * currently  static) */
+	    } else
+	      /* Don't print anything, if there is no unicode to print! */
+	      return;
+	 }
+      }
+
+      if (plsc->dev_unicode) {
+	args.string=NULL; /* We are using unicode */
+      }
+      else  {
+	args.string = string;
+      }
+
+      plP_esc(PLESC_HAS_TEXT, &args);
+#ifndef DEBUG_TEXT
+   } else {
+#endif
+      plstr(base, xform, refx, refy, string);
+   }
+}
+
+/* convert utf8 string to ucs4 unichar */
+static char *
+utf8_to_ucs4(const char *ptr, PLUNICODE *unichar)
+{
+   char tmp;
+   int isFirst = 1;
+   int cnt;
+   
+   do {
+      /* Get next character in string */
+      tmp = *ptr++;
+      if (isFirst) { /* First char in UTF8 sequence */
+	 isFirst = 0;
+	 /* Determine length of sequence */
+	 if ((unsigned char)(tmp & 0x80) == 0x00) { /* single char */
+	    *unichar = (unsigned int)tmp & 0x7F;
+	    cnt = 0;
+	 } else if ((unsigned char)(tmp & 0xE0) == 0xC0) { /* 2 chars */
+	    *unichar = (unsigned int)tmp & 0x1F;
+	    cnt = 1;
+	 } else if ((unsigned char)(tmp & 0xF0) == 0xE0) { /* 3 chars */
+	    *unichar = (unsigned char)tmp & 0x0F;
+	    cnt = 2;
+	 } else if ((unsigned char)(tmp & 0xF8) == 0xF0) { /* 4 chars */
+	    *unichar = (unsigned char)tmp & 0x07;
+	    cnt = 3;
+	 } else if ((unsigned char)(tmp & 0xFC) == 0xF8) { /* 5 chars */
+	    *unichar = (unsigned char)tmp & 0x03;
+	    cnt = 4;
+	 } else if ((unsigned char)(tmp & 0xFE) == 0xFC) { /* 6 chars */
+	    *unichar = (unsigned char)tmp & 0x01;
+	    cnt = 5;
+	 } else { /* Malformed */
+	    ptr = NULL;
+	    cnt = 0;
+	 }
+      } else { /* Subsequent char in UTF8 sequence */
+	 if ((unsigned char)(tmp & 0xC0) == 0x80) {
+	    *unichar = (*unichar << 6) | ((unsigned int)tmp & 0x3F);
+	    cnt--;
+	 } else { /* Malformed */
+	    ptr = NULL;
+	    cnt = 0;
+	 }
+      }
+   } while (cnt > 0);
+   return (char *) ptr;
+}
+
+/* convert ucs4 unichar to utf8 string */
+int
+ucs4_to_utf8(PLUNICODE unichar, char *ptr)
+{
+  unsigned char *tmp;
+  int len;
+
+  tmp = ptr;
+
+  if ( (unichar & 0xffff80) == 0 ) {  /* single byte */
+    *tmp = (unsigned char) unichar;
+    tmp++;
+    len = 1;
+  }
+  else if ( (unichar & 0xfff800) == 0) { /* two bytes */
+    *tmp = (unsigned char) 0xc0 | (unichar >> 6);
+    tmp++;
+    *tmp = (unsigned char) 0x80 | (unichar & 0x3f);
+    tmp++;
+    len = 2;
+  }
+  else if ( (unichar & 0xff0000) == 0) { /* three bytes */
+    *tmp = (unsigned char) 0xe0 | (unichar >> 12);
+    tmp++;
+    *tmp = (unsigned char) 0x80 | ((unichar >> 6) & 0x3f);
+    tmp++;      
+    *tmp = (unsigned char) 0x80 | (unichar & 0x3f);
+    tmp++;
+    len = 3;
+  }
+  else if ( (unichar & 0xe0000) == 0){ /* four bytes */
+    *tmp = (unsigned char) 0xf0 | (unichar >> 18);
+    tmp++;
+    *tmp = (unsigned char) 0x80 | ((unichar >> 12) & 0x3f);
+    tmp++;      
+    *tmp = (unsigned char) 0x80 | ((unichar >> 6) & 0x3f);
+    tmp++;      
+    *tmp = (unsigned char) 0x80 | (unichar & 0x3f);
+    tmp++;
+    len = 4;
+  }
+  else {  /* Illegal coding */
+    len = 0;
+  }
+  *tmp = '\0';
+
+  return len;
 }
 
 static void
 grline(short *x, short *y, PLINT npts)
 {
-    (*plsc->dispatch_table->pl_line) ( (struct PLStream_struct *) plsc,
-                                       x[0], y[0], x[1], y[1] );
+   (*plsc->dispatch_table->pl_line) ( (struct PLStream_struct *) plsc,
+				      x[0], y[0], x[1], y[1] );
 }
 
 static void
 grpolyline(short *x, short *y, PLINT npts)
 {
-    (*plsc->dispatch_table->pl_polyline) ( (struct PLStream_struct *) plsc,
-                                           x, y, npts );
+   (*plsc->dispatch_table->pl_polyline) ( (struct PLStream_struct *) plsc,
+					  x, y, npts );
 }
 
 static void
 grfill(short *x, short *y, PLINT npts)
 {
-    plsc->dev_npts = npts;
-    plsc->dev_x = x;
-    plsc->dev_y = y;
+   plsc->dev_npts = npts;
+   plsc->dev_x = x;
+   plsc->dev_y = y;
 
-    (*plsc->dispatch_table->pl_esc) ( (struct PLStream_struct *) plsc,
-                                      PLESC_FILL, NULL );
+   (*plsc->dispatch_table->pl_esc) ( (struct PLStream_struct *) plsc,
+				     PLESC_FILL, NULL );
 }
 
 /*--------------------------------------------------------------------------*\
@@ -1338,6 +1783,9 @@ c_plend(void)
       free_mem(dispatch_table[i]);
     }
     free_mem(dispatch_table);
+
+    plP_FreeDrvOpts();
+
     lib_initialized = 0;
 }
 
@@ -1495,6 +1943,8 @@ plstrm_init(void)
 	if (plsc->cmap1 == NULL)
 	    plscmap1n(0);
     }
+
+    plsc->psdoc = NULL;
 }
 
 /*--------------------------------------------------------------------------*\
@@ -1643,6 +2093,14 @@ int plInBuildTree()
   if (inited == 0) {
     char currdir[256];
 
+/* AM: getcwd has a somewhat strange status on Windows, its proper
+   name is _getcwd, this is a problem in the case of DLLs, like with
+   the Java bindings.
+*/
+#ifdef WIN32
+#define getcwd _getcwd
+#endif
+
     if (getcwd(currdir, 256) == NULL) {
       pldebug("plInBuildTree():", "Not enough buffer space");
     } else if (strncmp(BUILD_DIR, currdir, strlen(BUILD_DIR)) == 0)
@@ -2178,6 +2636,9 @@ c_plgpage(PLFLT *p_xp, PLFLT *p_yp,
 void
 c_plspage(PLFLT xp, PLFLT yp, PLINT xleng, PLINT yleng, PLINT xoff, PLINT yoff)
 {
+    if (plsc->level > 0)
+        plwarn("calling plspage() after plinit() may give unpredictable results");
+
     if (xp)
 	plsc->xdpi = xp;
     if (yp)
@@ -2462,9 +2923,53 @@ plgesc(char *p_esc)
     *p_esc = plsc->esc;
 }
 
+/* Set the FCI (font characterization integer) for unicode-enabled device
+ * drivers.
+ */
+void
+c_plsfci(PLUNICODE fci)
+{
+   /* Always mark FCI as such. */
+   plsc->fci = fci | PL_FCI_MARK;
+}
+
+/* Get the FCI (font characterization integer) for unicode-enabled device
+ * drivers.
+ */
+void
+c_plgfci(PLUNICODE *pfci)
+{
+   /* Always mark FCI as such. */
+   *pfci = plsc->fci | PL_FCI_MARK;
+}
+/* Store hex digit value shifted to the left by hexdigit hexadecimal digits
+ * into pre-existing FCI.
+ */
+void
+plP_hex2fci(unsigned char hexdigit, unsigned char hexpower, PLUNICODE *pfci)
+{
+   PLUNICODE mask;
+   hexpower = hexpower & PL_FCI_HEXPOWER_MASK;
+   mask = ~ (((PLUNICODE) PL_FCI_HEXDIGIT_MASK) << ((PLUNICODE) 4*hexpower));
+   *pfci = *pfci & mask;
+   mask = (((PLUNICODE) (hexdigit & PL_FCI_HEXDIGIT_MASK)) << (4*hexpower));
+   *pfci = *pfci | mask;
+}
+
+/* Retrieve hex digit value from FCI that is masked out and shifted to the
+ * right by hexpower hexadecimal digits. */
+void
+plP_fci2hex(PLUNICODE fci, unsigned char *phexdigit, unsigned char hexpower)
+{
+   PLUNICODE mask;
+   hexpower = hexpower & PL_FCI_HEXPOWER_MASK;
+   mask = (((PLUNICODE) PL_FCI_HEXPOWER_MASK) << ((PLUNICODE) (4*hexpower)));
+   *phexdigit = (unsigned char) ((fci & mask) >>
+				 ((PLUNICODE) (4*hexpower)));
+}
+
 /* Get the current library version number */
 /* Note: you MUST have allocated space for this (80 characters is safe) */
-
 void
 c_plgver(char *p_ver)
 {
