@@ -1,4 +1,4 @@
-/* $Id: plctrl.c,v 1.76 2004/01/17 16:41:40 rlaboiss Exp $
+/* $Id: plctrl.c,v 1.84 2005/04/28 15:15:38 tomduck Exp $
 
 	Misc. control routines, like begin, end, exit, change graphics/text
 	mode, change color.  Includes some spillage from plcore.c.  If you
@@ -21,7 +21,7 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 */
 
@@ -417,7 +417,7 @@ c_plscmap1l(PLINT itype, PLINT npts, PLFLT *pos,
 	    r = coord1[n];
 	    g = coord2[n];
 	    b = coord3[n];
-	    plRGB_HLS(r, g, b, &h, &l, &s);
+	    c_plrgbhls(r, g, b, &h, &l, &s);
 	}
 
 	plsc->cmap1cp[n].h = h;
@@ -494,7 +494,7 @@ plcmap1_calc(void)
 	    while (h < 0.)
 		h += 360.;
 
-	    plHLS_RGB(h, l, s, &r, &g, &b);
+	    c_plhlsrgb(h, l, s, &r, &g, &b);
 
 	    plsc->cmap1[i].r = MAX(0, MIN(255, (int) (256. * r)));
 	    plsc->cmap1[i].g = MAX(0, MIN(255, (int) (256. * g)));
@@ -553,6 +553,9 @@ c_plscmap0n(PLINT ncol0)
 
     plsc->ncol0 = ncol;
     plcmap0_def(imin, imax);
+
+    if (plsc->level > 0)
+	plP_state(PLSTATE_CMAP0);
 }
 
 /*--------------------------------------------------------------------------*\
@@ -653,6 +656,23 @@ plcmap0_def(int imin, int imax)
     color_def(14, 250, 128, 114, "salmon" );	/* salmon */
     color_def(15, 255, 255, 255, "white" );	/* white */
 
+/*     color_def(0, 255, 255, 255, "white" );	/\* white *\/ */
+/*     color_def(1,    0,   0,   0, "black" );	/\* black *\/ */
+/*     color_def(2,    0,   0, 255, "blue" );	/\* blue *\/ */
+/*     color_def(3,  255,   0,   0, "red");	/\* red *\/ */
+/*     color_def(4,  165,  42,  42, "brown" );	/\* brown *\/ */
+/*     color_def(5, 250, 128, 114, "salmon" );	/\* salmon *\/ */
+/*     color_def(6,  255, 192, 203, "pink" );	/\* pink *\/ */
+/*     color_def(7,  127, 255, 212, "aquamarine" );	/\* aquamarine *\/ */
+/*     color_def(8,  245, 222, 179, "wheat" );	/\* wheat *\/ */
+/*     color_def(9,  64, 224, 208, "turquoise" );	/\* turquoise *\/ */
+/*     color_def(10,  190, 190, 190, "grey" );	/\* grey *\/ */
+/*     color_def(11,   0, 255, 255, "cyan" );	/\* cyan *\/ */
+/*     color_def(12,    0, 255,   0, "green" );	/\* green *\/ */
+/*     color_def(13,  255, 255,   0, "yellow" );	/\* yellow *\/ */
+/*     color_def(14, 255,   0, 255, "magenta" );	/\* magenta *\/ */
+/*     color_def(15, 138,  43, 226, "BlueViolet" );	/\* Blue Violet *\/ */
+
 /* Any others are just arbitrarily set */
 
     for (i = 16; i <= imax; i++)
@@ -730,6 +750,9 @@ plcmap1_def(void)
     s[5] = 1;
 
     c_plscmap1l(0, 6, i, h, l, s, NULL);
+
+    if (plsc->level > 0)
+	plP_state(PLSTATE_CMAP1);
 }
 
 /*--------------------------------------------------------------------------*\
@@ -810,14 +833,14 @@ c_plhls(PLFLT h, PLFLT l, PLFLT s)
 {
     PLFLT r, g, b;
 
-    plHLS_RGB(h, l, s, &r, &g, &b);
+    c_plhlsrgb(h, l, s, &r, &g, &b);
     plrgb(r, g, b);
 }
 
 /*--------------------------------------------------------------------------*\
  * void value()
  *
- * Auxiliary function used by plHLS_RGB().
+ * Auxiliary function used by c_plhlsrgb().
 \*--------------------------------------------------------------------------*/
 
 static PLFLT
@@ -843,7 +866,7 @@ value(double n1, double n2, double hue)
 }
 
 /*--------------------------------------------------------------------------*\
- * void plHLS_RGB()
+ * void c_plhlsrgb()
  *
  * Convert HLS color to RGB color.
  * Bounds on HLS (input):
@@ -857,7 +880,7 @@ value(double n1, double n2, double hue)
 \*--------------------------------------------------------------------------*/
 
 void
-plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b)
+c_plhlsrgb(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b)
 {
     PLFLT m1, m2;
 
@@ -874,7 +897,7 @@ plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b)
 }
 
 /*--------------------------------------------------------------------------*\
- * void plRGB_HLS()
+ * void c_plrgbhls()
  *
  * Convert RGB color to HLS color.
  * Bounds on RGB (input) is always [0., 1.].
@@ -885,7 +908,7 @@ plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b)
 \*--------------------------------------------------------------------------*/
 
 void
-plRGB_HLS(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s)
+c_plrgbhls(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s)
 {
     PLFLT h, l, s, d, rc, gc, bc, rgb_min, rgb_max;
 
@@ -1551,11 +1574,22 @@ void
 plP_getmember(PLStream *pls)
 {
     char tmp[256];
+    char prefix[256];
+    char* suffix; 
 
     if (pls->FileName == NULL)
 	pls->FileName = (char *) malloc(10 + strlen(pls->BaseName));
 
-    sprintf(tmp, "%s.%%0%1ii", pls->BaseName, (int) pls->fflen);
+    suffix = strstr (pls->BaseName, "%n");
+
+    if (suffix == NULL) 
+      sprintf (tmp, "%s.%%0%1ii", pls->BaseName, (int) pls->fflen);
+    else {
+      strncpy (prefix, pls->BaseName, 256);
+      prefix [suffix - pls->BaseName] = 0;
+      sprintf (tmp, "%s%%0%1ii%s", prefix, (int) pls->fflen, suffix + 2);
+    }
+  
     sprintf(pls->FileName, tmp, pls->member);
 }
 
@@ -1649,7 +1683,7 @@ plGetFam(PLStream *pls)
 
 void
 plRotPhy(PLINT orient, PLINT xmin, PLINT ymin, PLINT xmax, PLINT ymax,
-	 int *px, int *py)
+	 PLINT *px, PLINT *py)
 {
     int x, y;
 
