@@ -1,4 +1,4 @@
-/* $Id: plline.c,v 1.30 2004/01/17 16:41:40 rlaboiss Exp $
+/* $Id: plline.c,v 1.38 2005/06/22 09:25:57 arjenmarkus Exp $
 
 	Routines dealing with line generation.
 
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 
 */
 
@@ -53,6 +53,11 @@ genlin(short *x, short *y, PLINT npts);
 static void
 grdashline(short *x, short *y);
 
+/* Determines if a point is inside a polygon or not */
+
+static int
+pointinpolygon( int n, PLINT *x, PLINT *y, PLINT xp, PLINT yp );
+
 /*----------------------------------------------------------------------*\
  * void pljoin()
  *
@@ -484,6 +489,10 @@ pllclp(PLINT *x, PLINT *y, PLINT npts)
  * void plP_pllclp()
  *
  * Draws a polyline within the clip limits.
+ *
+ * (AM)
+ * Wanted to change the type of xclp, yclp to avoid overflows!
+ * But that changes the type for the drawing routines too!
 \*----------------------------------------------------------------------*/
 
 void
@@ -493,9 +502,19 @@ plP_pllclp(PLINT *x, PLINT *y, PLINT npts,
 {
     PLINT x1, x2, y1, y2;
     PLINT i, iclp = 0;
-    short xclp[PL_MAXPOLY], yclp[PL_MAXPOLY];
+
+    short _xclp[PL_MAXPOLY], _yclp[PL_MAXPOLY];
+    short *xclp, *yclp;
     int drawable;
 
+    if ( npts < PL_MAXPOLY ) {
+        xclp = _xclp;
+        yclp = _yclp;
+    } else {
+        xclp = (short *) malloc( npts*sizeof(short) ) ;
+        yclp = (short *) malloc( npts*sizeof(short) ) ;
+    }
+
     for (i = 0; i < npts - 1; i++) {
 	x1 = x[i];
 	x2 = x[i + 1];
@@ -549,6 +568,11 @@ plP_pllclp(PLINT *x, PLINT *y, PLINT npts,
 
     plsc->currx = x[npts-1];
     plsc->curry = y[npts-1];
+
+    if ( xclp != _xclp ) {
+        free( xclp );
+        free( yclp );
+    }
 }
 
 /*----------------------------------------------------------------------*\
@@ -566,13 +590,23 @@ plP_pllclp(PLINT *x, PLINT *y, PLINT npts,
  * the typical return value is 3 or -3, since 3 turns are necessary in order
  * to complete the fill region.  Only for really oddly shaped fill regions
  * will it give the wrong answer.
+ *
+ * AM:
+ * Changed the computation: use the outer product to compute the surface
+ * area, the sign determines if the polygon is followed clockwise or
+ * counterclockwise. This is more reliable. Floating-point numbers
+ * are used to avoid overflow.
 \*----------------------------------------------------------------------*/
 
 static int
 circulation(PLINT *x, PLINT *y, PLINT npts)
 {
-    int xproduct, direction = 0;
-    PLINT i, x1, y1, x2, y2, x3, y3;
+    PLFLT xproduct;
+    int direction = 0;
+    PLFLT x1, y1, x2, y2, x3, y3;
+    int i;
+
+    xproduct = 0.0 ;
     for (i = 0; i < npts - 1; i++) {
 	x1 = x[i]; x2 = x[i+1];
 	y1 = y[i]; y2 = y[i+1];
@@ -581,11 +615,11 @@ circulation(PLINT *x, PLINT *y, PLINT npts)
 	} else {
 	    x3 = x[0]; y3 = y[0];
 	}
-	xproduct = (x2-x1)*(y3-y2) - (y2-y1)*(x3-x2);
-
-	if (xproduct > 0) direction++;
-	if (xproduct < 0) direction--;
+	xproduct = xproduct + (x2-x1)*(y3-y2) - (y2-y1)*(x3-x2);
     }
+
+    if (xproduct > 0.0) direction = 1;
+    if (xproduct < 0.0) direction = -1;
     return direction;
 }
 
@@ -601,17 +635,38 @@ plP_plfclp(PLINT *x, PLINT *y, PLINT npts,
 	   void (*draw) (short *, short *, PLINT))
 {
     PLINT i, x1, x2, y1, y2;
-    int iclp = 0;
-    short xclp[2*PL_MAXPOLY], yclp[2*PL_MAXPOLY];
+    int iclp = 0, iout = 2;
+    short _xclp[2*PL_MAXPOLY+2], _yclp[2*PL_MAXPOLY+2];
+    short *xclp, *yclp;
     int drawable;
     int crossed_xmin1 = 0, crossed_xmax1 = 0;
     int crossed_ymin1 = 0, crossed_ymax1 = 0;
     int crossed_xmin2 = 0, crossed_xmax2 = 0;
     int crossed_ymin2 = 0, crossed_ymax2 = 0;
+    int crossed_up    = 0, crossed_down  = 0;
+    int crossed_left  = 0, crossed_right = 0;
+    int inside_lb ;
+    int inside_lu ;
+    int inside_rb ;
+    int inside_ru ;
 
 /* Must have at least 3 points and draw() specified */
     if (npts < 3 || !draw) return;
 
+    if ( npts < PL_MAXPOLY ) {
+        xclp = _xclp;
+        yclp = _yclp;
+    } else {
+        xclp = (short *) malloc( (2*npts+2)*sizeof(short) ) ;
+        yclp = (short *) malloc( (2*npts+2)*sizeof(short) ) ;
+    }
+
+
+    inside_lb = pointinpolygon(npts,x,y,xmin,ymin) ;
+    inside_lu = pointinpolygon(npts,x,y,xmin,ymax) ;
+    inside_rb = pointinpolygon(npts,x,y,xmax,ymin) ;
+    inside_ru = pointinpolygon(npts,x,y,xmax,ymax) ;
+
     for (i = 0; i < npts - 1; i++) {
 	x1 = x[i]; x2 = x[i+1];
 	y1 = y[i]; y2 = y[i+1];
@@ -625,6 +680,11 @@ plP_plfclp(PLINT *x, PLINT *y, PLINT npts,
 	    crossed_xmin2 = (x1 == xmin); crossed_xmax2 = (x1 == xmax);
 	    crossed_ymin2 = (y1 == ymin); crossed_ymax2 = (y1 == ymax);
 
+	    crossed_left  = (crossed_left  || crossed_xmin2);
+	    crossed_right = (crossed_right || crossed_xmax2);
+	    crossed_down  = (crossed_down  || crossed_ymin2);
+	    crossed_up    = (crossed_up    || crossed_ymax2);
+	    iout = iclp+2;
 	/* If the first segment, just add it. */
 
 	    if (iclp == 0) {
@@ -645,30 +705,68 @@ plP_plfclp(PLINT *x, PLINT *y, PLINT npts,
 	 */
 
 	    else {
-	    /* Treat the case where we encircled two corners */
+	    /* Treat the case where we encircled two corners:
+	       Construct a polygon out of the subset of vertices
+	       Note that the direction is important too when adding
+	       the extra points */
+		xclp[iclp+1] = x2; yclp[iclp+1] = y2;
+		xclp[iclp+2] = x1; yclp[iclp+2] = y1;
+		iout = iout - iclp + 1;
 	    /* Upper two */
-		if ( (crossed_xmin1 && crossed_xmax2 && y1 >= ymax) )
+		if ( ((crossed_xmin1 && crossed_xmax2) ||
+			     (crossed_xmin2 && crossed_xmax1)) &&
+			inside_lu )
 		{
-		    xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
-		    xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
+		    if ( crossed_xmin1 )
+		    {
+		        xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
+		        xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
+		    } else {
+		        xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
+		        xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
+		    }
 		}
 	    /* Lower two */
-		else if ( (crossed_xmin1 && crossed_xmax2 && y1 <= ymin) )
+		else if ( ((crossed_xmin1 && crossed_xmax2) ||
+			          (crossed_xmin2 && crossed_xmax1)) &&
+			inside_lb )
 		{
-		    xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
-		    xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		    if ( crossed_xmin1 )
+		    {
+		        xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
+		        xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		    } else {
+		        xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		        xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
+		    }
 		}
 	    /* Left two */
-		else if ( (crossed_ymin1 && crossed_ymax2 && x1 <= xmin) )
+		else if ( ((crossed_ymin1 && crossed_ymax2) ||
+			          (crossed_ymin2 && crossed_ymax1)) &&
+			inside_lb )
 		{
-		    xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
-		    xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
+		    if ( crossed_ymin1 )
+		    {
+		        xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
+		        xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
+		    } else {
+		        xclp[iclp] = xmin; yclp[iclp] = ymax; iclp++;
+		        xclp[iclp] = xmin; yclp[iclp] = ymin; iclp++;
+		    }
 		}
 	    /* Right two */
-		else if ( (crossed_ymin1 && crossed_ymax2 && x1 >= xmax) )
+		else if ( ((crossed_ymin1 && crossed_ymax2) ||
+			          (crossed_ymin2 && crossed_ymax1)) &&
+			inside_rb )
 		{
-		    xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
-		    xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		    if ( crossed_ymin1 )
+		    {
+		        xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		        xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
+		    } else {
+		        xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
+		        xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
+		    }
 		}
 	    /* Now the case where we encircled one corner */
 	    /* Lower left */
@@ -727,6 +825,13 @@ plP_plfclp(PLINT *x, PLINT *y, PLINT npts,
 	    xclp[iclp] = xmax; yclp[iclp] = ymax; iclp++;
 	    xclp[iclp] = xmax; yclp[iclp] = ymin; iclp++;
 	    (*draw)(xclp, yclp, iclp);
+
+	    if ( xclp != _xclp ) {
+	        free( xclp );
+	        free( yclp );
+	    }
+
+
 	    return;
 	}
     }
@@ -902,10 +1007,49 @@ plP_plfclp(PLINT *x, PLINT *y, PLINT npts,
 	}
     }
 
-/* Draw the sucker */
+/* Check for the case that only one side has been crossed
+   (AM) Just checking a single point turns out not to be
+   enough, apparently the crossed_*1 and crossed_*2 variables
+   are not quite what I expected.
+*/
+    if ( crossed_left+crossed_right+crossed_down+crossed_up == 1 &&
+		inside_lb+inside_rb+inside_lu+inside_ru == 4 ) {
+	int dir = circulation(x, y, npts);
+	PLINT xlim[4], ylim[4];
+	int insert;
+	int incr;
 
+	xlim[0] = xmin ; ylim[0] = ymin ;
+	xlim[1] = xmax ; ylim[1] = ymin ;
+	xlim[2] = xmax ; ylim[2] = ymax ;
+	xlim[3] = xmin ; ylim[3] = ymax ;
+	if ( dir > 0 ) {
+	    incr   = 1;
+	    insert = 0*crossed_left + 1*crossed_down + 2*crossed_right +
+	             3*crossed_up ;
+	} else {
+	    incr   = -1;
+	    insert = 3*crossed_left + 2*crossed_up + 1*crossed_right +
+	             0*crossed_down ;
+	}
+	for ( i=0; i < 4; i ++ ) {
+	    xclp[iclp] = xlim[insert] ;
+	    yclp[iclp] = ylim[insert] ;
+	    iclp   ++ ;
+	    insert += incr ;
+	    if ( insert > 3 ) insert = 0 ;
+	    if ( insert < 0 ) insert = 3 ;
+	}
+    }
+
+/* Draw the sucker */
     if (iclp >= 3)
 	(*draw)(xclp, yclp, iclp);
+
+    if ( xclp != _xclp ) {
+        free( xclp );
+        free( yclp );
+    }
 }
 
 /*----------------------------------------------------------------------*\
@@ -1154,3 +1298,104 @@ grdashline(short *x, short *y)
 	lasty = ytmp;
     }
 }
+
+/*----------------------------------------------------------------------*\
+ * int pointinpolygon()
+ *
+ * Returns 1 if the point is inside the polygon, 0 otherwise
+ * Note:
+ * Points on the polygon are considered to be outside
+\*----------------------------------------------------------------------*/
+
+static int
+pointinpolygon( int n, PLINT *x, PLINT *y, PLINT xp, PLINT yp )
+{
+    int i;
+    int count_crossings;
+    PLFLT x1, y1, x2, y2, xpp, ypp, xout, yout, xmax;
+    PLFLT xvp, yvp, xvv, yvv, xv1, yv1, xv2, yv2;
+    PLFLT inprod1, inprod2;
+
+    xpp = (PLFLT) xp;
+    ypp = (PLFLT) yp;
+
+    count_crossings = 0;
+
+
+    /* Determine a point outside the polygon  */
+
+    xmax = x[0] ;
+    xout = x[0] ;
+    yout = y[0] ;
+    for ( i = 0; i < n ; i ++ ) {
+        if ( xout > x[i] ) {
+            xout = x[i] ;
+        }
+        if ( xmax < x[i] ) {
+            xmax = x[i] ;
+        }
+    }
+    xout = xout - (xmax-xout) ;
+
+    /* Determine for each side whether the line segment between
+       our two points crosses the vertex */
+
+    xpp = (PLFLT) xp;
+    ypp = (PLFLT) yp;
+
+    xvp = xpp - xout;
+    yvp = ypp - yout;
+
+    for ( i = 0; i < n; i ++ ) {
+        x1 = (PLFLT) x[i] ;
+        y1 = (PLFLT) y[i] ;
+        if ( i < n-1 ) {
+            x2 = (PLFLT) x[i+1] ;
+            y2 = (PLFLT) y[i+1] ;
+        } else {
+            x2 = (PLFLT) x[0] ;
+            y2 = (PLFLT) y[0] ;
+        }
+
+        /* Skip zero-length segments */
+        if ( x1 == x2 && y1 == y2 ) {
+            continue;
+        }
+
+        /* Line through the two fixed points:
+           Are x1 and x2 on either side? */
+        xv1 = x1 - xout;
+        yv1 = y1 - yout;
+        xv2 = x2 - xout;
+        yv2 = y2 - yout;
+        inprod1 = xv1*yvp - yv1*xvp; /* Well, with the normal vector */
+        inprod2 = xv2*yvp - yv2*xvp;
+        if ( inprod1 * inprod2 >= 0.0 ) {
+            /* No crossing possible! */
+            continue;
+        }
+
+        /* Line through the two vertices:
+           Are xout and xpp on either side? */
+        xvv = x2   - x1;
+        yvv = y2   - y1;
+        xv1 = xpp  - x1;
+        yv1 = ypp  - y1;
+        xv2 = xout - x1;
+        yv2 = yout - y1;
+        inprod1 = xv1*yvv - yv1*xvv;
+        inprod2 = xv2*yvv - yv2*xvv;
+        if ( inprod1 * inprod2 >= 0.0 ) {
+            /* No crossing possible! */
+            continue;
+        }
+
+        /* We do have a crossing */
+        count_crossings ++;
+    }
+
+    /* Return the result: an even number of crossings means the
+       point is outside the polygon */
+
+    return (count_crossings%2);
+}
