@@ -1,4 +1,4 @@
-/* $Id: plot3d.c,v 1.63 2004/06/14 21:51:58 rlaboiss Exp $
+/* $Id: plot3d.c,v 1.65 2005/04/27 06:44:25 rlaboiss Exp $
 
 	3d plot routines.
 
@@ -20,7 +20,7 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include "plplotP.h"
@@ -444,7 +444,7 @@ c_plsurf3dl(PLFLT *x, PLFLT *y, PLFLT **z, PLINT nx, PLINT ny,
   /* we've got to draw the background grid first, hidden line code has to draw it last */
   if (zbflg) {
     PLFLT bx[3], by[3], bz[3];
-    PLFLT tick=0, tp;
+    PLFLT tick=zbtck, tp;
     PLINT nsub=0;
 
     /* get the tick spacing */
