@@ -1,4 +1,4 @@
-/* $Id: plvect.c,v 1.7 2004/06/24 00:06:30 airwin Exp $
+/* $Id: plvect.c,v 1.13 2005/04/27 06:44:25 rlaboiss Exp $
 
 	Vector plotting routines.
 
@@ -18,7 +18,7 @@
 
    You should have received a copy of the GNU Library General Public License
    along with PLplot; if not, write to the Free Software
-   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #define NEED_PLDEBUG
@@ -28,13 +28,7 @@
 
 /* Static function prototypes */
 
-void plP_plotvect(PLFLT x, PLFLT y, PLFLT u, PLFLT v, PLFLT scale);
-    
-void plvecf_int(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer), 
-		PLPointer f2evalv_data, PLPointer f2evalc_data,
-		PLINT nx, PLINT ny, PLFLT scale,
-		void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
-		PLPointer pltr_data);
+static void plP_plotvect(PLFLT x, PLFLT y, PLFLT u, PLFLT v, PLFLT scale);
 
 /*--------------------------------------------------------------------------*\
  * void plarrows()
@@ -167,7 +161,7 @@ c_plsvect(PLFLT *arrowx, PLFLT *arrowy, PLINT npts, PLINT fill) {
 /*
  * Plot an individual vector
  */
-void
+static void
 plP_plotvect(PLFLT x, PLFLT y, PLFLT u, PLFLT v, PLFLT scale) {
 	
     PLFLT uu, vv, px0, py0, dpx, dpy;
@@ -210,18 +204,18 @@ plP_plotvect(PLFLT x, PLFLT y, PLFLT u, PLFLT v, PLFLT scale) {
 }
 
 /*
- * void plvecf_int()
+ * void plfvect()
  *
- * Internal routine to plot a vector array with arbitrary coordinate 
+ * Internal routine to plot a vector array with arbitrary coordinate
  * and vector transformations
  */
-void plvecf_int(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer), 
+void plfvect(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer),
 		PLPointer f2eval_data1, PLPointer f2eval_data2,
 		PLINT nx, PLINT ny, PLFLT scale,
 		void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 		PLPointer pltr_data) {
-    PLINT i, j;
-    PLFLT **u, **v, **x, **y; 
+    PLINT i, j, i1, j1;
+    PLFLT **u, **v, **x, **y;
     PLFLT lscale, dx, dy, dxmin, dymin, umax, vmax;
 
     plAlloc2dGrid(&u, nx, ny);
@@ -236,21 +230,29 @@ void plvecf_int(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer),
 	    pltr((PLFLT) i, (PLFLT) j, &x[i][j], &y[i][j], pltr_data);
 	}
     }
-	    
+	
     /* Calculate apropriate scaling if necessary */
     if (scale <= 0.0 ) {
-	if (nx <= 1 && ny <= 1) {
-	    fprintf(stderr,"plvecf_int: not enough points for autoscaling\n");
-	    return;
-	}
-	dxmin = x[1][0]-x[0][0];
-	dymin = y[0][1]-y[0][0];
-        for (j=0; j<ny-1; j++) {
-            for (i=0 ;i<nx-1 ;i++) {
-		dx = (x[i+1][j]-x[i][j]);
-		dy = (y[i][j+1]-y[i][j]);
-		dxmin = (dx<dxmin)?dx:dxmin;
-		dymin = (dy<dymin)?dy:dymin;
+        if (nx <= 1 && ny <= 1) {
+            fprintf(stderr,"plfvect: not enough points for autoscaling\n");
+            return;
+        }
+	dxmin = 10E10;
+	dymin = 10E10;
+        for (j=0; j<ny; j++) {
+            for (i=0 ;i<nx ;i++) {
+                for (j1=j; j1<ny; j1++) {
+                    for (i1=0 ;i1<nx ;i1++) {
+		        dx = fabs(x[i1][j1]-x[i][j]);
+		        dy = fabs(y[i1][j1]-y[i][j]);
+			if (dx > 0) {
+		            dxmin = (dx<dxmin)?dx:dxmin;
+			}
+			if (dy > 0) {
+		            dymin = (dy<dymin)?dy:dymin;
+			}
+		    }
+		}
 	    }
 	}
 	umax = u[0][0];
@@ -264,7 +266,7 @@ void plvecf_int(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer),
 	umax = umax/dxmin;
 	vmax = vmax/dymin;
 	lscale = (umax<vmax)?umax:vmax;
-	lscale = 2.0/lscale;
+	lscale = 1.5/lscale;
 	if (scale < 0.0) {
 	    scale = -scale*lscale;
 	}
@@ -287,7 +289,7 @@ void plvecf_int(PLFLT (*plf2eval) (PLINT, PLINT, PLPointer),
 }
 
 void
-c_plvect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale, 
+c_plvect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale,
 	void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
 	PLPointer pltr_data)
 {
@@ -296,6 +298,6 @@ c_plvect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale,
     grid1.f = u;
     grid2.f = v;
 
-    plvecf_int(plf2eval2, (PLPointer) &grid1, (PLPointer) &grid2, 
+    plfvect(plf2eval2, (PLPointer) &grid1, (PLPointer) &grid2,
 	       nx, ny, scale, pltr, pltr_data);
 }
