@@ -1,6 +1,6 @@
-/* $Id: plplot.h,v 1.3 2004/01/19 18:09:21 airwin Exp $
+/* $Id: plplot.h,v 1.9 2005/05/22 17:11:36 airwin Exp $
 
-    Copyright (C) 1992 by 
+    Copyright (C) 1992 by
     Maurice J. LeBrun, Geoff Furnish, Tony Richardson.
 
     Macros and prototypes for the PLplot package.  This header file must
@@ -18,7 +18,7 @@
     Note: some systems allow the Fortran & C namespaces to clobber each
     other.  So for PLplot to work from Fortran, we do some rather nasty
     things to the externally callable C function names.  This shouldn't
-    affect any user programs in C as long as this file is included. 
+    affect any user programs in C as long as this file is included.
 */
 
 #ifndef __PLPLOT_H__
@@ -28,11 +28,11 @@
 
 /*--------------------------------------------------------------------------*\
  *    USING PLplot
- * 
- * To use PLplot from C or C++, it is only necessary to 
- * 
+ *
+ * To use PLplot from C or C++, it is only necessary to
+ *
  *      #include "plplot.h"
- * 
+ *
  * This file does all the necessary setup to make PLplot accessible to
  * your program as documented in the manual.  Additionally, this file
  * allows you to request certain behavior by defining certain symbols
@@ -49,7 +49,7 @@
  * will conform to this rule in order to keep namespace pollution of the
  * user code to a minimum.  All the PLplot source files actually include
  * "plplotP.h", which includes this file as well as all the internally-
- * visible declarations, etc.  
+ * visible declarations, etc.
 \*--------------------------------------------------------------------------*/
 
 /* The majority of PLplot source files require these, so.. */
@@ -106,16 +106,19 @@
  * Only those that are necessary for function prototypes are defined here.
  * Notes:
  *
- * PLINT is typedef'd to an int by default.  This is a change from some
- * previous versions, where a long was used.  Under MSDOS, a PLINT is
- * typedef'd to a long, since 16 bits is too inaccurate for some PLplot
- * functions.  So under MSDOS you must use type PLINT for integer array
- * arguments to PLplot functions, but on other systems you can just use
- * an int.
+ * PLINT is typedef'd to a long by default.  This is a change from some
+ * previous versions, where a int was used.  However, so long as you have
+ * used type PLINT for integer array arguments as specified by the API,
+ * this change will be transparent for you.
  *
  * short is currently used for device page coordinates, so they are
  * bounded by (-32767, 32767).  This gives a max resolution of about 3000
  * dpi, and improves performance in some areas over using a PLINT.
+ *
+ * PLUNICODE should be a 32-bit unsigned integer on all platforms.
+ * For now, we are using unsigned int for our Linux ix86 unicode experiments,
+ * but that doesn't guarantee 32 bits exactly on all platforms so this will
+ * be subject to change.
 \*--------------------------------------------------------------------------*/
 
 #if defined(PL_DOUBLE) || defined(DOUBLE)
@@ -128,11 +131,7 @@ typedef float PLFLT;
 #define PLFLT_MIN  FLT_MIN
 #endif
 
-#if defined(MSDOS) && !defined(__WIN32__)
 typedef long PLINT;
-#else
-typedef int PLINT;
-#endif
 
 #if defined(WIN32)
 #if defined(__PLSTUBS_H__)
@@ -146,6 +145,15 @@ typedef int PLINT;
 #endif
 
 
+/* Subject to change. */
+#if defined(HAVE_STDINT_H)
+#include <stdint.h>
+/* This is apparently portable if stdint.h exists. */
+typedef uint32_t PLUNICODE;
+#else
+typedef unsigned int PLUNICODE;
+#endif
+
 
 /* For passing user data, as with X's XtPointer */
 
@@ -229,9 +237,33 @@ typedef void* PLPointer;
 #define PL_PARSE_NODASH         0x0040  /* Set if leading dash NOT required */
 #define PL_PARSE_SKIP           0x0080  /* Skip over unrecognized args */
 
+/* FCI (font characterization integer) related constants. */
+#define PL_FCI_MARK 0x80000000
+#define PL_FCI_IMPOSSIBLE 0x00000000
+#define PL_FCI_HEXDIGIT_MASK 0xf
+#define PL_FCI_HEXPOWER_MASK 0x7
+#define PL_FCI_HEXPOWER_IMPOSSIBLE 0xf
+/* These define hexpower values corresponding to each font attribute. */
+#define PL_FCI_FAMILY  0x0
+#define PL_FCI_STYLE  0x1
+#define PL_FCI_WEIGHT  0x2
+/* These are legal values for font family attribute */
+#define PL_FCI_SANS 0x0
+#define PL_FCI_SERIF 0x1
+#define PL_FCI_MONO 0x2
+#define PL_FCI_SCRIPT 0x3
+#define PL_FCI_SYMBOL 0x4
+/* These are legal values for font style attribute */
+#define PL_FCI_UPRIGHT 0x0
+#define PL_FCI_ITALIC 0x1
+#define PL_FCI_OBLIQUE 0x2
+/* These are legal values for font weight attribute */
+#define PL_FCI_MEDIUM 0x0
+#define PL_FCI_BOLD 0x1
+
 /* Obsolete names */
 
-#define plParseInternalOpts(a, b, c)    plParseOpts(a, b, c)
+#define plParseInternalOpts(a, b, c)    plparseopts(a, b, c)
 #define plSetInternalOpt(a, b)          plSetOpt(a, b)
 
 /* Option table definition */
@@ -464,6 +496,7 @@ typedef struct {
 #define    plgdiori     c_plgdiori
 #define    plgdiplt     c_plgdiplt
 #define    plgfam       c_plgfam
+#define    plgfci       c_plgfci
 #define    plgfnam      c_plgfnam
 #define    plglevel     c_plglevel
 #define    plgpage      c_plgpage
@@ -479,6 +512,7 @@ typedef struct {
 #define    plgzax       c_plgzax
 #define    plhist       c_plhist
 #define    plhls        c_plhls
+#define    plhlsrgb     c_plhlsrgb
 #define    plinit       c_plinit
 #define    pljoin       c_pljoin
 #define    pllab        c_pllab
@@ -492,6 +526,8 @@ typedef struct {
 #define    plmtex       c_plmtex
 #define    plot3d       c_plot3d
 #define    plot3dc	c_plot3dc
+#define    plot3dcl     c_plot3dcl
+#define    plparseopts  c_plparseopts
 #define    plpat        c_plpat
 #define    plpoin       c_plpoin
 #define    plpoin3      c_plpoin3
@@ -502,6 +538,7 @@ typedef struct {
 #define    plreplot     c_plreplot
 #define    plrgb        c_plrgb
 #define    plrgb1       c_plrgb1
+#define    plrgbhls     c_plrgbhls
 #define    plschr       c_plschr
 #define    plscmap0     c_plscmap0
 #define    plscmap0n    c_plscmap0n
@@ -521,6 +558,7 @@ typedef struct {
 #define    plsesc       c_plsesc
 #define    plsetopt	c_plsetopt
 #define    plsfam       c_plsfam
+#define    plsfci       c_plsfci
 #define    plsfnam      c_plsfnam
 #define    plshade      c_plshade
 #define    plshade1     c_plshade1
@@ -541,6 +579,8 @@ typedef struct {
 #define    plstripd	c_plstripd
 #define    plstyl       c_plstyl
 #define    plsurf3d	c_plsurf3d
+#define    plsurf3dl    c_plsurf3dl
+#define    plsvect      c_plsvect
 #define    plsvpa       c_plsvpa
 #define    plsxax       c_plsxax
 #define    plsyax       c_plsyax
@@ -548,6 +588,7 @@ typedef struct {
 #define    plszax       c_plszax
 #define    pltext       c_pltext
 #define    plvasp       c_plvasp
+#define    plvect       c_plvect
 #define    plvpas       c_plvpas
 #define    plvpor       c_plvpor
 #define    plvsta       c_plvsta
@@ -596,6 +637,7 @@ typedef struct {
 #define    c_plgdiori       plgdiori
 #define    c_plgdiplt       plgdiplt
 #define    c_plgfam         plgfam
+#define    c_plgfci         plgfci
 #define    c_plgfnam        plgfnam
 #define    c_plglevel       plglevel
 #define    c_plgpage        plgpage
@@ -611,6 +653,7 @@ typedef struct {
 #define    c_plgzax         plgzax
 #define    c_plhist         plhist
 #define    c_plhls          plhls
+#define    c_plhlsrgb       plhlsrgb
 #define    c_plinit         plinit
 #define    c_pljoin         pljoin
 #define    c_pllab          pllab
@@ -624,6 +667,7 @@ typedef struct {
 #define    c_plmtex         plmtex
 #define    c_plot3d         plot3d
 #define    c_plot3dc        plot3dc
+#define    c_plot3dcl       plot3dcl
 #define    c_plpat          plpat
 #define    c_plpoin         plpoin
 #define    c_plpoin3        plpoin3
@@ -634,6 +678,7 @@ typedef struct {
 #define    c_plreplot       plreplot
 #define    c_plrgb          plrgb
 #define    c_plrgb1         plrgb1
+#define    c_plrgbhls       plrgbhls
 #define    c_plschr         plschr
 #define    c_plscmap0       plscmap0
 #define    c_plscmap0n      plscmap0n
@@ -653,6 +698,7 @@ typedef struct {
 #define    c_plsesc         plsesc
 #define    c_plsetopt       plsetopt
 #define    c_plsfam         plsfam
+#define    c_plsfci         plsfci
 #define    c_plsfnam        plsfnam
 #define    c_plshade        plshade
 #define    c_plshade1       plshade1
@@ -673,6 +719,8 @@ typedef struct {
 #define    c_plstripd       plstripd
 #define    c_plstyl         plstyl
 #define    c_plsurf3d       plsurf3d
+#define    c_plsurf3dl      plsurf3dl
+#define    c_plsvect        plsvect
 #define    c_plsvpa         plsvpa
 #define    c_plsxax         plsxax
 #define    c_plsyax         plsyax
@@ -680,6 +728,7 @@ typedef struct {
 #define    c_plszax         plszax
 #define    c_pltext         pltext
 #define    c_plvasp         plvasp
+#define    c_plvect         plvect
 #define    c_plvpas         plvpas
 #define    c_plvpor         plvpor
 #define    c_plvsta         plvsta
@@ -688,8 +737,10 @@ typedef struct {
 #define    c_plwind         plwind
 #define    c_plxormod       plxormod
 
+#define    plAlloc2dGrid    win_plAlloc2dGrid
+#define    plFree2dGrid     win_plFree2dGrid
 #define    plGetCursor      win_plGetCursor
-#define    plParseOpts      win_plParseOpts
+#define    c_plparseopts    win_plParseOpts
 #define    plMergeOpts      win_plMergeOpts
 
 #else
@@ -714,6 +765,8 @@ typedef struct {
 #define    plP_gvpd	plgvpd
 #define    plP_gvpw	plgvpw
 #define    plotsh3d(x,y,z,nx,ny,opt)     plsurf3d(x,y,z,nx,ny,opt, NULL, 0)
+#define    plHLS_RGB    plhlsrgb
+#define    plRGB_HLS    plrgbhls
 
 #endif /* __PLSTUBS_H__ */
 
@@ -750,6 +803,14 @@ void API
 plarrows(PLFLT *u, PLFLT *v, PLFLT *x, PLFLT *y, PLINT n,
          PLFLT scale, PLFLT dx, PLFLT dy) ;
 
+void API
+c_plvect(PLFLT **u, PLFLT **v, PLINT nx, PLINT ny, PLFLT scale,
+void (*pltr) (PLFLT, PLFLT, PLFLT *, PLFLT *, PLPointer),
+              PLPointer pltr_data);
+
+void API
+c_plsvect(PLFLT *arrowx, PLFLT *arrowy, PLINT npts, PLINT fill);
+
 /* This functions similarly to plbox() except that the origin of the axes */
 /* is placed at the user-specified point (x0, y0). */
 
@@ -801,7 +862,7 @@ void API
 c_plcol1(PLFLT col1);
 
 /* Draws a contour plot from data in f(nx,ny).  Is just a front-end to
- * plfcont, with a particular choice for f2eval and f2eval_data. 
+ * plfcont, with a particular choice for f2eval and f2eval_data.
  */
 
 void API
@@ -812,7 +873,7 @@ c_plcont(PLFLT **f, PLINT nx, PLINT ny, PLINT kx, PLINT lx,
 
 /* Draws a contour plot using the function evaluator f2eval and data stored
  * by way of the f2eval_data pointer.  This allows arbitrary organizations
- * of 2d array data to be used. 
+ * of 2d array data to be used.
  */
 
 void API
@@ -949,6 +1010,11 @@ c_plgdiori(PLFLT *p_rot);
 void API
 c_plgdiplt(PLFLT *p_xmin, PLFLT *p_ymin, PLFLT *p_xmax, PLFLT *p_ymax);
 
+/* Get FCI (font characterization integer) */
+
+void API
+c_plgfci(PLUNICODE *pfci);
+
 /* Get family file parameters */
 
 void API
@@ -1042,6 +1108,11 @@ c_plhist(PLINT n, PLFLT *data, PLFLT datmin, PLFLT datmax,
 void API
 c_plhls(PLFLT h, PLFLT l, PLFLT s);
 
+/* Functions for converting between HLS and RGB color space */
+
+void API
+c_plhlsrgb(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b);
+
 /* Initializes PLplot, using preset or default options */
 
 void API
@@ -1085,7 +1156,7 @@ plmap(void (*mapform)(PLINT, PLFLT *, PLFLT *), char *type,
 /* Plot the latitudes and longitudes on the background. */
 
 void API
-plmeridians(void (*mapform)(PLINT, PLFLT *, PLFLT *), 
+plmeridians(void (*mapform)(PLINT, PLFLT *, PLFLT *),
 	    PLFLT dlong, PLFLT dlat,
 	    PLFLT minlong, PLFLT maxlong, PLFLT minlat, PLFLT maxlat);
 
@@ -1124,6 +1195,15 @@ c_plot3dc(PLFLT *x, PLFLT *y, PLFLT **z,
 	 PLINT nx, PLINT ny, PLINT opt,
 	 PLFLT *clevel, PLINT nlevel);
 
+/* Plots a 3-d representation of the function z[x][y] with contour and
+ * y index limits. */
+
+void
+c_plot3dcl(PLFLT *x, PLFLT *y, PLFLT **z,
+         PLINT nx, PLINT ny, PLINT opt,
+         PLFLT *clevel, PLINT nlevel,
+         PLINT ixstart, PLINT ixn, PLINT *indexymin, PLINT*indexymax);
+
 /*
  * definitions for the opt argument in plot3dc() and plsurf3d()
  *
@@ -1202,6 +1282,11 @@ c_plrgb(PLFLT r, PLFLT g, PLFLT b);
 void API
 c_plrgb1(PLINT r, PLINT g, PLINT b);
 
+/* Functions for converting between HLS and RGB color space */
+
+void API
+c_plrgbhls(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);
+
 /* Set character height. */
 
 void API
@@ -1295,6 +1380,10 @@ c_plsesc(char esc);
 
 void API
 c_plsfam(PLINT fam, PLINT num, PLINT bmax);
+/* Set FCI (font characterization integer) */
+
+void API
+c_plsfci(PLUNICODE fci);
 
 /* Set the output file name. */
 
@@ -1339,7 +1428,7 @@ plfshade(PLFLT (*f2eval) (PLINT, PLINT, PLPointer),
 	 PLPointer f2eval_data,
 	 PLFLT (*c2eval) (PLINT, PLINT, PLPointer),
 	 PLPointer c2eval_data,
-	 PLINT nx, PLINT ny, 
+	 PLINT nx, PLINT ny,
 	 PLFLT left, PLFLT right, PLFLT bottom, PLFLT top,
 	 PLFLT shade_min, PLFLT shade_max,
 	 PLINT sh_cmap, PLFLT sh_color, PLINT sh_width,
@@ -1444,6 +1533,14 @@ void API
 c_plsurf3d(PLFLT *x, PLFLT *y, PLFLT **z, PLINT nx, PLINT ny,
 	   PLINT opt, PLFLT *clevel, PLINT nlevel);
 
+/* Plots the 3d surface representation of the function z[x][y] with y
+ * index limits. */
+
+void
+c_plsurf3dl(PLFLT *x, PLFLT *y, PLFLT **z, PLINT nx, PLINT ny,
+           PLINT opt, PLFLT *clevel, PLINT nlevel,
+           PLINT ixstart, PLINT ixn, PLINT *indexymin, PLINT*indexymax);
+
 /* Sets the edges of the viewport to the specified absolute coordinates */
 
 void API
@@ -1589,7 +1686,8 @@ pltr1(PLFLT x, PLFLT y, PLFLT *tx, PLFLT *ty, PLPointer pltr_data);
 /* Does linear interpolation from doubly dimensioned coord arrays */
 /* (column dominant, as per normal C 2d arrays). */
 
-void API
+/* void API */
+void
 pltr2(PLFLT x, PLFLT y, PLFLT *tx, PLFLT *ty, PLPointer pltr_data);
 
 /* Just like pltr2() but uses pointer arithmetic to get coordinates from */
@@ -1669,7 +1767,7 @@ plSetOpt(char *opt, char *optarg);
 /* Process options list using current options info. */
 
 int API
-plParseOpts(int *p_argc, char **argv, PLINT mode);
+c_plparseopts(int *p_argc, char **argv, PLINT mode);
 
 /* Print usage & syntax message. */
 
@@ -1741,14 +1839,6 @@ plFree2dGrid(PLFLT **f, PLINT nx, PLINT ny);
 void API
 plMinMax2dGrid(PLFLT **f, PLINT nx, PLINT ny, PLFLT *fmax, PLFLT *fmin);
 
-/* Functions for converting between HLS and RGB color space */
-
-void API
-plHLS_RGB(PLFLT h, PLFLT l, PLFLT s, PLFLT *p_r, PLFLT *p_g, PLFLT *p_b);
-
-void API
-plRGB_HLS(PLFLT r, PLFLT g, PLFLT b, PLFLT *p_h, PLFLT *p_l, PLFLT *p_s);
-
 /* Wait for graphics input event and translate to world coordinates */
 
 int API
@@ -1798,6 +1888,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plgdiori
 #undef    c_plgdiplt
 #undef    c_plgfam
+#undef    c_plgfci
 #undef    c_plgfnam
 #undef    c_plglevel
 #undef    c_plgpage
@@ -1813,6 +1904,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plgzax
 #undef    c_plhist
 #undef    c_plhls
+#undef    c_plhlsrgb
 #undef    c_plinit
 #undef    c_pljoin
 #undef    c_pllab
@@ -1826,6 +1918,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plmtex
 #undef    c_plot3d
 #undef    c_plot3dc
+#undef    c_plot3dcl
 #undef    c_plpat
 #undef    c_plpoin
 #undef    c_plpoin3
@@ -1836,6 +1929,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plreplot
 #undef    c_plrgb
 #undef    c_plrgb1
+#undef    c_plrgbhls
 #undef    c_plschr
 #undef    c_plscmap0
 #undef    c_plscmap0n
@@ -1855,6 +1949,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plsesc
 #undef    c_plsetopt
 #undef    c_plsfam
+#undef    c_plsfci
 #undef    c_plsfnam
 #undef    c_plshade
 #undef    c_plshade1
@@ -1875,6 +1970,8 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plstripd
 #undef    c_plstyl
 #undef    c_plsurf3d
+#undef    c_plsurf3dl
+#undef    c_plsvect
 #undef    c_plsvpa
 #undef    c_plsxax
 #undef    c_plsyax
@@ -1882,6 +1979,7 @@ plTranslateCursor(PLGraphicsIn *gin);
 #undef    c_plszax
 #undef    c_pltext
 #undef    c_plvasp
+#undef    c_plvect
 #undef    c_plvpas
 #undef    c_plvpor
 #undef    c_plvsta
